Attribute VB_Name = "APIs"
Option Explicit

'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: APIS.BAS ##
'
' OBJETIVO: Establecer el marco de trabajo para poder manipular las APIs de Windows, o Interfaz
'           de Programacin de la Aplicacin como conjunto de funciones que Windows expone a los
'           programadores.
'-------------------------------------------------------------------------------------------------------

'--------------------------------------------------------------------------------------------------
' - DECLARACIN DE TIPOS Y CONSTANTES NECESARIOS PARA LA LLAMADA A LAS FUNCIONES APIs DE WINDOWS -
'
' OBJETIVO: Declarar los tipos y constantes necesarios a la hora de enumerar los parmetros de las
'           funciones APIs.
'--------------------------------------------------------------------------------------------------

'MBITO GLOBAL

'MBITO LOCAL AL MDULO
Private Type STARTUPINFO
        cb As Long
        lpReserved As String
        lpDesktop As String
        lpTitle As String
        dwX As Long
        dwY As Long
        dwXSize As Long
        dwYSize As Long
        dwXCountChars As Long
        dwYCountChars As Long
        dwFillAttribute As Long
        dwFlags As Long
        wShowWindow As Integer
        cbReserved2 As Integer
        lpReserved2 As Long
        hStdInput As Long
        hStdOutput As Long
        hStdError As Long
End Type

Private Type SECURITY_ATTRIBUTES
        nLength As Long
        lpSecurityDescriptor As Long
        bInheritHandle As Long
End Type

Private Type PROCESS_INFORMATION
        hProcess As Long
        hThread As Long
        dwProcessId As Long
        dwThreadId As Long
End Type

Private Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
End Type

Private Type POINTAPI
        x As Long
        y As Long
End Type


'---------------------------------
' -- DECLARACIN DE CONSTANTES --
'---------------------------------
'MBITO GLOBAL
Public Const RGN_AND = 1
Public Const RGN_OR = 2
Public Const RGN_XOR = 3
Public Const RGN_DIFF = 4
Public Const RGN_COPY = 5
Public Const RGN_MAX = RGN_COPY
Public Const RGN_MIN = RGN_AND

Public Const ALTERNATE = 1
Public Const WINDING = 2

'MBITO LOCAL AL MDULO
Private Const NORMAL_PRIORITY_CLASS = &H20&
Private Const INFINITE = -1&

Private Const SND_ASYNC = &H1     ' Reproduce de forma asncrona
Private Const SND_NODEFAULT = &H2 ' No usar el sonido predeterminado
Private Const SND_MEMORY = &H4    ' lpszSoundName apunta al archivo de memoria

Private Const SND_ALIAS = &H10000
Private Const SND_SYNC = &H0
Private Const SND_OWAIT = &H2000
Private Const SND_LOOP = &H8

Private Const SW_RESTORE = 9

' Opciones de seguridad de clave del Registro...
Private Const READ_CONTROL = &H20000
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_ALL_ACCESS = KEY_QUERY_VALUE + KEY_SET_VALUE + _
                               KEY_CREATE_SUB_KEY + KEY_ENUMERATE_SUB_KEYS + _
                               KEY_NOTIFY + KEY_CREATE_LINK + READ_CONTROL
                     
' Tipos ROOT de clave del Registro...
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const ERROR_SUCCESS = 0
Private Const REG_SZ = 1                         ' Cadena Unicode terminada en valor nulo
Private Const REG_DWORD = 4                      ' Nmero de 32 bits

Private Const gREGKEYSYSINFOLOC = "SOFTWARE\Microsoft\Shared Tools Location"
Private Const gREGVALSYSINFOLOC = "MSINFO"
Private Const gREGKEYSYSINFO = "SOFTWARE\Microsoft\Shared Tools\MSINFO"
Private Const gREGVALSYSINFO = "PATH"

Private Const HWND_TOPMOST = -1
Private Const HWND_NOTOPMOST = -2


'--------------------------------------------------------------------------------------------------
'                            DECLARACIN DE FUNCIONES DE LA API WINDOWS
'
' OBJETIVO: Para poder usar una F.Api en primer lugar hay que declararla, es decir, se establece
'           un vnculo de nuestro programa con la DLL Externa (librera) en la que se encuentra
'           almacenada la funcin.
'--------------------------------------------------------------------------------------------------


'**************************************************************************************************
' DECLARACIN DE F.APIs: Function GetPrivateProfileString, Function WritePrivateProfileString
'
' OBJETIVO: UTILIZAR ARCHIVOS.INI (No tiene pq ser extensin INI puede ser cualquier extensin
'                                  que cumpla con el formato de datos de abajo, de forma que
'                                  existan la Secciones [], Claves y Valores asociados a cada
'                                  una de ellas).
'
' NOTA ACLARATORIA: ESTRUCTURACIN DE LA INFORMACIN CONTENIDA EN UN ARCHIVO.INI
'
' [SECCION 1]
' ;  AL EMPEZAR POR PUNTO Y COMA ESTA LNEA SERA COMENTARIO !!
' CLAVE 1 = VALOR 1
'   "        "
'   "        "
'
' [SECCION N]
' CLAVE N = VALOR N
'   "        "
'   "        "
'
'**************************************************************************************************
Private Declare Function GetPrivateProfileString Lib "kernel32" Alias _
        "GetPrivateProfileStringA" (ByVal lpApplicationName _
        As String, ByVal lpKeyName As Any, ByVal lpDefault _
        As String, ByVal lpReturnedString As String, ByVal _
        nSize As Long, ByVal lpFileName As String) As Long
        
Private Declare Function WritePrivateProfileString Lib "kernel32" Alias _
        "WritePrivateProfileStringA" (ByVal lpApplicationName _
        As String, ByVal lpKeyName As Any, ByVal lpString As Any, _
        ByVal lpFileName As String) As Long

'***************************************************************************************************
' DECLARACIN DE F.APIs: Function WaitForSingleObject,Function CreateProcessA,Function CloseHandle
'
' OBJETIVO: MTODO DE MICROSOFT PARA ESPERAR A QUE UN PROGRAMA SE TERMINE DE EJECUTAR AL SER
'           LLAMADO MEDIANTE EL SHELL
'
'***************************************************************************************************
 Private Declare Function WaitForSingleObject Lib "kernel32" (ByVal hHandle As Long, ByVal _
                                                            dwMilliseconds As Long) As Long
                                                            
 Private Declare Function CreateProcessA Lib "kernel32" (ByVal lpApplicationName As String, _
                                                ByVal lpCommandLine As String, lpProcessAttributes As Long, _
                                                lpThreadAttributes As Long, ByVal bInheritHandles As Long, _
                                                ByVal dwCreationFlags As Long, lpEnvironment As Any, ByVal lpCurrentDriectory _
                                                As String, lpStartupInfo As STARTUPINFO, lpProcessInformation As PROCESS_INFORMATION) As Long
                           
 Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

'***************************************************************************************************
' DECLARACIN DE F.API: Function sndPlaySound
'
' OBJETIVO: REPRODUCTOR DE SONIDO ALTO NIVEL
'
'***************************************************************************************************
Private Declare Function sndPlaySound Lib "WINMM.DLL" Alias "sndPlaySoundA" (lpszSoundName As Any, ByVal uFlags As Long) As Long

'***************************************************************************************************
' DECLARACIN F.APIs: Function SetWindowPos
'
' OBJETIVO: ESTABLECER POSICION DE FORMULARIO SEGUN COORDENADAS
'
'***************************************************************************************************
Public Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, _
                                                   ByVal x As Long, ByVal y As Long, ByVal cx As Long, _
                                                   ByVal cy As Long, ByVal wFlags As Long) As Long
                                                   
'***************************************************************************************************
' DECLARACION DE F.APIs: Function FindWindow, Function ShowWindow, Function SetForegroundWindow
'
' OBJETIVO: IMPEDIR EL ARRANQUE DE COPIAS EXTRAS DE LA APLICACIN
'
'***************************************************************************************************
Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" _
                         (ByVal lpClassName As String, ByVal lpWindowName As String) As Long
Private Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, _
                         ByVal nCmdShow As Long) As Long
Private Declare Function SetForegroundWindow Lib "user32" (ByVal hWnd As Long) As Long

'**************************************************************************************************
' DECLARACIN DE F.APIs: Function RegOpenKeyEx, RegQueryValueExm, RegCloseKey
'
' OBJETIVO: PERMITIR OBTENER LA INFORMACIN DEL SISTEMA ATRAVS
'           DE LA LLAMADA A LA APLICACIN MSINFO32
'
'**************************************************************************************************
Private Declare Function RegOpenKeyEx Lib "advapi32" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, _
                                                                            ByVal ulOptions As Long, ByVal samDesired As Long, _
                                                                            ByRef phkResult As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, _
                                                                                  ByVal lpReserved As Long, ByRef lpType As Long, _
                                                                                  ByVal lpData As String, ByRef lpcbData As Long) As Long
Private Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long

'**************************************************************************************************
' DECLARACIN DE F.APIs: CreateEllipticRgn, CreateEllipticRgnIndirect, CreatePolygonRgn,
'                        CreatePolyPolygonRgn, CreateRectRgn, CreateRectRgnIndirect, CreateRoundRectRgn,
'                        EqualRgn, FillRgn, PaintRgn, InvertRgn, FrameRgn, OffsetRgn, PtInRegion,
'                        CombineRgn, SetWindowRgn
'
' OBJETIVO: VA A PERMITIR DESARROLLAR VENTANAS MULTIFORMA EN LUGAR DE LA TPICA RECTNGULAR Y PARA
'           ELLO SE HACE USO DEL CONCEPTO DE REGIN QUE NOS PERMITIRN DETERMINAR LA ZONA DE
'           CLIPPING (RECORTAR) DE UNA VENTANA PARA QUE FUERA DE LOS LMITES MARCADOS NO SE DIBUJE
'           UN SLO PIXEL.
'
'           REGIN: ZONA O PORCIN OBTENIDA ATRAVS DE COMBINACIN DE DISTINTAS FORMAS
'                   DE UNA VENTANTA. CADA VENTANA TIENE UNA REGIN VISIBLE QUE ESTABLECE LO QUE PUEDE
'                   VER EL USUARIO EN PANTALLA.
'                   WINDOWS CAMBIA LA REGIN VISIBLE DE UNA VENTANA CADA VEZ QUE STA CAMBIA DE
'                   TAMAO O SE OCULTA POR OTRA VENTANA QUE SE HA SUPERPUESTO. HAY QUE TENER EN
'                   CUENTA QUE UNA APLICACIN NO PUEDE CAMBIAR LA REGIN VISIBLE DIRECTAMENTE, AUNQUE
'                   PODEMOS ESTABLECER REGIONES PARA QUE WINDOWS SEPA DNDE TIENE QUE "CORTAR" LA VENTANA
'
'**************************************************************************************************

'--------------- OPERACIONES CON REGIONES ---------------
'Permiten crear distintas formas de regiones rectngulo, elipse, polgono...
'Crea una regin elptica o circular mediante 4 coordenadas
Public Declare Function CreateEllipticRgn Lib "gdi32" (ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long) As Long
'Crea una regin elptica o circular mediante una RECT
Public Declare Function CreateEllipticRgnIndirect Lib "gdi32" (lpRect As RECT) As Long
'Crea una regin poligonal
Public Declare Function CreatePolygonRgn Lib "gdi32" (lpPoint As POINTAPI, ByVal nCount As Long, ByVal nPolyFillMode As Long) As Long
'Crea una regin consistente en una serie de polgonos
Public Declare Function CreatePolyPolygonRgn Lib "gdi32" (lpPoint As POINTAPI, lpPolyCounts As Long, ByVal nCount As Long, ByVal nPolyFillMode As Long) As Long
'Crea una regin rectngular
Public Declare Function CreateRectRgn Lib "gdi32" (ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long) As Long
'Crea una regin rectngular con una estructura RECT
Public Declare Function CreateRectRgnIndirect Lib "gdi32" (lpRect As RECT) As Long
'Crea una regin rectngular con los bordes redondeados
Public Declare Function CreateRoundRectRgn Lib "gdi32" (ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long, ByVal x3 As Long, ByVal y3 As Long) As Long
'Comprueba si dos regiones son iguales, es decir, si son idnticas en tamao y forma
Public Declare Function EqualRgn Lib "gdi32" (ByVal hSrcRgn1 As Long, ByVal hSrcRgn2 As Long) As Long
'Permite rellenar una regin
Public Declare Function FillRgn Lib "gdi32" (ByVal hdc As Long, ByVal hRgn As Long, ByVal hBrush As Long) As Long
'Permite pintar regiones, utiliza para ello modos de relleno de polgonos ALTERNATE y WINDING
Public Declare Function PaintRgn Lib "gdi32" (ByVal hdc As Long, ByVal hRgn As Long) As Long
'Permite invertir los colores dentro de la regin
Public Declare Function InvertRgn Lib "gdi32" (ByVal hdc As Long, ByVal hRgn As Long) As Long
'Permite crear bordes sobre la regin aplicada a la ventana
Public Declare Function FrameRgn Lib "gdi32" (ByVal hdc As Long, ByVal hRgn As Long, ByVal hBrush As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
'Permite mover regiones
Public Declare Function OffsetRgn Lib "gdi32" (ByVal hRgn As Long, ByVal x As Long, ByVal y As Long) As Long
'Permite saber las coordenadas de la posicin actual del cursor dentro de la regin aplicada a la ventana
Public Declare Function PtInRegion Lib "gdi32" (ByVal hRgn As Long, ByVal x As Long, ByVal y As Long) As Long
'Permite combinar varias regiones en una (dos a dos)
Public Declare Function CombineRgn Lib "gdi32" (ByVal hDestRgn As Long, ByVal hSrcRgn1 As Long, ByVal hSrcRgn2 As Long, ByVal nCombineMode As Long) As Long
'Establece la regin en la ventana correspondiente
Public Declare Function SetWindowRgn Lib "user32" (ByVal hWnd As Long, ByVal hRgn As Long, ByVal bRedraw As Boolean) As Long


'--------------------------------------------------------------------------------------------------
'                                        FUNCIONES DE AJUSTE
'
' OBJETIVO: Establecen un paso intermedio entre las funciones APIs de Windows y las
'           necesidades del programador.
'
'--------------------------------------------------------------------------------------------------

'**************************************************************************************************
' FUNCIN DE AJUSTE:
'
' OBJETIVO: LECTURA DE ARCHIVOS.INI (No por su extensin sino por la forma en que se almacena
'                                    la informacin en este tipo de ficheros).
'
'  IMPORTANTE !! La cadena Por_defecto ser el valor que devuelva esta
'                  funcin en el caso de que alguno de los parmetros que
'                  se le de no sean correctos, es decir, cuando no exita el
'                  fichero o seccin o clave especificados.
'                  Sera adecuado poner p.ej Por_defecto = "?" as se
'                  comprobara el error :
'                      If LeerINI(nombfichero,seccion,clave,"?") = "?"
'                      then <Accion de error>
'
'**************************************************************************************************
Public Function LeerINI(Rutaynombre_fichero_ini As String, Seccion As String, _
                        Clave As String, Por_defecto As String) As String
    Dim Temp_cadena As String * 256
    Dim Longitud As Integer
    'Llenado completo de la cadena temporal de espacios
    Temp_cadena = Space$(256)
    'Llamada a la API WINDOWS para lectura de datos en el archivo.ini
    Longitud = GetPrivateProfileString(Seccion, Clave, Por_defecto, Temp_cadena, _
                                       255, Rutaynombre_fichero_ini)
    'Se devuelve todos los caracteres empezando por la izquierda de la cadena temporal (max Longitud)
    LeerINI = Left$(Temp_cadena, Longitud)
                
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: LeerNumeroINI()
'**************************************************************************************************
Public Sub LeerNumeroINI(Ruta As String, Seccion As String, Clave As String, ValorDefecto As Integer, VariableAlmacenar As Integer)
  Dim Cadena As String
  
  Cadena = LeerINI(Ruta, Seccion, Clave, "?")
  If Cadena = "?" Or Cadena = "" Or Not IsNumeric(Cadena) Then
    VariableAlmacenar = ValorDefecto
  ElseIf CInt(Cadena) <= 0 Then
    VariableAlmacenar = ValorDefecto
  Else
    VariableAlmacenar = CInt(Cadena)
  End If
  
End Sub

'**************************************************************************************************
' PROCEDIMIENTO DE AJUSTE: EscribirINI
'
' OBJETIVO: Almacenar informacin en fichero de texto (normalmente *.ini no tiene pq!) bajo una
'           estructura determinada de Seccin [], Clave igual a Valor.
'
'**************************************************************************************************
Public Sub EscribirINI(Rutaynombre_fichero_ini As String, Seccion As String, _
                            Clave As String, Valor As String)
    Dim Temp_cadena As String
    Dim n As Integer
    
    Temp_cadena = Valor
    
    'En el caso de que el valor tuviese Enter introducidos, es decir, distintos valores
    'estos retornos de carro sern reemplazados con espacios
    For n = 1 To Len(Temp_cadena)
        'Caracter a caracter se busca el retorno de carro (vbCr) y nueva linea (vbLf)
        If MId$(Temp_cadena, n, 1) = vbCr Or MId$(Temp_cadena, n, 1) = vbLf _
        Then Mid$(Temp_cadena, n) = " " 'Si se encuentra se sustituye por espacio
    Next n
    'Llamada a la API WINDOWS para escribir los datos en el archivo.ini
    n = WritePrivateProfileString(Seccion, Clave, Temp_cadena, Rutaynombre_fichero_ini)
                  
End Sub

'***************************************************************************************************
' PROCEDIMIENTO DE AJUSTE: EsperarTerminarProgramaExternoShell
'
' OBJETIVO: MTODO DE MICROSOFT PARA ESPERAR A QUE UN PROGRAMA SE
'           TERMINE DE EJECUTAR AL SER LLAMADO MEDIANTE EL SHELL
'
'***************************************************************************************************
Public Sub EsperarTerminarProgramaExternoShell(Cmdline As String)
    Dim proc As PROCESS_INFORMATION
    Dim start As STARTUPINFO
    Dim Ret As Long
    
    'Inicializar la estructura de STARTUPINFO:
    start.cb = Len(start)
    'Inicializar la aplicacin Shell:
    Ret = CreateProcessA(0&, Cmdline, 0&, 0&, 1&, NORMAL_PRIORITY_CLASS, 0&, 0&, start, proc)
    'Esperar a que la aplicacin Shell termine:
    Ret = WaitForSingleObject(proc.hProcess, INFINITE)
    Ret = CloseHandle(proc.hProcess)

End Sub

'***************************************************************************************************
' PROCEDIMIENTOS DE AJUSTE: PlaySonido, StopSonido
'
' OBJETIVO: REPRODUCIR ARCHIVOS DE SONIDO *.WAV ALMACENADOS EN EL ARCHIVO DE RECURSOS *.RES
'
'***************************************************************************************************
Public Sub PlaySonido(ByVal IdRecurso As Integer, SeccionRecurso As String)
   On Error Resume Next
   
   Dim Puntero() As Byte
   Puntero = LoadResData(IdRecurso, SeccionRecurso)
   'En el caso de que no se encuentre el recurso con el identificador (IdRecurso) especificado se anular la reproduccin
   If Err.Number = 326 Then
     Err.Clear
     Exit Sub
   End If
   sndPlaySound Puntero(0), SND_ASYNC Or SND_NODEFAULT Or SND_MEMORY
   
End Sub

Public Sub StopSonido()
    sndPlaySound ByVal vbNullString, 0&
End Sub

'***************************************************************************************************
' PROCEDIMIENTO DE AJUSTE : PlaySonidoSolo
'
' OBJETIVO: Reproducir sonido de un archivo *.wav de forma que si este archivo existe se
'           reproducir paralizandose el sistema mientras su reproduccin
'
'***************************************************************************************************
Public Sub PlaySonidoSolo(FicheroSonidoWAV As String)
  Dim lRetVal As Long
  
  lRetVal = sndPlaySound(FicheroSonidoWAV, SND_SYNC)
  lRetVal = sndPlaySound("SystemStart", SND_ALIAS + SND_ASYNC + SND_OWAIT)
  
End Sub

'***************************************************************************************************
' FUNCIN DE AJUSTE: ImpedirArranqueCopiasAplicacion
'
' OBJETIVO: IMPEDIR EL ARRANQUE DE COPIAS EXTRAS DE LA APLICACIN
'
'***************************************************************************************************
Public Sub ImpedirArranqueCopiasAplicacion(NombreFormulario As String)

    Dim sTitulo As String
    Dim hWnd As Long
    Dim lValoretorno As Long
    'Comprueba la propiedad PrevInstance
    'del objeto App (Determina o especifica informacin acerca del ttulo de la aplicacin, informacin de la versin,
    'la ruta de acceso y el nombre de su archivo ejecutable y los archivos de Ayuda, y si est ejecutndose o no una instancia
    'anterior de la aplicacin.
    If App.PrevInstance Then
        MsgBox "No se puede arrancar copias extras de la aplicacin dado que podra ocasionar conflictos de memoria!", vbExclamation + vbOKOnly, "Atencin!"
        sTitulo = NombreFormulario
        'Se renombra el ttulo y propiedades de ttulo (caption) de la aplicacin activa de forma que la Api FindWindow
        'no la encuentre cuando busque la anterior que posee el mismo ttulo y caption
        App.Title = "Nueva Copia"
        NombreFormulario = "Nueva Copia"
        'Posteriormente se localiza la ventana perteneciente a la aplicacin anterior, se restaura y se mueve al primer plano
        hWnd = FindWindow(vbNullString, sTitulo)
        If hWnd <> 0 Then
            lValoretorno = ShowWindow(hWnd, SW_RESTORE)
            lValoretorno = SetForegroundWindow(hWnd)
        End If
        End
    End If
    
End Sub

'***************************************************************************************************
' FUNCIN DE AJUSTE: GetKeyValue
'***************************************************************************************************
Private Function GetKeyValue(KeyRoot As Long, KeyName As String, SubKeyRef As String, ByRef KeyVal As String) As Boolean
    Dim I As Long                                           ' Contador de bucle
    Dim rc As Long                                          ' Cdigo de retorno
    Dim hKey As Long                                        ' Controlador de una clave de Registro abierta
    Dim hDepth As Long                                      '
    Dim KeyValType As Long                                  ' Tipo de datos de una clave de Registro
    Dim tmpVal As String                                    ' Almacenamiento temporal para un valor de clave de Registro
    Dim KeyValSize As Long                                  ' Tamao de variable de clave de Registro
    '------------------------------------------------------------
    ' Abrir clave de registro bajo KeyRoot {HKEY_LOCAL_MACHINE...}
    '------------------------------------------------------------
    rc = RegOpenKeyEx(KeyRoot, KeyName, 0, KEY_ALL_ACCESS, hKey) ' Abrir clave de Registro
    
    If (rc <> ERROR_SUCCESS) Then GoTo GetKeyError          ' Error de controlador...
    
    tmpVal = String$(1024, 0)                             ' Asignar espacio de variable
    KeyValSize = 1024                                       ' Marcar tamao de variable
    
    '------------------------------------------------------------
    ' Obtener valor de clave de Registro...
    '------------------------------------------------------------
    rc = RegQueryValueEx(hKey, SubKeyRef, 0, _
                         KeyValType, tmpVal, KeyValSize)    ' Obtener o crear valor de clave
                        
    If (rc <> ERROR_SUCCESS) Then GoTo GetKeyError          ' Controlar errores
    
    If (Asc(MId(tmpVal, KeyValSize, 1)) = 0) Then           ' Win95 agregar cadena terminada en valor nulo...
        tmpVal = Left(tmpVal, KeyValSize - 1)               ' Encontrado valor nulo, se va a quitar de la cadena
    Else                                                    ' En WinNT las cadenas no terminan en valor nulo...
        tmpVal = Left(tmpVal, KeyValSize)                   ' No se ha encontrado valor nulo, slo se va a extraer la cadena
    End If
    '------------------------------------------------------------
    ' Determinar tipo de valor de clave para conversin...
    '------------------------------------------------------------
    Select Case KeyValType                                  ' Buscar tipos de datos...
    Case REG_SZ                                             ' Tipo de datos String de clave de Registro
        KeyVal = tmpVal                                     ' Copiar valor de cadena
    Case REG_DWORD                                          ' Tipo de datos Double Word de clave del Registro
        For I = Len(tmpVal) To 1 Step -1                    ' Convertir cada bit
            KeyVal = KeyVal + Hex(Asc(MId(tmpVal, I, 1)))   ' Generar valor carcter a carcter
        Next
        KeyVal = Format$("&h" + KeyVal)                     ' Convertir Double Word a cadena
    End Select
    
    GetKeyValue = True                                      ' Se ha devuelto correctamente
    rc = RegCloseKey(hKey)                                  ' Cerrar clave de Registro
    Exit Function                                           ' Salir
    
GetKeyError:      ' Borrar despus de que se produzca un error...
    KeyVal = ""                                             ' Establecer valor a cadena vaca
    GetKeyValue = False                                     ' Fallo de retorno
    rc = RegCloseKey(hKey)                                  ' Cerrar clave de Registro
End Function

'***************************************************************************************************
' FUNCIN DE AJUSTE: StartSysInfo
'
' OBJETIVO: Ejecutar programa de Windows que muestra informacin general del sistema.
'
'***************************************************************************************************
Public Sub StartSysInfo()
    On Error GoTo SysInfoErr
  
    Dim rc As Long
    Dim SysInfoPath As String
    
    ' Intentar obtener ruta de acceso y nombre del programa de Info. del sistema a partir del Registro...
    If GetKeyValue(HKEY_LOCAL_MACHINE, gREGKEYSYSINFO, gREGVALSYSINFO, SysInfoPath) Then
    ' Intentar obtener slo ruta del programa de Info. del sistema a partir del Registro...
    ElseIf GetKeyValue(HKEY_LOCAL_MACHINE, gREGKEYSYSINFOLOC, gREGVALSYSINFOLOC, SysInfoPath) Then
        ' Validar la existencia de versin conocida de 32 bits del archivo
        If (Dir(SysInfoPath & "\MSINFO32.EXE") <> "") Then
            SysInfoPath = SysInfoPath & "\MSINFO32.EXE"
            
        ' Error: no se puede encontrar el archivo...
        Else
            GoTo SysInfoErr
        End If
    ' Error: no se puede encontrar la entrada del Registro...
    Else
        GoTo SysInfoErr
    End If
    
    Call Shell(SysInfoPath, vbNormalFocus)
    
    Exit Sub
SysInfoErr:
    MsgBox "La informacin del sistema no est disponible en este momento", vbOKOnly
End Sub

'***************************************************************************************************
' FUNCIN DE AJUSTE: EnPrimerPlano
'
' OBJETIVO: ACTIVAR/DESACTIVAR (Opcion) LA POSIBILIDAD DE ESTABLECER COMO VENTANA EN PRIMER PLANO
'           EL FORMULARIO QUE RECIBE (Formulario) PARA QUE ESTE SIEMPRE EST POR ENCIMA DE CUALQUIER
'           OTRA VENTANA DE LA PANTALLA.
'
'           USO IDEAL PARA LA VENTANA DE INICIALIZACIN DE UNA APLICACIN HACIENDO QUE ESTA SEA
'           VISIBLE MIENTRAS LAS RESTANTES SE VAN CARGANDO.
'
'***************************************************************************************************
Public Sub EnPrimerPlano(Formulario As Form, SiPrimerPlano As Boolean)
   Dim iSuceso As Integer, Opcion As Integer
   Dim Flags As Long
    
   If SiPrimerPlano Then
     Opcion = HWND_TOPMOST
   Else
     Opcion = HWND_NOTOPMOST
   End If
   
  ' Establece que la ventana que recibe permanezca en primer plano, es decir, visible por encima de todas las existentes en la pantalla.
  '  NOTA IMPORTANTE !! : La llamada a esta API requiere como coordenadas la resolucin del monitor, es decir,
  '                         si se tiene una resolucin de 640 x 480, la API SetWindowPos acepta entre 0 -> 640 y 0 -> 480
  '                         como coordenadas vlidas y si se le dan directamente la de los formularios est no corresponden
  '                         a no ser que se haga una divisin por 15.
  iSuceso = SetWindowPos(Formulario.hWnd, Opcion, Formulario.Left / 15, Formulario.Top / 15, _
                         Formulario.Width / 15, Formulario.Height / 15, Flags)
 
End Sub

'***************************************************************************************************
' FUNCIN DE AJUSTE: VentanaEliptica()
'***************************************************************************************************
Public Sub VentanaEliptica(Formulario As Form, x1 As Long, y1 As Long, x2 As Long, y2 As Long)
  Dim Rgn As Long, Ret As Long
  
  Rgn = CreateEllipticRgn(x1, y1, x2, y2)
  Ret = SetWindowRgn(Formulario.hWnd, Rgn, True)
  
End Sub

'***************************************************************************************************
' FUNCIN DE AJUSTE: VentanaRectangularRedondeada()
'***************************************************************************************************
Public Sub VentanaRectangularRedondeada(Formulario As Form, x1 As Long, y1 As Long, x2 As Long, y2 As Long, x3 As Long, y3 As Long)
  Dim Rgn As Long, Ret As Long
  
  Rgn = CreateRoundRectRgn(x1, y1, x2, y2, x3, y3)
  Ret = SetWindowRgn(Formulario.hWnd, Rgn, True)
  
End Sub
