Attribute VB_Name = "Defuzzy"
Option Explicit

'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: DEFUZZY.BAS ##
'
' OBJETIVO: Contener el conjunto de funciones y procedimientos necesarios para el
'           proceso de concrecin o defuzzificacion sobre el resultado de la fase de
'           inferencia en la variables de salida del sistema de control difuso.
'-------------------------------------------------------------------------------------------------------


'**************************************************************************************************
'   PROPIEDAD: QDefuzzificador()
'**************************************************************************************************
Public Property Get QDefuzzificador(TipoConcresor As ETDEFUZZIFICADORES) As String
  Dim Cad As String
  
  Select Case TipoConcresor
    
    Case CoG_Centro_Gravedad:
      Cad = "Centro de Gravedad (CoG)"
    Case CoG_Mayor_Altura:
      Cad = "CoG de Mayor Altura"
    Case CoG_Mayor_Area:
      Cad = "CoG de Mayor Area"
    Case CoG_Ponderado_Altura:
      Cad = "CoG ponderado por la Altura"
    Case CoG_Ponderado_Area:
      Cad = "CoG ponderado por el Area"
    Case CoG_Valores_Importantes:
      Cad = "CoG de Valores Importantes"
      
    Case PMC_Mayor_Altura:
      Cad = "PMC de Mayor Altura"
    Case PMC_Mayor_Area:
      Cad = "PMC de Mayor Area"
    Case PMC_Media:
      Cad = "Media de los Puntos de Mximo Criterio (PMC)"
    Case PMC_Media_Maximos:
      Cad = "Media de Mximos (MoM)"
    Case PMC_Media_Min_Max:
      Cad = "Media del Mnimo y Mximo PMC"
    Case PMC_Ponderado_Altura:
      Cad = "PMC ponderado por la Altura"
    Case PMC_Ponderado_Area:
      Cad = "PMC ponderado por el Area"
    Case PMC_Primer_Maximo:
      Cad = "Primer Mximo"
    Case PMC_Ultimo_Maximo:
      Cad = "ltimo Mximo"
    
    Case CoA_Centro_Area:
      Cad = "Centro de rea (CoA)"
    Case CoA_Valores_Importantes:
      Cad = "CoA de Valores Importantes"
      
  End Select
  QDefuzzificador = Cad
  
End Property

'**************************************************************************************************
'   PROPIEDAD: QTipoDefuzzificador()
'**************************************************************************************************
Public Property Get QTipoDefuzzificador(TipoConcresor As String) As ETDEFUZZIFICADORES
  Dim Tipo As ETDEFUZZIFICADORES
  
  Select Case TipoConcresor
    
    Case "Centro de Gravedad (CoG)":
      Tipo = CoG_Centro_Gravedad
    Case "CoG de Mayor Altura":
      Tipo = CoG_Mayor_Altura
    Case "CoG de Mayor Area":
      Tipo = CoG_Mayor_Area
    Case "CoG ponderado por la Altura":
      Tipo = CoG_Ponderado_Altura
    Case "CoG ponderado por el Area":
      Tipo = CoG_Ponderado_Area
    Case "CoG de Valores Importantes":
      Tipo = CoG_Valores_Importantes
      
    Case "PMC de Mayor Altura":
      Tipo = PMC_Mayor_Altura
    Case "PMC de Mayor Area":
      Tipo = PMC_Mayor_Area
    Case "Media de los Puntos de Mximo Criterio (PMC)":
      Tipo = PMC_Media
    Case "Media de Mximos (MoM)":
      Tipo = PMC_Media_Maximos
    Case "Media del Mnimo y Mximo PMC":
      Tipo = PMC_Media_Min_Max
    Case "PMC ponderado por la Altura":
      Tipo = PMC_Ponderado_Altura
    Case "PMC ponderado por el Area":
      Tipo = PMC_Ponderado_Area
    Case "Primer Mximo":
      Tipo = PMC_Primer_Maximo
    Case "ltimo Mximo":
      Tipo = PMC_Ultimo_Maximo
    
    Case "Centro de rea (CoA)":
      Tipo = CoA_Centro_Area
    Case "CoA de Valores Importantes":
      Tipo = CoA_Valores_Importantes
      
  End Select
  QTipoDefuzzificador = Tipo
  
End Property

'**************************************************************************************************
'   FUNCION: Spline_Area()
'
'       OBJETIVO: Clculo del rea formada por una funcin polinmica a trozos (spline)
'
'**************************************************************************************************
Public Function Spline_Area(CoordX() As Double, CoordY() As Double) As Double
  Dim I As Integer, j As Integer
  Dim TempArea As Double
  
  I = UBound(CoordX(), 1)
  TempArea = 0#
  
  For j = 1 To I - 1
    TempArea = TempArea + CoordY(j) * (CoordX(j + 1) - CoordX(j - 1)) / 2
  Next j
  Spline_Area = TempArea
  
End Function

'**************************************************************************************************
'   FUNCION: Spline_AreaX()
'**************************************************************************************************
Public Function Spline_AreaX(CoordX() As Double, CoordY() As Double) As Double
  Dim I As Integer, j As Integer
  Dim TempAreaX As Double
  
  I = UBound(CoordX(), 1)
  TempAreaX = 0#
  
  For j = 1 To I - 1
    TempAreaX = TempAreaX + CoordY(j) * (CoordX(j + 1) ^ 2 - CoordX(j - 1) ^ 2 + CoordX(j) * CoordX(j + 1) - CoordX(j) * CoordX(j - 1)) / 6
  Next j
  Spline_AreaX = TempAreaX
  
End Function

'**************************************************************************************************
'   FUNCION: Spline_CoG()
'**************************************************************************************************
Public Function Spline_CoG(CoordX() As Double, CoordY() As Double) As Double
  
  If Spline_Area(CoordX(), CoordY()) = 0 Then
    Spline_CoG = 0
  Else
    Spline_CoG = Spline_AreaX(CoordX(), CoordY()) / Spline_Area(CoordX(), CoordY())
  End If
  
End Function

'**************************************************************************************************
'   FUNCION: Spline_PMC()
'**************************************************************************************************
Public Function Spline_PMC(CoordX() As Double, CoordY() As Double) As Double
  Dim I As Integer, j As Integer, NumIntervalos As Integer
  Dim PosicionesYMax() As Integer, TempSumMax, Temp As Double
  Dim PosibleNuevoInterv As Boolean
  
  I = UBound(CoordY(), 1)
  PosibleNuevoInterv = False
  
  For j = 0 To I
    If j = 0 Then
      ReDim PosicionesYMax(0)
      PosicionesYMax(j) = j
      NumIntervalos = 1
    Else
      If CoordY(j) = CoordY(PosicionesYMax(UBound(PosicionesYMax(), 1))) Or Abs(CoordY(j) - CoordY(PosicionesYMax(UBound(PosicionesYMax(), 1)))) < 0.05 Then
         
        ReDim Preserve PosicionesYMax(UBound(PosicionesYMax(), 1) + 1)
        PosicionesYMax(UBound(PosicionesYMax(), 1)) = j
        If PosibleNuevoInterv Then
          NumIntervalos = NumIntervalos + 1
          PosibleNuevoInterv = False
        End If
        
      ElseIf CoordY(j) > CoordY(PosicionesYMax(UBound(PosicionesYMax(), 1))) Then
        Erase PosicionesYMax()
        ReDim PosicionesYMax(0)
        PosicionesYMax(0) = j
        NumIntervalos = 1
        PosibleNuevoInterv = False
      
      Else
        If j < I Then
          PosibleNuevoInterv = True
        End If
      End If
    End If
  Next j
  
  For I = 0 To UBound(CoordY(), 1)
    If CoordY(I) > 0# Then
      Exit For
    Else
      If I = UBound(CoordY(), 1) Then
        Spline_PMC = 0#
        Exit Function
      End If
    End If
  Next I
  
  If NumIntervalos = 1 Then
    Spline_PMC = (CoordX(PosicionesYMax(0)) + CoordX(PosicionesYMax(UBound(PosicionesYMax(), 1)))) / 2
    Exit Function
    
  End If
  
  TempSumMax = CoordX(PosicionesYMax(0))
  
  For I = 1 To UBound(PosicionesYMax(), 1)
    TempSumMax = TempSumMax + CoordX(PosicionesYMax(I))
  Next I
  TempSumMax = TempSumMax / (2 * NumIntervalos)
  
  Spline_PMC = TempSumMax
  
End Function

'**************************************************************************************************
'   FUNCION: Spline_Altura()
'**************************************************************************************************
Public Function Spline_Altura(CoordX() As Double, CoordY() As Double) As Double
  Dim I As Integer, PosYmax As Integer
  Dim ValorYmax As Double
  
  PosYmax = 0
  ValorYmax = 0#
  
  For I = 0 To UBound(CoordY(), 1)
    If CoordY(I) > ValorYmax Then
      PosYmax = I
      ValorYmax = CoordY(I)
    End If
  Next I
  
  Spline_Altura = CoordY(PosYmax)
  
End Function



'-----------------------------------------------


'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_CoG()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_CoG(IdSalida As Integer, IdVCrisp As Integer)
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = Spline_CoG(SistActual.Salida(IdSalida).CoordXAgreg(), SistActual.Salida(IdSalida).CoordYAgreg())
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MoM()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MoM(IdSalida As Integer, IdVCrisp As Integer)
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = Spline_PMC(SistActual.Salida(IdSalida).CoordXAgreg(), SistActual.Salida(IdSalida).CoordYAgreg())
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_CoG_PONDERADO_AREA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_CoG_PONDERADO_AREA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer
  Dim TempArea As Double, TempCoG As Double, TempResultado As Double
  Dim TempSumArea As Double
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  TempResultado = 0#
  TempSumArea = 0#
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    TempArea = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.S
    TempSumArea = TempSumArea + TempArea
    TempCoG = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.w
    
    TempResultado = TempResultado + TempArea * TempCoG
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = TempResultado / TempSumArea
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_CoG_PONDERADO_ALTURA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_CoG_PONDERADO_ALTURA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer
  Dim TempAltura As Double, TempCoG As Double, TempResultado As Double
  Dim TempSumAltura As Double
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  TempResultado = 0#
  TempSumAltura = 0#
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    TempAltura = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.H
    TempSumAltura = TempSumAltura + TempAltura
    TempCoG = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.w
    
    TempResultado = TempResultado + TempAltura * TempCoG
  Next I
  
  If TempResultado = 0# Or TempSumAltura = 0# Then
    SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = SistActual.Salida(IdSalida).UnivDiscurso(0) + EPSILON
  Else
    SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = TempResultado / TempSumAltura
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MoM_PONDERADO_AREA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MoM_PONDERADO_AREA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer
  Dim TempArea As Double, TempMoM As Double, TempResultado As Double
  Dim TempSumArea As Double
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  TempResultado = 0#
  TempSumArea = 0#
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    TempArea = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.S
    TempSumArea = TempSumArea + TempArea
    TempMoM = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.G
    
    TempResultado = TempResultado + TempArea * TempMoM
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = TempResultado / TempSumArea
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MoM_PONDERADO_ALTURA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MoM_PONDERADO_ALTURA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer
  Dim TempAltura As Double, TempMoM As Double, TempResultado As Double
  Dim TempSumAltura As Double
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  TempResultado = 0#
  TempSumAltura = 0#
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    TempAltura = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.H
    TempSumAltura = TempSumAltura + TempAltura
    TempMoM = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.G
    
    TempResultado = TempResultado + TempAltura * TempMoM
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = TempResultado / TempSumAltura
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MEDIA_MoM()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MEDIA_MoM(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer, j As Integer
  Dim TempMoM As Double, TempSuma As Double
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  TempSuma = 0#
  j = 0
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    TempMoM = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.G
    If TempMoM > 0# Then
      j = j + 1
      TempSuma = TempSuma + TempMoM
    End If
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = TempSuma / j
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MEDIA_MINMAX_MoM()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MEDIA_MINMAX_MoM(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer
  Dim TempMoM As Double, MoMMin As Double, MoMMax As Double
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  MoMMin = 0#
  MoMMax = 0#
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    TempMoM = SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.G
    If TempMoM > MoMMax Then
      MoMMax = TempMoM
    End If
    If TempMoM < MoMMin Then
      MoMMin = TempMoM
    End If
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = (MoMMin + MoMMax) / 2
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_CoG_MAYOR_AREA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_CoG_MAYOR_AREA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer, IdMayorArea As Integer
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  IdMayorArea = 0
  For I = 0 To NConjDif(S, IdSalida) - 1
    If SistActual.Salida(IdSalida).ConjDif(IdMayorArea).Caracteristicas.S < SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.S Then
      IdMayorArea = I
    End If
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = SistActual.Salida(IdSalida).ConjDif(IdMayorArea).Caracteristicas.w
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_CoG_MAYOR_ALTURA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_CoG_MAYOR_ALTURA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer, IdMayorAltura
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    If SistActual.Salida(IdSalida).ConjDif(IdMayorAltura).Caracteristicas.H < SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.H Then
      IdMayorAltura = I
    End If
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = SistActual.Salida(IdSalida).ConjDif(IdMayorAltura).Caracteristicas.w
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MoM_MAYOR_AREA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MoM_MAYOR_AREA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer, IdMayorArea As Integer
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  IdMayorArea = 0
  For I = 0 To NConjDif(S, IdSalida) - 1
    If SistActual.Salida(IdSalida).ConjDif(IdMayorArea).Caracteristicas.S < SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.S Then
      IdMayorArea = I
    End If
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = SistActual.Salida(IdSalida).ConjDif(IdMayorArea).Caracteristicas.G
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_MoM_MAYOR_ALTURA()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_MoM_MAYOR_ALTURA(IdSalida As Integer, IdVCrisp As Integer)
  Dim I As Integer, IdMayorAltura
  
  CALCULAR_CARACTERISTICAS_SALIDA IdSalida
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    If SistActual.Salida(IdSalida).ConjDif(IdMayorAltura).Caracteristicas.H < SistActual.Salida(IdSalida).ConjDif(I).Caracteristicas.H Then
      IdMayorAltura = I
    End If
  Next I
  
  SistActual.Salida(IdSalida).VCrisp(IdVCrisp) = SistActual.Salida(IdSalida).ConjDif(IdMayorAltura).Caracteristicas.G
End Sub

'**************************************************************************************************
'   FUNCION: CALCULAR_CARACTERISTICAS_CONJDIF()
'**************************************************************************************************
Public Sub CALCULAR_CARACTERISTICAS_CONJDIF(IdSalida As Integer, IdConjDif As Integer)
  
  'CLCULO DEL PUNTO DE MXIMO CRITERIO PMC o MoM O G
  SistActual.Salida(IdSalida).ConjDif(IdConjDif).Caracteristicas.G = Spline_PMC(SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordXGActiv(), _
                                                                        SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordYGActiv())
                                                                        
  'CLCULO DEL REA O SUPERFICIE S
  SistActual.Salida(IdSalida).ConjDif(IdConjDif).Caracteristicas.S = Spline_Area(SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordXGActiv(), _
                                                                         SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordYGActiv())
                                                                           
                                                                           
  'CLCULO DEL CENTRO DE GRAVEDAD CoG o W
  SistActual.Salida(IdSalida).ConjDif(IdConjDif).Caracteristicas.w = Spline_CoG(SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordXGActiv(), _
                                                                        SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordYGActiv())
                                                                        
  'CLCULO DE LA ALTURA
  SistActual.Salida(IdSalida).ConjDif(IdConjDif).Caracteristicas.H = Spline_Altura(SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordXGActiv(), _
                                                                           SistActual.Salida(IdSalida).ConjDif(IdConjDif).CoordYGActiv())
End Sub

'**************************************************************************************************
'   FUNCION: CALCULAR_CARACTERISTICAS_CONJDIF()
'**************************************************************************************************
Public Sub CALCULAR_CARACTERISTICAS_SALIDA(IdSalida As Integer)
  Dim I As Integer
  
  For I = 0 To NConjDif(S, IdSalida) - 1
    CALCULAR_CARACTERISTICAS_CONJDIF IdSalida, I
  Next I
  
End Sub
'**************************************************************************************************
'   PROCEDIMIENTO: DEFUZZIFICAR_SALIDAS()
'**************************************************************************************************
Public Sub DEFUZZIFICAR_SALIDAS(IdVCrisp As Integer, Optional AllSalidas As Boolean = False, Optional IdSalida As Integer)
  Dim I As Integer
  
  If AllSalidas Then
    For I = 0 To NSalidas - 1
        If IdVCrisp = 0 Then
          ReDim SistActual.Salida(I).VCrisp(0)
        Else
          ReDim Preserve SistActual.Salida(I).VCrisp(IdVCrisp)
        End If

        Select Case SistActual.Salida(I).Defuzzificador
      
          Case CoA_Centro_Area:
            
          Case CoA_Valores_Importantes:
          
          Case CoG_Centro_Gravedad:
            DEFUZZIFICAR_CoG I, IdVCrisp
            
          Case CoG_Mayor_Altura:
            DEFUZZIFICAR_CoG_MAYOR_ALTURA I, IdVCrisp
            
          Case CoG_Mayor_Area:
            DEFUZZIFICAR_CoG_MAYOR_AREA I, IdVCrisp
            
          Case CoG_Ponderado_Altura:
            DEFUZZIFICAR_CoG_PONDERADO_ALTURA I, IdVCrisp
            
          Case CoG_Ponderado_Area:
            DEFUZZIFICAR_CoG_PONDERADO_AREA I, IdVCrisp
            
          Case CoG_Valores_Importantes:
          
          Case PMC_Mayor_Altura:
            DEFUZZIFICAR_MoM_MAYOR_ALTURA I, IdVCrisp
            
          Case PMC_Mayor_Area:
            DEFUZZIFICAR_MoM_MAYOR_AREA I, IdVCrisp
            
          Case PMC_Media:
            DEFUZZIFICAR_MEDIA_MoM I, IdVCrisp
            
          Case PMC_Media_Maximos:
            DEFUZZIFICAR_MoM I, IdVCrisp
          
          Case PMC_Media_Min_Max:
            DEFUZZIFICAR_MEDIA_MINMAX_MoM I, IdVCrisp
            
          Case PMC_Ponderado_Altura:
            DEFUZZIFICAR_MoM_PONDERADO_ALTURA I, IdVCrisp
            
          Case PMC_Ponderado_Area:
            DEFUZZIFICAR_MoM_PONDERADO_AREA I, IdVCrisp
          
          Case PMC_Primer_Maximo:
          
          Case PMC_Ultimo_Maximo:
          
        End Select
    Next I
    
  Else
    If IdVCrisp = 0 Then
      ReDim SistActual.Salida(IdSalida).VCrisp(0)
    Else
      ReDim Preserve SistActual.Salida(IdSalida).VCrisp(IdVCrisp + 1)
    End If
    
    Select Case SistActual.Salida(IdSalida).Defuzzificador
      
      Case CoA_Centro_Area:
        
      Case CoA_Valores_Importantes:
      
      Case CoG_Centro_Gravedad:
        DEFUZZIFICAR_CoG IdSalida, IdVCrisp
        
      Case CoG_Mayor_Altura:
        DEFUZZIFICAR_CoG_MAYOR_ALTURA IdSalida, IdVCrisp
        
      Case CoG_Mayor_Area:
        DEFUZZIFICAR_CoG_MAYOR_AREA IdSalida, IdVCrisp
        
      Case CoG_Ponderado_Altura:
        DEFUZZIFICAR_CoG_PONDERADO_ALTURA IdSalida, IdVCrisp
        
      Case CoG_Ponderado_Area:
        DEFUZZIFICAR_CoG_PONDERADO_AREA IdSalida, IdVCrisp
        
      Case CoG_Valores_Importantes:
      
      Case PMC_Mayor_Altura:
        DEFUZZIFICAR_MoM_MAYOR_ALTURA IdSalida, IdVCrisp
        
      Case PMC_Mayor_Area:
        DEFUZZIFICAR_MoM_MAYOR_AREA IdSalida, IdVCrisp
        
      Case PMC_Media:
        DEFUZZIFICAR_MEDIA_MoM IdSalida, IdVCrisp
        
      Case PMC_Media_Maximos:
        DEFUZZIFICAR_MoM IdSalida, IdVCrisp
        
      Case PMC_Media_Min_Max:
        DEFUZZIFICAR_MEDIA_MINMAX_MoM IdSalida, IdVCrisp
        
      Case PMC_Ponderado_Altura:
        DEFUZZIFICAR_MoM_PONDERADO_ALTURA IdSalida, IdVCrisp
        
      Case PMC_Ponderado_Area:
        DEFUZZIFICAR_MoM_PONDERADO_AREA IdSalida, IdVCrisp
        
      Case PMC_Primer_Maximo:
      
      Case PMC_Ultimo_Maximo:
      
    End Select
      
  End If

End Sub
