Attribute VB_Name = "Graphs"
Option Explicit

'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: GRAPHS ##
'
' OBJETIVO: Procedimientos y funciones que tienen como objetivo la representacin a travs de controles
'           y mtodos grficos de las caractersticas de las variables difusor, conjuntos difusos, secuencias, ...
'-------------------------------------------------------------------------------------------------------



'**************************************************************************************************
'   PROPIEDAD: QColorAsociadoCDifuso
'**************************************************************************************************
Public Property Get QColorAsociadoCDifuso(TipoCDifuso As ETCONJDIF) As Long
  
  Select Case TipoCDifuso
    Case L: QColorAsociadoCDifuso = vbBlue
    Case Gamma: QColorAsociadoCDifuso = vbBlue
    
    Case DifSingleton: QColorAsociadoCDifuso = vbWhite
    Case Singleton: QColorAsociadoCDifuso = vbWhite
    
    Case DifIntervalo: QColorAsociadoCDifuso = vbCyan
    Case Intervalo: QColorAsociadoCDifuso = vbCyan
    
    Case DifTriangulo: QColorAsociadoCDifuso = vbRed
    Case Triangulo: QColorAsociadoCDifuso = vbRed
    
    Case DifTrapecio: QColorAsociadoCDifuso = vbGreen
    Case Trapecio: QColorAsociadoCDifuso = vbGreen
    
    Case DifTrapecioExtendido: QColorAsociadoCDifuso = vbMagenta
    Case TrapecioExtendido: QColorAsociadoCDifuso = vbMagenta
  End Select
  
End Property


'**************************************************************************************************
'   PROCEDIMIENTO: DibujarGraficosVariable()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub DibujarGraficosVariable(EoS As ETEOS, IdEoS As Integer, _
                                   PBoxControlVariable As PictureBox, _
                                   LblEtiqueta As Label, _
                                   Optional LblMin As Label, Optional LblMed As Label, Optional LblMax As Label, _
                                   Optional PboxControlDifusor As PictureBox, _
                                   Optional LblMinD As Label, Optional LblMedD As Label, Optional LblMaxD As Label, _
                                   Optional QCDifusoSeleccionado As Integer = -1, Optional SeleccionadoDifusor As Boolean = False)
                                   
  
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single
  Dim Min, Max As Double
  
  Static xD, xincD, xdecD, yD, yincD, ydecD As Single
  Dim xdivD, ydivD As Single
  Dim RelacionEscalaXD, RelacionEscalaYD As Single
  Dim MinD, MaxD As Double
  
  
  'N DE FILAS Y COLUMNAS
  Dim NDV As Integer
  Dim NDH As Integer
  
  NDV = OpcionesInicio.NDivisionesVerticales
  NDH = OpcionesInicio.NDivisionesHorizontales
  
  PBoxControlVariable.Cls

  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 7
  Const PorcentajeXdec = 2
  Const PorcentajeYinc = 13
  Const PorcentajeYdec = 12
  
  Const PorcentajeXincD = 12
  Const PorcentajeXdecD = 5
  Const PorcentajeYincD = 12
  Const PorcentajeYdecD = 12
  
  
  'RANGO DE VALORES
  Select Case EoS
    Case E:
      PboxControlDifusor.Cls
      
      Min = SistActual.Entrada(IdEoS).UnivDiscurso(0) + EPSILON
      Max = SistActual.Entrada(IdEoS).UnivDiscurso(1) - EPSILON
      
      
      MinD = SistActual.Entrada(IdEoS).Difusor.PBasicos(0)
      MaxD = SistActual.Entrada(IdEoS).Difusor.PBasicos(3)
      
      If Abs(MaxD - MinD) < 4 * EPSILON Then
        MinD = -1
        MaxD = 1
      End If
      
    Case S:
    
      Min = SistActual.Salida(IdEoS).UnivDiscurso(0) + EPSILON
      Max = SistActual.Salida(IdEoS).UnivDiscurso(1) - EPSILON
      
  End Select
  
  If Abs(Max - Min) < 4 * EPSILON Then
        Min = -1
        Max = 1
  End If
  
  'ESCALADO
  xinc = (PBoxControlVariable.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBoxControlVariable.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBoxControlVariable.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBoxControlVariable.ScaleHeight * PorcentajeYdec) / 100
  
  
  If EoS = E Then
                                        
    'Parte del Difusor
    xincD = (PboxControlDifusor.ScaleWidth * PorcentajeXincD) / 100
    xdecD = (PboxControlDifusor.ScaleWidth * PorcentajeXdecD) / 100
    yincD = (PboxControlDifusor.ScaleHeight * PorcentajeYincD) / 100
    ydecD = (PboxControlDifusor.ScaleHeight * PorcentajeYdecD) / 100
    
    RelacionEscalaXD = Abs(MaxD - MinD) / (PboxControlDifusor.ScaleWidth - xincD - xdecD)
    RelacionEscalaYD = 1 / (PboxControlDifusor.ScaleHeight - yincD - ydecD)
      
  End If

  RelacionEscalaX = Abs(Max - Min) / (PBoxControlVariable.ScaleWidth - xinc - xdec)
  RelacionEscalaY = 1 / (PBoxControlVariable.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  If EoS = E Then
    xincD = xincD * RelacionEscalaXD
    xdecD = xdecD * RelacionEscalaXD
    yincD = yincD * RelacionEscalaYD
    ydecD = ydecD * RelacionEscalaYD
  End If
  
  'PERSONALIZACIN DEL ESCALADO
    With PBoxControlVariable
      .ScaleLeft = Min - xinc
      .ScaleTop = 1 + ydec
      .ScaleWidth = Abs(Max - Min) + xinc + xdec
      .ScaleHeight = -(1 + yinc + ydec)
    End With
    
    If EoS = E Then
      With PboxControlDifusor
        .ScaleLeft = MinD - xincD
        .ScaleTop = 1 + ydecD
        .ScaleWidth = Abs(MaxD - MinD) + xincD + xdecD
        .ScaleHeight = -(1 + yincD + ydecD)
      End With
    End If
    
    
  'CUADRCULA
  xdiv = Abs(Max - Min) / NDV
  ydiv = 1 / NDH
  
  If EoS = E Then
    xdivD = Abs(MaxD - MinD) / NDV
    ydivD = 1 / NDH
  End If
  
  'ETIQUETAS
  If NDV <= 3 Then
    If EoS = E Then
      LblMinD.Caption = CStr(FormatNumber(MinD, 4))
      LblMinD.Left = MinD - (0.5 * xdivD) / 10
      
      LblMedD.Caption = CStr(FormatNumber(((MaxD + MinD) / 2), 4))
      LblMedD.Left = ((MaxD + MinD) / 2) - (0.5 * xdivD) / 10
      
      LblMaxD.Caption = CStr(FormatNumber(MaxD, 4))
      LblMaxD.Left = MaxD - (0.5 * xdivD) / 10
    
    End If
    
    LblMin.Caption = CStr(FormatNumber(Min, 4)) & " " & QUndSigla(EoS, IdEoS)
    LblMin.Left = Min - (0.25 * xdiv) / 10
    
    LblMed.Caption = CStr(FormatNumber((Max + Min) / 2, 4))
    LblMed.Left = ((Max + Min) / 2) - (0.25 * xdiv) / 10
    
    LblMax.Caption = CStr(FormatNumber(Max, 4))
    LblMax.Left = Max - (0.25 * xdiv) / 10
    
    
  Else
    
    If EoS = E Then
      LblMinD.Caption = CStr(FormatNumber(MinD, 4))
      LblMinD.Left = MinD - (3 * xdivD) / 10
      
      LblMedD.Caption = CStr(FormatNumber(((MaxD + MinD) / 2), 4))
      LblMedD.Left = ((MaxD + MinD) / 2) - (3 * xdivD) / 10
      
      LblMaxD.Caption = CStr(FormatNumber(MaxD, 4))
      LblMaxD.Left = MaxD - (3 * xdivD) / 10
      
    End If
    
    LblMin.Caption = CStr(FormatNumber(Min, 4)) & " " & QUndSigla(EoS, IdEoS)
    LblMin.Left = Min - (1.9 * xdiv) / 10
    
    LblMed.Caption = CStr(FormatNumber((Max + Min) / 2, 4))
    LblMed.Left = ((Max + Min) / 2) - (1.9 * xdiv) / 10
    
    LblMax.Caption = CStr(FormatNumber(Max, 4))
    LblMax.Left = Max - (1.9 * xdiv) / 10
  
  End If
  
  If QCDifusoSeleccionado >= 0 Then
    LblEtiqueta.Caption = "ETIQUETA SELECCIONADA: << " & UCase$(QEtiqueta(EoS, IdEoS, QCDifusoSeleccionado)) & " >>"
  Else
    LblEtiqueta.Caption = "C. DIFUSOS - ETIQUETAS LINGSTICAS"
  End If
  
  If EoS = E Then
    PboxControlDifusor.DrawWidth = 1
  End If
  PBoxControlVariable.DrawWidth = 1
  
  
  'LINEAS HORIZONTALES
  y = 0
  For i = 1 To NDH + 1
    If i = 1 Then
      PBoxControlVariable.Line (Min, y)-(Max + xinc / 7, y), vbWhite
    Else
      PBoxControlVariable.Line (Min, y)-(Max, y), vbWhite
    End If
    If NDH = 1 Then Exit For
    y = y + ydiv
  Next i
  
  'PEQUEAS LINEAS HORIZONTALES INDICADORAS DE 0, 0.25, 0.5, 0.75, 1.0
  y = 0
  For i = 0 To 4
    PBoxControlVariable.Line (Min - (xinc / 7), y)-(Min, y), vbWhite
    y = y + 0.25
  Next i
  
  'LINEAS VERTICALES
  x = Min
  For i = 1 To NDV + 1
    If i = 1 Then
      PBoxControlVariable.Line (x, 0)-(x, 1 + yinc / 3), vbWhite
    Else
      PBoxControlVariable.Line (x, 0)-(x, 1), vbWhite
    End If
    If NDV = 1 Then Exit For
    x = x + xdiv
  Next i
  
  'PEQUEAS LINEAS HORIZONTALES INDICADORAS DEL MNIMO, MEDIO Y MXIMO
  PBoxControlVariable.Line (Min, 0 - (yinc / 6))-(Min, 0), vbWhite
  PBoxControlVariable.Line ((Min + Max) / 2, 0 - (yinc / 6))-((Min + Max) / 2, 0), vbWhite
  PBoxControlVariable.Line (Max, 0 - (yinc / 6))-(Max, 0), vbWhite
  
  'Para el Difusor
  If EoS = E Then
    'LINEAS HORIZONTALES
    yD = 0
    For i = 1 To NDH + 1
      If i = 1 Then
        PboxControlDifusor.Line (MinD, yD)-(MaxD + xincD / 5, yD), vbWhite
      Else
        PboxControlDifusor.Line (MinD, yD)-(MaxD, yD), vbWhite
      End If
      If NDH = 1 Then Exit For
      yD = yD + ydivD
    Next i
    
    'PEQUEAS LINEAS HORIZONTALES INDICADORAS DE 0, 0.25, 0.5, 0.75, 1.0
    yD = 0
    For i = 0 To 4
      PboxControlDifusor.Line (MinD - (xincD / 7), yD)-(MinD, yD), vbWhite
      yD = yD + 0.25
    Next i
    
    'LINEAS VERTICALES
    xD = MinD
    For i = 1 To NDV + 1
      If i = 1 Then
        PboxControlDifusor.Line (xD, 0)-(xD, 1 + yincD / 3), vbWhite
      Else
        PboxControlDifusor.Line (xD, 0)-(xD, 1), vbWhite
      End If
      If NDV = 1 Then Exit For
      xD = xD + xdivD
    Next i
    
    'PEQUEAS LINEAS HORIZONTALES INDICADORAS DEL MNIMO, MEDIO Y MXIMO
    PboxControlDifusor.Line (MinD, 0 - (yincD / 7))-(MinD, 0), vbWhite
    PboxControlDifusor.Line ((MinD + MaxD) / 2, 0 - (yincD / 7))-((MinD + MaxD) / 2, 0), vbWhite
    PboxControlDifusor.Line (MaxD, 0 - (yincD / 7))-(MaxD, 0), vbWhite
    
  End If
  
  If EoS = E Then
    If SeleccionadoDifusor Then
      DibujarCDifusosVariable E, IdEoS, PBoxControlVariable, PboxControlDifusor, , True
    Else
      DibujarCDifusosVariable E, IdEoS, PBoxControlVariable, PboxControlDifusor, QCDifusoSeleccionado
    End If
  Else
    DibujarCDifusosVariable S, IdEoS, PBoxControlVariable, , QCDifusoSeleccionado
  End If
  
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DibujarGraficosVariableProceso()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub DibujarGraficosVariableProceso(EoS As ETEOS, IdEoS As Integer, PBoxControlVariable As PictureBox)
                                   
  
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single
  Dim Min, Max As Double
  
  
  'N DE FILAS Y COLUMNAS
  Dim NDV As Integer
  Dim NDH As Integer
  
  NDV = OpcionesInicio.NDivisionesVerticales
  NDH = OpcionesInicio.NDivisionesHorizontales
  
  PBoxControlVariable.Cls

  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
  
  
  'RANGO DE VALORES
  Select Case EoS
    Case E:
      
      Min = SistActual.Entrada(IdEoS).UnivDiscurso(0) + EPSILON
      Max = SistActual.Entrada(IdEoS).UnivDiscurso(1) - EPSILON
      
      
    Case S:
    
      Min = SistActual.Salida(IdEoS).UnivDiscurso(0) + EPSILON
      Max = SistActual.Salida(IdEoS).UnivDiscurso(1) - EPSILON
      
  End Select
  
  If Abs(Max - Min) < 4 * EPSILON Then
        Min = -1
        Max = 1
  End If
  
  'ESCALADO
  xinc = (PBoxControlVariable.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBoxControlVariable.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBoxControlVariable.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBoxControlVariable.ScaleHeight * PorcentajeYdec) / 100
  
  

  RelacionEscalaX = Abs(Max - Min) / (PBoxControlVariable.ScaleWidth - xinc - xdec)
  RelacionEscalaY = 1 / (PBoxControlVariable.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  
  'PERSONALIZACIN DEL ESCALADO
    With PBoxControlVariable
      .ScaleLeft = Min - xinc
      .ScaleTop = 1 + ydec
      .ScaleWidth = Abs(Max - Min) + xinc + xdec
      .ScaleHeight = -(1 + yinc + ydec)
    End With
    
  
  DibujarCDifusosVariableProceso EoS, IdEoS, PBoxControlVariable
  
  
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DibujarCDifusosVariableProceso()
'
'       OBJETIVO:
'
'**************************************************************************************************
Private Sub DibujarCDifusosVariableProceso(EoS As ETEOS, IdEoS As Integer, PBoxControlVariable As PictureBox)
 
 'En caso de que se produzca un error en tiempo de ejecucin se ejecutar la instruccin siguiente
  Dim Color As Long
  Dim PtosCDifuso() As Double
  Dim i, j As Integer
  
  Select Case EoS
    Case E:
      If NConjDif(EoS, IdEoS) = 0 Then
        Exit Sub
      Else
        PBoxControlVariable.DrawWidth = 2
        For i = 0 To NConjDif(EoS, IdEoS) - 1
          Color = QColorAsociadoCDifuso(SistActual.Entrada(IdEoS).ConjDif(i).Tipo)
          VectorPtosCDifuso SistActual.Entrada(IdEoS).ConjDif(i), PtosCDifuso
          For j = 2 To UBound(PtosCDifuso(), 1) Step 2
            PBoxControlVariable.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
          Next j
        Next i
      End If
      
    Case S:
      If NConjDif(EoS, IdEoS) = 0 Then
        Exit Sub
      Else
        PBoxControlVariable.DrawWidth = 2
        For i = 0 To NConjDif(EoS, IdEoS) - 1
          Color = QColorAsociadoCDifuso(SistActual.Salida(IdEoS).ConjDif(i).Tipo)
          VectorPtosCDifuso SistActual.Salida(IdEoS).ConjDif(i), PtosCDifuso
          For j = 2 To UBound(PtosCDifuso(), 1) Step 2
            PBoxControlVariable.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
          Next j
        Next i
      End If
  
  End Select
  
     
End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: DibujarCDifusosVariable()
'
'       OBJETIVO:
'
'**************************************************************************************************
Private Sub DibujarCDifusosVariable(EoS As ETEOS, IdEoS As Integer, _
                                    PBoxControlVariable As PictureBox, Optional PboxControlDifusor As PictureBox, _
                                    Optional QCDifusoSeleccionado As Integer = -1, Optional SeleccionadoDifusor As Boolean = False)
 
 'En caso de que se produzca un error en tiempo de ejecucin se ejecutar la instruccin siguiente
  Dim Color As Long
  Dim PtosCDifuso() As Double
  Dim i, j As Integer
  
  Select Case EoS
    Case E:
      
      'Difusor
      PboxControlDifusor.DrawWidth = 2
      Color = QColorAsociadoCDifuso(SistActual.Entrada(IdEoS).Difusor.Tipo)
      
      If SeleccionadoDifusor Then
        Color = vbYellow
        If SistActual.Entrada(IdEoS).Difusor.Tipo <> DifTrapecioExtendido Then
          PboxControlDifusor.DrawWidth = 6
          PboxControlDifusor.PSet (SistActual.Entrada(IdEoS).Difusor.PBasicos(0), 0), Color
          PboxControlDifusor.PSet (SistActual.Entrada(IdEoS).Difusor.PBasicos(1), 1), Color
          PboxControlDifusor.PSet (SistActual.Entrada(IdEoS).Difusor.PBasicos(2), 1), Color
          PboxControlDifusor.PSet (SistActual.Entrada(IdEoS).Difusor.PBasicos(3), 0), Color
          PboxControlDifusor.DrawWidth = 2
          PboxControlDifusor.Line (SistActual.Entrada(IdEoS).Difusor.PBasicos(0), 0)-(SistActual.Entrada(IdEoS).Difusor.PBasicos(1), 1), Color
          PboxControlDifusor.Line (SistActual.Entrada(IdEoS).Difusor.PBasicos(1), 1)-(SistActual.Entrada(IdEoS).Difusor.PBasicos(2), 1), Color
          PboxControlDifusor.Line (SistActual.Entrada(IdEoS).Difusor.PBasicos(2), 1)-(SistActual.Entrada(IdEoS).Difusor.PBasicos(3), 0), Color
        Else
          VectorPtosDifusor SistActual.Entrada(IdEoS), PtosCDifuso
          For j = 2 To UBound(PtosCDifuso(), 1) Step 2
              PboxControlDifusor.DrawWidth = 6
              PboxControlDifusor.PSet (PtosCDifuso(j - 2), PtosCDifuso(j - 1)), Color
              PboxControlDifusor.PSet (PtosCDifuso(j), PtosCDifuso(j + 1)), Color
              PboxControlDifusor.DrawWidth = 2
              PboxControlDifusor.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
          Next j
        End If
        
      Else
        If SistActual.Entrada(IdEoS).Difusor.Tipo <> DifTrapecioExtendido Then
          PboxControlDifusor.Line (SistActual.Entrada(IdEoS).Difusor.PBasicos(0), 0)-(SistActual.Entrada(IdEoS).Difusor.PBasicos(1), 1), Color
          PboxControlDifusor.Line (SistActual.Entrada(IdEoS).Difusor.PBasicos(1), 1)-(SistActual.Entrada(IdEoS).Difusor.PBasicos(2), 1), Color
          PboxControlDifusor.Line (SistActual.Entrada(IdEoS).Difusor.PBasicos(2), 1)-(SistActual.Entrada(IdEoS).Difusor.PBasicos(3), 0), Color
        Else
          VectorPtosDifusor SistActual.Entrada(IdEoS), PtosCDifuso
          For j = 2 To UBound(PtosCDifuso(), 1) Step 2
            PboxControlDifusor.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
          Next j
        End If
        
      End If
        
      
      If NConjDif(EoS, IdEoS) = 0 Then
        Exit Sub
      Else
        PBoxControlVariable.DrawWidth = 2
        For i = 0 To NConjDif(EoS, IdEoS) - 1
          
          If QCDifusoSeleccionado = i Then
            Color = vbYellow
            VectorPtosCDifuso SistActual.Entrada(IdEoS).ConjDif(i), PtosCDifuso
            For j = 2 To UBound(PtosCDifuso(), 1) Step 2
              PBoxControlVariable.DrawWidth = 6
              PBoxControlVariable.PSet (PtosCDifuso(j - 2), PtosCDifuso(j - 1)), Color
              PBoxControlVariable.PSet (PtosCDifuso(j), PtosCDifuso(j + 1)), Color
              PBoxControlVariable.DrawWidth = 2
              PBoxControlVariable.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
            Next j
            GoTo SGTECONJD1
            
          Else
            Color = QColorAsociadoCDifuso(SistActual.Entrada(IdEoS).ConjDif(i).Tipo)
          End If
          
          VectorPtosCDifuso SistActual.Entrada(IdEoS).ConjDif(i), PtosCDifuso
          For j = 2 To UBound(PtosCDifuso(), 1) Step 2
            PBoxControlVariable.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
          Next j
SGTECONJD1:
        Next i
        
      End If
      
    Case S:
      
      
      If NConjDif(EoS, IdEoS) = 0 Then
        Exit Sub
      Else
        PBoxControlVariable.DrawWidth = 2
        For i = 0 To NConjDif(EoS, IdEoS) - 1
          
          If QCDifusoSeleccionado = i Then
            Color = vbYellow
            VectorPtosCDifuso SistActual.Salida(IdEoS).ConjDif(i), PtosCDifuso
            For j = 2 To UBound(PtosCDifuso(), 1) Step 2
              PBoxControlVariable.DrawWidth = 6
              PBoxControlVariable.PSet (PtosCDifuso(j - 2), PtosCDifuso(j - 1)), Color
              PBoxControlVariable.PSet (PtosCDifuso(j), PtosCDifuso(j + 1)), Color
              PBoxControlVariable.DrawWidth = 2
              PBoxControlVariable.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
            Next j
            GoTo SGTECONJD2
            
          Else
            Color = QColorAsociadoCDifuso(SistActual.Salida(IdEoS).ConjDif(i).Tipo)
          End If
          
          
          VectorPtosCDifuso SistActual.Salida(IdEoS).ConjDif(i), PtosCDifuso
          For j = 2 To UBound(PtosCDifuso(), 1) Step 2
            PBoxControlVariable.Line (PtosCDifuso(j - 2), PtosCDifuso(j - 1))-(PtosCDifuso(j), PtosCDifuso(j + 1)), Color
          Next j
SGTECONJD2:
        Next i
        
      End If
      
  End Select
  
     
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO:
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub DibujarCuadricula(PBox As PictureBox)
  
  
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single
  Dim Min, Max As Integer
  
  'N DE FILAS Y COLUMNAS
  Dim NDV As Integer
  Dim NDH As Integer
  
  NDV = OpcionesInicio.NDivisionesVerticales
  NDH = OpcionesInicio.NDivisionesHorizontales
  

  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
  
  
  'RANGO DE VALORES
  Min = 0
  Max = 1
      
  PBox.Cls
  
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = Abs(Max - Min) / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = 1 / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = Min - xinc
    .ScaleTop = 1 + ydec
    .ScaleWidth = Abs(Max - Min) + xinc + xdec
    .ScaleHeight = -(1 + yinc + ydec)
  End With
       
  'CUADRCULA
  xdiv = Abs(Max - Min) / NDV
  ydiv = 1 / NDH
  
  PBox.DrawWidth = 1
  'LINEAS HORIZONTALES
  y = 0
  For i = 1 To NDH + 1
    PBox.Line (Min, y)-(Max, y), vbWhite
    If NDH = 1 Then Exit For
    y = y + ydiv
  Next i
  
  'LINEAS VERTICALES
  x = Min
  For i = 1 To NDV + 1
    PBox.Line (x, 0)-(x, 1), vbWhite
    If NDV = 1 Then Exit For
    x = x + xdiv
  Next i
  
 
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO:
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub DibujarCuadriculaTemporal(PBox As PictureBox, NDV As Integer, NDH As Integer)
  
  
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single
  Dim Min, Max As Integer
  

  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
  
  
  'RANGO DE VALORES
  Min = 0
  Max = 1
      
  PBox.Cls
  
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = Abs(Max - Min) / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = 1 / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = Min - xinc
    .ScaleTop = 1 + ydec
    .ScaleWidth = Abs(Max - Min) + xinc + xdec
    .ScaleHeight = -(1 + yinc + ydec)
  End With
       
  'CUADRCULA
  xdiv = Abs(Max - Min) / NDV
  ydiv = 1 / NDH
  
  PBox.DrawWidth = 1
  'LINEAS HORIZONTALES
  y = 0
  For i = 1 To NDH + 1
    PBox.Line (Min, y)-(Max, y), vbWhite
    If NDH = 1 Then Exit For
    y = y + ydiv
  Next i
  
  'LINEAS VERTICALES
  x = Min
  For i = 1 To NDV + 1
    PBox.Line (x, 0)-(x, 1), vbWhite
    If NDV = 1 Then Exit For
    x = x + xdiv
  Next i
  
 
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: Dibujar_Spline()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub Dibujar_Spline(CoordX() As Double, CoordY() As Double, Color As Long, Grosor As Integer, PBox As PictureBox, _
                          Optional ConResaltePto As Boolean = False, Optional PtoX As Double, Optional PtoY As Double)
 
  
  Dim MId As Integer
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single
  Dim Min, Max As Double
  
  'N DE FILAS Y COLUMNAS
  'Dim NDV As Integer
  'Dim NDH As Integer
  
  'NDV = 1
  'NDH = 1
  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
  
  
  'RANGO DE VALORES
  Min = CoordX(0)
  Max = CoordX(UBound(CoordX(), 1))
      
  'PBox.Cls
  
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = Abs(Max - Min) / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = 1 / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = Min - xinc
    .ScaleTop = 1 + ydec
    .ScaleWidth = Abs(Max - Min) + xinc + xdec
    .ScaleHeight = -(1 + yinc + ydec)
  End With
       
  'CUADRCULA
  'xdiv = Abs(Max - Min) / NDV
  'ydiv = 1 / NDH
  
  'PBox.DrawWidth = Grosor
  'LINEAS HORIZONTALES
  'y = 0
  'For i = 1 To NDH + 1
  '  PBox.Line (Min, y)-(Max, y), vbWhite
  '  If NDH = 1 Then Exit For
  '  y = y + ydiv
  'Next i
  
  'LINEAS VERTICALES
  'x = Min
  'For i = 1 To NDV + 1
  '  PBox.Line (x, 0)-(x, 1), vbWhite
  '  If NDV = 1 Then Exit For
  '  x = x + xdiv
  'Next i
  
  MId = UBound(CoordX(), 1)
  
  PBox.DrawWidth = Grosor
  For i = 0 To MId - 1
    PBox.Line (CoordX(i), CoordY(i))-(CoordX(i + 1), CoordY(i + 1)), Color
  Next i
 
  If ConResaltePto Then
    PBox.DrawWidth = 10
    PBox.PSet (CSng(PtoX), CSng(PtoY)), Color
  End If
  
  PBox.DrawWidth = Grosor
End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: Dibujar_PtoVCrisp()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub Dibujar_PtoVCrisp(Min As Double, Max As Double, Valor As Double, Color As Long, Grosor As Integer, PBox As PictureBox)
 
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single
  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
      
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = Abs(Max - Min) / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = 1 / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = Min - xinc
    .ScaleTop = 1 + ydec
    .ScaleWidth = Abs(Max - Min) + xinc + xdec
    .ScaleHeight = -(1 + yinc + ydec)
  End With
       
  PBox.DrawWidth = Grosor
  For i = 0 To 6
    If i Mod 2 = 0 Then
      PBox.Line (Valor, i / 7)-(Valor, (i + 1) / 7), Color
    End If
  Next i
 
End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: Dibujar_Secuencia()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub Dibujar_Secuencia(Min As Double, Max As Double, CoordX() As Double, CoordY() As Double, Color As Long, Grosor As Integer, _
                             PBox As PictureBox, Optional ConResaltePto As Boolean = False, Optional PtoX As Double, Optional PtoY As Double)
 
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single

  'N DE FILAS Y COLUMNAS
  Dim NDV As Integer
  Dim NDH As Integer
  
  NDV = 1
  NDH = 1
  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 10
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
      
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = 1 / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = Abs(Max - Min) / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = -xinc
    .ScaleTop = Max + ydec
    .ScaleWidth = 1 + xinc + xdec
    .ScaleHeight = -(Abs(Max - Min) + yinc + ydec)
  End With

  'CUADRCULA
  xdiv = 1 / NDV
  ydiv = Abs(Max - Min) / NDH
  
  PBox.DrawWidth = Grosor
  'LINEAS HORIZONTALES
  y = Min
  For i = 1 To NDH + 1
    PBox.Line (0, y)-(1, y), vbWhite
    If NDH = 1 Then Exit For
    y = y + ydiv
  Next i
  
  'LINEAS VERTICALES
  x = 0
  For i = 1 To NDV + 1
    PBox.Line (x, Min)-(x, Max), vbWhite
    If NDV = 1 Then Exit For
    x = x + xdiv
  Next i
  
  PBox.DrawWidth = 8
  i = 0
  PBox.PSet (CSng(CoordX(i)), CSng(CoordY(i + 1))), Color
  PBox.DrawWidth = Grosor
  For i = 1 To UBound(CoordY(), 1) - 1
    PBox.DrawWidth = Grosor
    PBox.Line (CoordX(i - 1), CoordY(i))-(CoordX(i), CoordY(i + 1)), Color
    PBox.DrawWidth = 8
    PBox.PSet (CSng(CoordX(i)), CSng(CoordY(i + 1))), Color
  Next i
  
  If ConResaltePto Then
    PBox.DrawWidth = 10
    PBox.PSet (CSng(PtoX), CSng(PtoY)), vbRed
  End If
  
  PBox.DrawWidth = Grosor

End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: Dibujar_Secuencia_Seleccionada()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub Dibujar_Secuencia_Seleccionada(Min As Double, Max As Double, CoordX() As Double, CoordY() As Double, Color As Long, Grosor As Integer, _
                             PBox As PictureBox)
 
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single

  'N DE FILAS Y COLUMNAS
  Dim NDV As Integer
  Dim NDH As Integer
  
  NDV = 1
  NDH = 1
  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
      
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = 1 / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = Abs(Max - Min) / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = -xinc
    .ScaleTop = Max + ydec
    .ScaleWidth = 1 + xinc + xdec
    .ScaleHeight = -(Abs(Max - Min) + yinc + ydec)
  End With

  'CUADRCULA
  xdiv = 1 / NDV
  ydiv = Abs(Max - Min) / NDH
  
  PBox.DrawWidth = Grosor
  'LINEAS HORIZONTALES
  y = Min
  For i = 1 To NDH + 1
    PBox.Line (0, y)-(1, y), vbWhite
    If NDH = 1 Then Exit For
    y = y + ydiv
  Next i
  
  'LINEAS VERTICALES
  x = 0
  For i = 1 To NDV + 1
    PBox.Line (x, Min)-(x, Max), vbWhite
    If NDV = 1 Then Exit For
    x = x + xdiv
  Next i
  
  PBox.DrawWidth = 8
  i = 0
  PBox.PSet (CSng(CoordX(i)), CSng(CoordY(i + 1))), Color
  PBox.DrawWidth = Grosor
  For i = 1 To UBound(CoordY(), 1) - 1
    PBox.DrawWidth = Grosor
    PBox.Line (CoordX(i - 1), CoordY(i))-(CoordX(i), CoordY(i + 1)), Color
    PBox.DrawWidth = 8
    PBox.PSet (CSng(CoordX(i)), CSng(CoordY(i + 1))), Color
  Next i
  
  PBox.DrawWidth = Grosor

End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: Dibujar_Secuencia_SinPtos()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub Dibujar_Secuencia_SinPtos(Min As Double, Max As Double, CoordX() As Double, CoordY() As Double, Color As Long, Grosor As Integer, _
                             PBox As PictureBox)
 
  Dim i As Integer
  Static x, xinc, xdec, y, yinc, ydec As Single
  Dim xdiv, ydiv As Single
  Dim RelacionEscalaX, RelacionEscalaY As Single

  'N DE FILAS Y COLUMNAS
  Dim NDV As Integer
  Dim NDH As Integer
  
  NDV = 1
  NDH = 1
  
  'MRGENES DE SEPARACIN
  Const PorcentajeXinc = 5
  Const PorcentajeXdec = 5
  Const PorcentajeYinc = 12
  Const PorcentajeYdec = 18
      
  'ESCALADO
  xinc = (PBox.ScaleWidth * PorcentajeXinc) / 100
  xdec = (PBox.ScaleWidth * PorcentajeXdec) / 100
  yinc = (PBox.ScaleHeight * PorcentajeYinc) / 100
  ydec = (PBox.ScaleHeight * PorcentajeYdec) / 100

  RelacionEscalaX = 1 / (PBox.ScaleWidth - xinc - xdec)
  RelacionEscalaY = Abs(Max - Min) / (PBox.ScaleHeight - yinc - ydec)
  
  xinc = xinc * RelacionEscalaX
  xdec = xdec * RelacionEscalaX
  yinc = yinc * RelacionEscalaY
  ydec = ydec * RelacionEscalaY
  
  'PERSONALIZACIN DEL ESCALADO
  With PBox
    .ScaleLeft = -xinc
    .ScaleTop = Max + ydec
    .ScaleWidth = 1 + xinc + xdec
    .ScaleHeight = -(Abs(Max - Min) + yinc + ydec)
  End With

  'CUADRCULA
  xdiv = 1 / NDV
  ydiv = Abs(Max - Min) / NDH
  
  PBox.DrawWidth = Grosor
  'LINEAS HORIZONTALES
  y = Min
  For i = 1 To NDH + 1
    PBox.Line (0, y)-(1, y), vbWhite
    If NDH = 1 Then Exit For
    y = y + ydiv
  Next i
  
  'LINEAS VERTICALES
  x = 0
  For i = 1 To NDV + 1
    PBox.Line (x, Min)-(x, Max), vbWhite
    If NDV = 1 Then Exit For
    x = x + xdiv
  Next i
  
  i = 0
  PBox.DrawWidth = Grosor
  For i = 1 To UBound(CoordY(), 1) - 1
    PBox.Line (CoordX(i - 1), CoordY(i))-(CoordX(i), CoordY(i + 1)), Color
  Next i

End Sub

