Attribute VB_Name = "InOut"
'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: INOUT.BAS ##
'
' OBJETIVO: Gestionar los ndices, nmero de elementos y contenidos de las distintas
'           matrices de la aplicacin.
'-------------------------------------------------------------------------------------------------------

Option Explicit
'---------------------------------
' - DECLARACIN DE ENUMERACIONES -
'---------------------------------
'MBITO GLOBAL
Public Enum ETEOS
  E = 0
  S = 1
End Enum

Public Enum ETEOSOR
  Entrada = 0
  Salida = 1
  Regla = 2
End Enum

Public Enum ETDIFOCONDIF
  Difusor = 0
  ConjDif = 1
End Enum

'MBITO LOCAL AL MDULO
Public ModificarEoS As ETEOS

'**************************************************************************************************
'   PROPIEDAD: NVentanasEoS()
'**************************************************************************************************
Public Property Get NVentanasEoS(EoS As ETEOS) As Integer
  On Error Resume Next
  Dim I As Integer

  Select Case EoS
    Case Entrada:
      I = UBound(VentanaEntrada(), 1)
    Case Salida:
      I = UBound(VentanaSalida(), 1)
    End Select
  If Err.Number = 9 Then
    NVentanasEoS = 0
    Err.Clear
  Else
    NVentanasEoS = I + 1
  End If

End Property

'**************************************************************************************************
'   PROPIEDAD: NConjDif()
'**************************************************************************************************
Public Property Get NConjDif(EoS As ETEOS, IdEoS As Integer) As Integer
  
  On Error Resume Next
  Dim I As Integer
  Select Case EoS
    Case Entrada:
      I = UBound(SistActual.Entrada(IdEoS).ConjDif(), 1)
    Case Salida:
      I = UBound(SistActual.Salida(IdEoS).ConjDif(), 1)
  End Select
  If Err.Number = 9 Then
    NConjDif = 0
    Err.Clear
  Else
    NConjDif = I + 1
  End If

End Property

'**************************************************************************************************
'   PROPIEDAD: NVCrisps()
'**************************************************************************************************
Public Property Get NVCrisps(EoS As ETEOS, IdEoS As Integer) As Integer
  
  On Error Resume Next
  Dim I As Integer
  Select Case EoS
    Case Entrada:
      I = UBound(SistActual.Entrada(IdEoS).VCrisp(), 1)
    Case Salida:
      I = UBound(SistActual.Salida(IdEoS).VCrisp(), 1)
  End Select
  If Err.Number = 9 Then
    NVCrisps = 0
    Err.Clear
  Else
    NVCrisps = I + 1
  End If

End Property

'**************************************************************************************************
'   PROPIEDAD: QConjDif()
'**************************************************************************************************
Public Property Get QConjDif(EoS As ETEOS, IdEoS As Integer, IdConjDif As Integer) As String
  Dim Tipo As ETCONJDIF

  Select Case EoS
    Case E:
      Tipo = SistActual.Entrada(IdEoS).ConjDif(IdConjDif).Tipo
    Case S:
      Tipo = SistActual.Salida(IdEoS).ConjDif(IdConjDif).Tipo
  End Select
  Select Case Tipo
    Case Gamma:
      QConjDif = "Gamma"
    Case L:
      QConjDif = "L"
    Case Singleton:
      QConjDif = "Singleton"
    Case Intervalo:
      QConjDif = "Intervalo"
    Case Triangulo:
      QConjDif = "Triangulo"
    Case Trapecio:
      QConjDif = "Trapecio"
    Case TrapecioExtendido:
      QConjDif = "Trapecio Extendido"
  End Select

End Property

'**************************************************************************************************
'   PROPIEDAD: QNombreEoS()
'**************************************************************************************************
Public Property Get QNombreEoS(EoS As ETEOS, IdEoS As Integer) As String
  Dim Nombre As String
  If EoS = E Then
    Nombre = UCase$(Trim$(SistActual.Entrada(IdEoS).Nombre))
  ElseIf EoS = S Then
    Nombre = UCase$(Trim$(SistActual.Salida(IdEoS).Nombre))
  End If
  QNombreEoS = Nombre
End Property

'**************************************************************************************************
'   PROPIEDAD: QEtiqueta()
'**************************************************************************************************
Public Property Get QEtiqueta(EoS As ETEOS, IdEoS As Integer, IdConjDif As Integer) As String
  Dim Etiqueta As String
  If EoS = E Then
    Etiqueta = UCase$(Trim$(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).Etiqueta))
  ElseIf EoS = S Then
    Etiqueta = UCase$(Trim$(SistActual.Salida(IdEoS).ConjDif(IdConjDif).Etiqueta))
  End If
  QEtiqueta = Etiqueta
End Property
'**************************************************************************************************
'   PROPIEDAD: QUnivDiscurso()
'**************************************************************************************************
Public Property Get QUnivDiscurso(EoS As ETEOS, IdEoS As Integer) As String
  Dim Cadena As String
  Select Case EoS
    Case Entrada:
      Cadena = "[" & Round(SistActual.Entrada(IdEoS).UnivDiscurso(0) + EPSILON, NUMDEC) & "; " & Round(SistActual.Entrada(IdEoS).UnivDiscurso(1) - EPSILON, NUMDEC) & "]"
    Case Salida:
      Cadena = "[" & Round(SistActual.Salida(IdEoS).UnivDiscurso(0) + EPSILON, NUMDEC) & "; " & Round(SistActual.Salida(IdEoS).UnivDiscurso(1) - EPSILON, NUMDEC) & "]"
  End Select
  QUnivDiscurso = Cadena
  
End Property

'**************************************************************************************************
'   PROPIEDAD: QEstado()
'**************************************************************************************************
Public Property Get QEstado(EoSoR As ETEOSOR, IdEoSoR As Integer) As String
  Dim Cadena As String
  Const Act = "Activa"
  Const Sem = "Semi-Activa"
  Const Pas = "Pasiva"
  
  Select Case EoSoR
    Case Entrada:
      If SistActual.Entrada(IdEoSoR).Activa = Si Then
        Cadena = Act
      Else
        Cadena = Pas
      End If
    Case Salida:
      If SistActual.Salida(IdEoSoR).Activa = Si Then
        Cadena = Act
      Else
        Cadena = Pas
      End If
    Case Regla:
      If SistActual.Regla(IdEoSoR).Estado = Activo Then
        Cadena = Act
      ElseIf SistActual.Regla(IdEoSoR).Estado = Pasivo Then
        Cadena = Pas
      Else
        Cadena = Sem
      End If
  End Select
  QEstado = Cadena
  
End Property

'**************************************************************************************************
'   PROPIEDAD: EstaActivaEoS()
'**************************************************************************************************
Public Property Get EstaActivaEoS(EoS As ETEOS, IdEoS As Integer) As ETACTIVADO
  
  Select Case EoS
    Case E:
      EstaActivaEoS = SistActual.Entrada(IdEoS).Activa
    Case S:
      EstaActivaEoS = SistActual.Salida(IdEoS).Activa
  End Select
  
End Property

'**************************************************************************************************
'   PROPIEDAD: QIdEoS()
'**************************************************************************************************
Public Property Get QIdEoS(EoS As ETEOS, NombreEoS As String) As Integer
  Dim I As Integer
  
  Select Case EoS
    Case E:
      For I = 1 To NEntradas
        If UCase$(Trim$(SistActual.Entrada(I - 1).Nombre)) = UCase$(Trim$(NombreEoS)) Then
          QIdEoS = I - 1
          Exit Property
        End If
      Next I
    Case S:
      For I = 1 To NSalidas
        If UCase$(Trim$(SistActual.Salida(I - 1).Nombre)) = UCase$(Trim$(NombreEoS)) Then
          QIdEoS = I - 1
          Exit Property
        End If
      Next I
  End Select
  'Si se llega aqu es por que el NombreEoS no se encuentra
  QIdEoS = -1
End Property

'**************************************************************************************************
'   PROPIEDAD: QIdEtiqueta()
'**************************************************************************************************
Public Property Get QIdEtiqueta(EoS As ETEOS, IdEoS As Integer, Etiqueta As String) As Integer
  Dim I As Integer
  
  Select Case EoS
    Case E:
      For I = 1 To NConjDif(E, IdEoS)
        If UCase$(Trim$(SistActual.Entrada(IdEoS).ConjDif(I - 1).Etiqueta)) = UCase$(Trim$(Etiqueta)) Then
          QIdEtiqueta = I - 1
          Exit Property
        End If
      Next I
    Case S:
      For I = 1 To NConjDif(S, IdEoS)
        If UCase$(Trim$(SistActual.Salida(IdEoS).ConjDif(I - 1).Etiqueta)) = UCase$(Trim$(Etiqueta)) Then
          QIdEtiqueta = I - 1
          Exit Property
        End If
      Next I
  End Select
  'Si se llega aqu es por que el NombreEoS no se encuentra
  QIdEtiqueta = -1
End Property

'**************************************************************************************************
'   PROPIEDAD: EstaYaNombreEoS()
'**************************************************************************************************
Public Property Get EstaYaNombreEoS(EoS As ETEOS, Nombre As String, Optional MsgAdv As Boolean = True) As Boolean
  Dim I As Integer
  
  Select Case EoS
    Case Entrada:
        For I = 0 To NEntradas - 1
          If UCase$(Trim$(SistActual.Entrada(I).Nombre)) = UCase$(Trim$(Nombre)) Then
            If MsgAdv Then
              MsgBox "El nombre de la entrada <" & Nombre & "> ya existe" & vbCr & vbLf & "en E[" & I & _
                     "], por favor, modifquelo por otro distinto.", vbExclamation, "Atencin! - Nombre repetido"
            End If
            EstaYaNombreEoS = True
            Exit Property
          End If
        Next I
    Case Salida:
        For I = 0 To NSalidas - 1
          If UCase$(Trim$(SistActual.Salida(I).Nombre)) = UCase$(Trim$(Nombre)) Then
            If MsgAdv Then
              MsgBox "El nombre de la salida <" & Nombre & "> ya existe" & vbCr & vbLf & "en S[" & I & _
                     "], por favor, modifquelo por otro distinto.", vbExclamation, "Atencin! - Nombre repetido"
            End If
            EstaYaNombreEoS = True
            Exit Property
          End If
        Next I
  End Select
  EstaYaNombreEoS = False
  
End Property

'**************************************************************************************************
'   PROPIEDAD: EstaConjDif()
'**************************************************************************************************
Public Property Get EstaConjDif(EoS As ETEOS, IdEoS As Integer, Etiqueta As String) As Boolean
  Dim I As Integer
  
  Select Case EoS
    Case Entrada:
      For I = 0 To NConjDif(E, IdEoS) - 1
        If UCase$(Trim$(SistActual.Entrada(IdEoS).ConjDif(I).Etiqueta)) = UCase$(Trim$(Etiqueta)) Then
          MsgBox "La etiqueta <" & Etiqueta & "> ya existe en la actual entrada, por favor" & vbCr & vbLf & "modifquela por otra distinta", vbExclamation, "Atencin!"
          EstaConjDif = True
          Exit Property
        End If
      Next I
    Case Salida:
      For I = 0 To NConjDif(S, IdEoS) - 1
        If UCase$(Trim$(SistActual.Salida(IdEoS).ConjDif(I).Etiqueta)) = UCase$(Trim$(Etiqueta)) Then
          MsgBox "La etiqueta <" & Etiqueta & "> ya existe en en la actual salida, por favor" & vbCr & vbLf & "modifquela por otra distinta", vbExclamation, "Atencin!"
          EstaConjDif = True
          Exit Property
        End If
      Next I
  End Select
  EstaConjDif = False

End Property


'**************************************************************************************************
'   PROCEDIMIENTO: EliminarConjDifMatriz()
'**************************************************************************************************
Public Sub EliminarConjDifMatriz(Matriz() As TCONJDIFUSO, Posicion As Integer, ConjDifSuprimido As TCONJDIFUSO)
  On Error Resume Next
  Dim I As Integer, NElementosMatriz As Integer
  'Se calcula el nmero de elementos de la matriz
  I = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = I + 1
  End If
  'Antes de proceder a su eliminacin este se almacena como informacin a reconstruir en caso de querer
  'el usuario deshacer la eliminacin.
  ConjDifSuprimido = Matriz(Posicion)
  'Dado que la matriz comienza en el elemento Matriz(0) el ndice ira de 0 -> (NElementosMatriz -1)
  'SI LA MATRIZ CONTIENE UN SOLO ELEMENTO
  If NElementosMatriz = 1 Then
    'Se liberar memoria ocupada por la matriz dinmica Matriz()
    Erase Matriz()
  'SI LA MATRIZ CONTIENE MS DE UN ELEMENTO Y EL SELECCIONADO PARA ELIMINAR ES EL LTIMO
  ElseIf (Posicion = (NElementosMatriz - 1)) And NElementosMatriz > 1 Then
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  Else
    'Trasladamos los datos de la posicin i a i-1
    For I = Posicion + 1 To (NElementosMatriz - 1)
      Matriz(I - 1) = Matriz(I)
    Next I
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  End If
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: AadirCDifuso()
'**************************************************************************************************
Public Sub AadirCDifuso(Matriz() As TCONJDIFUSO, CDifusoInsertar As TCONJDIFUSO)
  On Error GoTo Fallo
  Dim I As Integer, NEMatriz As Integer
  
  'Se calcula el nmero de elementos de la matriz
  I = UBound(Matriz(), 1)
  NEMatriz = I + 1
  
Seguir:
  If NEMatriz = 0 Then
    ReDim Matriz(0)
    Matriz(0) = CDifusoInsertar
  ElseIf NEMatriz > 0 Then
    ReDim Preserve Matriz(UBound(Matriz(), 1) + 1)
    Matriz(UBound(Matriz(), 1)) = CDifusoInsertar
  End If
  Exit Sub
  
Fallo:
  If Err.Number = 9 Then
    NEMatriz = 0
    Err.Clear
    GoTo Seguir
  End If
End Sub

'**************************************************************************************************
'   PROPIEDAD: QUndSigla()
'**************************************************************************************************
Public Property Get QUndSigla(EoS As ETEOS, IdEoS As Integer) As String
  Dim SIGLA As String
  If EoS = E Then
    SIGLA = Trim$(SistActual.Entrada(IdEoS).UndSigla)
    If SIGLA = "" Then
      QUndSigla = ""
      Exit Property
    End If
    QUndSigla = "[" & SIGLA & "]"
  ElseIf EoS = S Then
    SIGLA = Trim$(SistActual.Salida(IdEoS).UndSigla)
    If SIGLA = "" Then
      QUndSigla = ""
      Exit Property
    End If
    QUndSigla = "[" & SIGLA & "]"
  End If

End Property

'**********************************************************
'   FUNCTION: RecalcularFIncremento()
'**********************************************************
Public Function RecalcularFIncremento(LimSuperior As Double, LimInferior As Double) As Double
  Dim Valor As Double
  
  If (LimSuperior - LimInferior) <= 1 Then
    Valor = 0.1
  ElseIf (LimSuperior - LimInferior) > 1 And (LimSuperior - LimInferior) <= 5 Then
    Valor = 0.5
  ElseIf (LimSuperior - LimInferior) > 5 And (LimSuperior - LimInferior) <= 10 Then
    Valor = 1
  ElseIf (LimSuperior - LimInferior) > 10 And (LimSuperior - LimInferior) <= 50 Then
    Valor = 2
  ElseIf (LimSuperior - LimInferior) > 50 And (LimSuperior - LimInferior) <= 100 Then
    Valor = 5
  Else
    Valor = 10
  End If
  RecalcularFIncremento = Valor
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: ModificarVCrispsUniversoModificadoMin()
'**************************************************************************************************
Public Sub ModificarVCrispsUniversoModificadoMin(EoS As ETEOS, IdEoS As Integer, NuevoMin As Double)
  Dim I As Integer
  
  'Modificacin Valores Crisps
    For I = 0 To UBound(SistActual.Entrada(IdEoS).VCrisp(), 1)
      SistActual.Entrada(IdEoS).VCrisp(I) = (SistActual.Entrada(IdEoS).VCrisp(I) * (NuevoMin - SistActual.Entrada(IdEoS).UnivDiscurso(1)) + SistActual.Entrada(IdEoS).UnivDiscurso(1) * (SistActual.Entrada(IdEoS).UnivDiscurso(0) - NuevoMin)) / _
                                         (SistActual.Entrada(IdEoS).UnivDiscurso(0) - SistActual.Entrada(IdEoS).UnivDiscurso(1))
    Next I
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: ModificarPtosUniversoModificadoMin()
'**************************************************************************************************
Public Sub ModificarPtosUniversoModificadoMin(EoS As ETEOS, IdEoS As Integer, NuevoMin As Double)
  Dim I As Integer, j As Integer
  
  If EoS = E Then
    For I = 0 To NConjDif(E, IdEoS) - 1
      'Modificacin P.Bsicos
      For j = 0 To 3
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(j) = (SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(j) * (NuevoMin - SistActual.Entrada(IdEoS).UnivDiscurso(1)) + SistActual.Entrada(IdEoS).UnivDiscurso(1) * (SistActual.Entrada(IdEoS).UnivDiscurso(0) - NuevoMin)) / _
                                         (SistActual.Entrada(IdEoS).UnivDiscurso(0) - SistActual.Entrada(IdEoS).UnivDiscurso(1))
      Next j
      'Modificacin P.Extendidos
      For j = 0 To NPExtendidos(E, IdEoS, ConjDif, I) - 1
        SistActual.Entrada(IdEoS).ConjDif(I).PExtendidos(2 * j) = (SistActual.Entrada(IdEoS).ConjDif(I).PExtendidos(2 * j) * (NuevoMin - SistActual.Entrada(IdEoS).UnivDiscurso(1)) + SistActual.Entrada(IdEoS).UnivDiscurso(1) * (SistActual.Entrada(IdEoS).UnivDiscurso(0) - NuevoMin)) / _
                                         (SistActual.Entrada(IdEoS).UnivDiscurso(0) - SistActual.Entrada(IdEoS).UnivDiscurso(1))
      Next j
    Next I
  Else
    For I = 0 To NConjDif(S, IdEoS) - 1
      'Modificacin P.Bsicos
      For j = 0 To 3
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(j) = (SistActual.Salida(IdEoS).ConjDif(I).PBasicos(j) * (NuevoMin - SistActual.Salida(IdEoS).UnivDiscurso(1)) + SistActual.Salida(IdEoS).UnivDiscurso(1) * (SistActual.Salida(IdEoS).UnivDiscurso(0) - NuevoMin)) / _
                                         (SistActual.Salida(IdEoS).UnivDiscurso(0) - SistActual.Salida(IdEoS).UnivDiscurso(1))
      Next j
      'Modificacin P.Extendidos
      For j = 0 To NPExtendidos(E, IdEoS, ConjDif, I) - 1
        SistActual.Salida(IdEoS).ConjDif(I).PExtendidos(2 * j) = (SistActual.Salida(IdEoS).ConjDif(I).PExtendidos(2 * j) * (NuevoMin - SistActual.Salida(IdEoS).UnivDiscurso(1)) + SistActual.Salida(IdEoS).UnivDiscurso(1) * (SistActual.Salida(IdEoS).UnivDiscurso(0) - NuevoMin)) / _
                                         (SistActual.Salida(IdEoS).UnivDiscurso(0) - SistActual.Salida(IdEoS).UnivDiscurso(1))
      Next j
    Next I
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: ModificarVCrispsUniversoModificadoMax()
'**************************************************************************************************
Public Sub ModificarVCrispsUniversoModificadoMax(EoS As ETEOS, IdEoS As Integer, NuevoMax As Double)
  Dim I As Integer
  
  'Modificacin Valores Crisps
    For I = 0 To UBound(SistActual.Entrada(IdEoS).VCrisp(), 1)
      SistActual.Entrada(IdEoS).VCrisp(I) = (SistActual.Entrada(IdEoS).VCrisp(I) * (SistActual.Entrada(IdEoS).UnivDiscurso(0) - NuevoMax) + SistActual.Entrada(IdEoS).UnivDiscurso(0) * (NuevoMax - SistActual.Entrada(IdEoS).UnivDiscurso(1))) / _
                                         (SistActual.Entrada(IdEoS).UnivDiscurso(0) - SistActual.Entrada(IdEoS).UnivDiscurso(1))
    Next I
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: ModificarPtosUniversoModificadoMax()
'**************************************************************************************************
Public Sub ModificarPtosUniversoModificadoMax(EoS As ETEOS, IdEoS As Integer, NuevoMax As Double)
  Dim I As Integer, j As Integer
  
  If EoS = E Then
    For I = 0 To NConjDif(E, IdEoS) - 1
      For j = 0 To 3
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(j) = (SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(j) * (SistActual.Entrada(IdEoS).UnivDiscurso(0) - NuevoMax) + SistActual.Entrada(IdEoS).UnivDiscurso(0) * (NuevoMax - SistActual.Entrada(IdEoS).UnivDiscurso(1))) / _
                                         (SistActual.Entrada(IdEoS).UnivDiscurso(0) - SistActual.Entrada(IdEoS).UnivDiscurso(1))
      Next j
      'Modificacin P.Extendidos
      For j = 0 To NPExtendidos(E, IdEoS, ConjDif, I) - 1
        SistActual.Entrada(IdEoS).ConjDif(I).PExtendidos(2 * j) = (SistActual.Entrada(IdEoS).ConjDif(I).PExtendidos(2 * j) * (SistActual.Entrada(IdEoS).UnivDiscurso(0) - NuevoMax) + SistActual.Entrada(IdEoS).UnivDiscurso(0) * (NuevoMax - SistActual.Entrada(IdEoS).UnivDiscurso(1))) / _
                                         (SistActual.Entrada(IdEoS).UnivDiscurso(0) - SistActual.Entrada(IdEoS).UnivDiscurso(1))
      Next j
    Next I
  Else
    For I = 0 To NConjDif(S, IdEoS) - 1
      For j = 0 To 3
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(j) = (SistActual.Salida(IdEoS).ConjDif(I).PBasicos(j) * (SistActual.Salida(IdEoS).UnivDiscurso(0) - NuevoMax) + SistActual.Salida(IdEoS).UnivDiscurso(0) * (NuevoMax - SistActual.Salida(IdEoS).UnivDiscurso(1))) / _
                                         (SistActual.Salida(IdEoS).UnivDiscurso(0) - SistActual.Salida(IdEoS).UnivDiscurso(1))
      Next j
      'Modificacin P.Extendidos
      For j = 0 To NPExtendidos(E, IdEoS, ConjDif, I) - 1
        SistActual.Salida(IdEoS).ConjDif(I).PExtendidos(2 * j) = (SistActual.Salida(IdEoS).ConjDif(I).PExtendidos(2 * j) * (SistActual.Salida(IdEoS).UnivDiscurso(0) - NuevoMax) + SistActual.Salida(IdEoS).UnivDiscurso(0) * (NuevoMax - SistActual.Salida(IdEoS).UnivDiscurso(1))) / _
                                         (SistActual.Salida(IdEoS).UnivDiscurso(0) - SistActual.Salida(IdEoS).UnivDiscurso(1))
      Next j
    Next I
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DisminuyeMinUD()
'**************************************************************************************************
Public Sub DisminuyeMinUD(EoS As ETEOS, IdEoS As Integer, NuevoMin As Double)
  Dim I As Integer
  
  If EoS = E Then
    For I = 0 To NConjDif(E, IdEoS) - 1
      If SistActual.Entrada(IdEoS).ConjDif(I).Tipo = L Then
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(0) = NuevoMin
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(1) = NuevoMin
      End If
    Next I
  Else
    For I = 0 To NConjDif(S, IdEoS) - 1
      If SistActual.Salida(IdEoS).ConjDif(I).Tipo = L Then
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(0) = NuevoMin
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(1) = NuevoMin
      End If
    Next I
  End If

End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: AumentaMinUD()
'**************************************************************************************************
Public Sub AumentaMinUD(EoS As ETEOS, IdEoS As Integer, NuevoMin As Double, AnuladaVariacion As Boolean)
  Dim I As Integer
  Dim Opcion As Long
  AnuladaVariacion = False
  Dim ConjBorrado As TCONJDIFUSO
  
  If EoS = E Then
VEMPEZAR_E:
    For I = 0 To NConjDif(E, IdEoS) - 1
      'Si se encuentra un C.Difuso L y el nuevo mnimo todavia no corta el segmento A-B este se mantendr
      If SistActual.Entrada(IdEoS).ConjDif(I).Tipo = L And NuevoMin < SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(2) Then
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(0) = NuevoMin
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(1) = NuevoMin
      Else
        'Si el nuevo mnimo corta algn C.Difuso se procede a su eliminacin si as lo desea el usuario
        If NuevoMin > SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(0) Then
          Opcion = MsgBox("Por variacin del U.Discurso el C.Difuso: " & QEtiqueta(E, IdEoS, I) & vbCr & vbLf & _
                          "queda fuera de los lmites del mismo. Desea proceder a su eliminacin?" & vbCr & vbLf & _
                          "Cancelar implica que el U.Discurso volver a su posicin inicial", vbExclamation + vbOKCancel, "Variacin U.Discurso")
          If Opcion = vbCancel Then
            AnuladaVariacion = True
            Exit Sub
          End If
          FrmEntradaModificar.NImplicacion = 2
          ModificarEoS = E
          FrmMsgImplicaciones.Show vbModal, FrmMdi
          If FrmEntradaModificar.OpcionImplicacion = vbOK Then
            EliminarTerminosQTenganConjDif E, IdEoS, I
            EliminarConjDifMatriz SistActual.Entrada(IdEoS).ConjDif(), I, ConjBorrado
            GoTo VEMPEZAR_E
          Else
            AnuladaVariacion = True
            Exit Sub
          End If
        End If
      End If
    Next I
  Else
VEMPEZAR_S:
    For I = 0 To NConjDif(S, IdEoS) - 1
      'Si se encuentra un C.Difuso L y el nuevo mnimo todavia no corta el segmento A-B este se mantendr
      If SistActual.Salida(IdEoS).ConjDif(I).Tipo = L And NuevoMin < SistActual.Salida(IdEoS).ConjDif(I).PBasicos(2) Then
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(0) = NuevoMin
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(1) = NuevoMin
      Else
        'Si el nuevo mnimo corta algn C.Difuso se procede a su eliminacin si as lo desea el usuario
        If NuevoMin > SistActual.Salida(IdEoS).ConjDif(I).PBasicos(0) Then
          Opcion = MsgBox("Por variacin del U.Discurso el C.Difuso: " & QEtiqueta(S, IdEoS, I) & vbCr & vbLf & _
                          "queda fuera de los lmites del mismo. Desea proceder a su eliminacin?" & vbCr & vbLf & _
                          "Cancelar implica que el U.Discurso volver a su posicin inicial", vbExclamation + vbOKCancel, "Variacin U.Discurso")
          If Opcion = vbCancel Then
            AnuladaVariacion = True
            Exit Sub
          End If
          FrmEntradaModificar.NImplicacion = 2
          ModificarEoS = S
          FrmMsgImplicaciones.Show vbModal, FrmMdi
          If FrmEntradaModificar.OpcionImplicacion = vbOK Then
            EliminarTerminosQTenganConjDif S, IdEoS, I
            EliminarConjDifMatriz SistActual.Salida(IdEoS).ConjDif(), I, ConjBorrado
            GoTo VEMPEZAR_S
          Else
            AnuladaVariacion = True
            Exit Sub
          End If
        End If
      End If
    Next I
  End If

End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DisminuyeMaxUD()
'**************************************************************************************************
Public Sub DisminuyeMaxUD(EoS As ETEOS, IdEoS As Integer, NuevoMax As Double, AnuladaVariacion As Boolean)
  Dim I As Integer
  Dim Opcion As Long
  AnuladaVariacion = False
  Dim ConjBorrado As TCONJDIFUSO
  
  If EoS = E Then
    For I = 0 To NConjDif(E, IdEoS) - 1
      'Si se encuentra un C.Difuso Gamma y el nuevo mximo todavia no corta el segmento A-B este se mantendr
      If SistActual.Entrada(IdEoS).ConjDif(I).Tipo = Gamma And SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(1) < NuevoMax Then
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(2) = NuevoMax
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(3) = NuevoMax
      Else
        'Si el nuevo mnimo corta algn C.Difuso se procede a su eliminacin si as lo desea el usuario
        If NuevoMax < SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(3) Then
          Opcion = MsgBox("Por variacin del U.Discurso el C.Difuso: " & QEtiqueta(E, IdEoS, I) & vbCr & vbLf & _
                          "queda fuera de los lmites del mismo. Desea proceder a su eliminacin?" & vbCr & vbLf & _
                          "Cancelar implica que el U.Discurso volver a su posicin inicial", vbExclamation + vbOKCancel, "Variacin U.Discurso")
          If Opcion = vbCancel Then
            AnuladaVariacion = True
            Exit Sub
          End If
          FrmEntradaModificar.NImplicacion = 2
          ModificarEoS = E
          FrmMsgImplicaciones.Show vbModal, FrmMdi
          If FrmEntradaModificar.OpcionImplicacion = vbOK Then
            EliminarTerminosQTenganConjDif E, IdEoS, I
            EliminarConjDifMatriz SistActual.Entrada(IdEoS).ConjDif(), I, ConjBorrado
          Else
            AnuladaVariacion = True
            Exit Sub
          End If
        End If
      End If
    Next I
  Else
    For I = 0 To NConjDif(S, IdEoS) - 1
      'Si se encuentra un C.Difuso L y el nuevo mnimo todavia no corta al segmento A-B este se mantendr
      If SistActual.Salida(IdEoS).ConjDif(I).Tipo = Gamma And SistActual.Salida(IdEoS).ConjDif(I).PBasicos(1) < NuevoMax Then
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(2) = NuevoMax
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(3) = NuevoMax
      Else
        'Si el nuevo mnimo corta algn C.Difuso se procede a su eliminacin si as lo desea el usuario
        If NuevoMax < SistActual.Salida(IdEoS).ConjDif(I).PBasicos(3) Then
          Opcion = MsgBox("Por variacin del U.Discurso el C.Difuso: " & QEtiqueta(S, IdEoS, I) & vbCr & vbLf & _
                          "queda fuera de los lmites del mismo. Desea proceder a su eliminacin?" & vbCr & vbLf & _
                          "Cancelar implica que el U.Discurso volver a su posicin inicial", vbExclamation + vbOKCancel, "Variacin U.Discurso")
          If Opcion = vbCancel Then
            AnuladaVariacion = True
            Exit Sub
          End If
          FrmEntradaModificar.NImplicacion = 2
          ModificarEoS = S
          FrmMsgImplicaciones.Show vbModal, FrmMdi
          If FrmEntradaModificar.OpcionImplicacion = vbOK Then
            EliminarTerminosQTenganConjDif S, IdEoS, I
            EliminarConjDifMatriz SistActual.Salida(IdEoS).ConjDif(), I, ConjBorrado
          Else
            AnuladaVariacion = True
            Exit Sub
          End If
        End If
      End If
    Next I
  End If

End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: AumentaMaxUD()
'**************************************************************************************************
Public Sub AumentaMaxUD(EoS As ETEOS, IdEoS As Integer, NuevoMax As Double)
  Dim I As Integer
  
  If EoS = E Then
    For I = 0 To NConjDif(E, IdEoS) - 1
      If SistActual.Entrada(IdEoS).ConjDif(I).Tipo = Gamma Then
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(2) = NuevoMax
        SistActual.Entrada(IdEoS).ConjDif(I).PBasicos(3) = NuevoMax
      End If
    Next I
  Else
    For I = 0 To NConjDif(S, IdEoS) - 1
      If SistActual.Salida(IdEoS).ConjDif(I).Tipo = Gamma Then
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(2) = NuevoMax
        SistActual.Salida(IdEoS).ConjDif(I).PBasicos(3) = NuevoMax
      End If
    Next I
  End If

End Sub

