Attribute VB_Name = "InferEngine"
Option Explicit

'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: INFERENGINE ##
'
' OBJETIVO: Contenener los procedimientos, funciones y propiedades necesarias para realizar el
'           proceso de inferencia por el sistema
'-------------------------------------------------------------------------------------------------------



'**************************************************************************************************
'   PROPIEDAD: QNorma
'**************************************************************************************************
Public Property Get QNorma(TipoNorma As ET_TNORMAS_SNORMAS) As String
  
  Select Case TipoNorma
    'T-NORMAS
    Case TNorma_Minino:
      QNorma = "Minimo"
    
    Case TNorma_Producto_Algebraico:
      QNorma = "Producto Algebraico"
      
    Case TNorma_Producto_Drastico:
      QNorma = "Producto Drastico"
      
    Case TNorma_Producto_Acotado:
      QNorma = "Producto Acotado"
      
    Case TNorma_Producto_Hamacher:
      QNorma = "Producto Hamacher"
    
    Case TNorma_Familia_Yager:
      QNorma = "T-Familia Yager"
      
    Case TNorma_Familia_DuboisPrade:
      QNorma = "T-Familia Dubois Prade"
    
    Case TNorma_Familia_Frank:
      QNorma = "T-Familia Frank"
      
    Case TNorma_Producto_Einstein:
      QNorma = "Producto Einstein"
      
    'S-NORMAS
    Case SNorma_Maximo:
      QNorma = "Maximo"
    
    Case SNorma_Suma_Producto:
      QNorma = "Suma Producto"
      
    Case SNorma_Suma_Acotada:
      QNorma = "Suma Acotada"
      
    Case SNorma_Suma_Drastica:
      QNorma = "Suma Drastica"
      
    Case SNorma_Familia_Sugeno:
      QNorma = "S-Familia Sugeno"
      
    Case SNorma_Familia_Yager:
      QNorma = "S-Familia Yager"
    
    Case SNorma_Familia_DuboisPrade:
      QNorma = "S-Familia Dubois Prade"
    
    Case SNorma_Familia_Frank:
      QNorma = "S-Familia Frank"

  End Select

End Property

'**************************************************************************************************
'   PROPIEDAD: QImplicacion
'**************************************************************************************************
Public Property Get QImplicacion(TipoImplicacion As ET_IMPLICACIONES) As String
  
  Select Case TipoImplicacion
   
   'R-IMPLICACIONES
   Case Implicacion_R_Gaines:
      QImplicacion = "R-Gaines"
      
   Case Implicacion_R_Godel:
      QImplicacion = "R-Godel"
      
   Case Implicacion_R_Goguen:
      QImplicacion = "R-Goguen"
   
   'S-IMPLICACIONES
   Case Implicacion_S_Diene:
      QImplicacion = "S-Diene"
      
   Case Implicacion_S_DuboisPrade:
      QImplicacion = "S-DuboisPrade"
      
   Case Implicacion_S_Mizumoto:
      QImplicacion = "S-Mizumoto"
   
   'RS-IMPLICACIONES
   Case Implicacion_RS_Lukasiewicz:
      QImplicacion = "RS-Lukasiewicz"
      
   Case Implicacion_QM_EarlyZadeh:
      QImplicacion = "QM-EarlyZadeh"
   
   'TNORMAS COMO IMPLICACIONES
   Case Implicacion_T_Minimo:
      QImplicacion = "T-Minimo"
      
   Case Implicacion_T_ProductoAcotado:
      QImplicacion = "T-Producto Acotado"
      
   Case Implicacion_T_ProductoAlgebraico:
      QImplicacion = "T-Producto Algebraico"
      
  End Select

End Property


'**************************************************************************************************
'   FUNCIN: Min()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Function Min(x As Double, y As Double) As Double
  If x <= y Then
    Min = x
  Else
    Min = y
  End If
  
End Function

'**************************************************************************************************
'   FUNCIN: Max()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Function Max(x As Double, y As Double) As Double
  If y > x Then
    Max = y
  Else
    Max = x
  End If
  
End Function

'--------------------------------------------------------------
' FASE: ACTIVACIN REGLAS (AND/OR)
'--------------------------------------------------------------

'**************************************************************************************************
'   FUNCIN: TyS_NORMAS()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Function TyS_NORMAS(x As Double, y As Double, Norma As TSNorma) As Double
  Dim p As Double, Temp As Double
  
  p = Norma.p
  Select Case Norma.TipoNorma
  
    '------------
    '% T-NORMAS %
    '------------
    'T-NORMA: MNIMO
    Case TNorma_Minino:
      TyS_NORMAS = Min(x, y)
      
    'T-NORMA: PRODUCTO (ALGEBRAICO)
    Case TNorma_Producto_Algebraico:
      TyS_NORMAS = x * y
      
    'T-NORMA: PRODUCTO DRASTICO
    Case TNorma_Producto_Drastico:
      If y = 1# Then
        Temp = x
      ElseIf x = 1# Then
        Temp = y
      Else
        Temp = 0#
      End If
      TyS_NORMAS = Temp
      
    'T-NORMA: PRODUCTO ACOTADO
    Case TNorma_Producto_Acotado:
      TyS_NORMAS = Max(0, (1 + p) * (x + y - 1) - p * x * y)
      
    'T-NORMA: PRODUCTO DE HAMMACHER
    Case TNorma_Producto_Hamacher:
      If x * y = 0# Then
        TyS_NORMAS = 0#
      Else
        TyS_NORMAS = (x * y) / (p + (1 - p) * (x + y - x * y))
      End If
      
    'T-NORMA FAMILIA YAGER:
    Case TNorma_Familia_Yager:
      TyS_NORMAS = 1 - Min(1, ((1 - x) ^ p + (1 - y) ^ p) ^ (1 / p))
      
    'T-NORMA FAMILIA DUBOIS PRADE:
    Case TNorma_Familia_DuboisPrade:
      If x * y = 0# Then
        TyS_NORMAS = 0#
      Else
        Temp = Max(x, y)
        TyS_NORMAS = (x * y) / Max(Temp, p)
      End If
      
    'T-NORMA FAMILIA FRANK:
    Case TNorma_Familia_Frank:
      TyS_NORMAS = Log(1 + ((p ^ x - 1) * (p ^ y - 1)) / (p - 1)) / Log(p)
      
    'T-NORMA PRODUCTO EINSTEIN:
    Case TNorma_Producto_Einstein:
      If x * y = 0# Then
        TyS_NORMAS = 0#
      Else
        TyS_NORMAS = (x * y) / (1 + (1 - x) + (1 - y))
      End If
      
    '-------------------------
    '% T-CONORMAS o S-NORMAS %
    '-------------------------
    'S-NORMA: SUMA
    Case SNorma_Maximo:
      TyS_NORMAS = Max(x, y)
      
    'S-NORMA: SUMA PRODUCTO
    Case SNorma_Suma_Producto:
      TyS_NORMAS = x + y - x * y
      
    'S-NORMA: SUMA DRSTICA
    Case SNorma_Suma_Drastica:
      If y = 0# Then
        Temp = x
      ElseIf x = 0# Then
        Temp = y
      Else
        Temp = 1#
      End If
      TyS_NORMAS = Temp
      
    'S-NORMA: SUMA ACOTADA
    Case SNorma_Suma_Acotada:
      TyS_NORMAS = Min(1, x + y + p * x * y)
    
    'S-NORMA FAMILIA SUGENO:
    Case SNorma_Familia_Sugeno:
      TyS_NORMAS = Min(1, x + y + p - x * y)
    
    'S-NORMA FAMILIA YAGER:
    Case SNorma_Familia_Yager:
      TyS_NORMAS = Min(1, (x ^ p + y ^ p) ^ (1 / p))
    
    'S-NORMA FAMILIA DUBOIS PRADE:
    Case SNorma_Familia_DuboisPrade:
      Temp = Max(1 - x, 1 - y)
      TyS_NORMAS = ((1 - x) * (1 - y)) / Max(Temp, p)
    
    'S-NORMA FAMILIA FRANK:
    Case SNorma_Familia_Frank:
      TyS_NORMAS = Log(1 + ((p ^ (1 - x) - 1) * (p ^ (1 - y) - 1)) / (p - 1)) / Log(p)
    
  End Select
      
End Function

'**************************************************************************************************
'   FUNCIN: PossTermino()
'
'       OBJETIVO: Obtener la posibilidad (Poss) asociada al C.Difuso de una entrada del sistema
'                 el cual se encuentra almacenado en el campo Poss de dicho conjunto y cuyo
'                 ndice ser igual al identificador del conjunto difuso (etiqueta).
'
'**************************************************************************************************
Private Function PossTermino(IdRegla As Integer, IdAntecedente As Integer, Nombre As String, Etiqueta As String) As Double
  Dim I As Integer, j As Integer
  'Se recorren las entradas del sistema buscando por su campo nombre
  For I = 0 To NEntradas - 1
    If UCase$(Trim$(SistActual.Entrada(I).Nombre)) = UCase$(Trim$(Nombre)) Then
      'Una vez encontrada a su vez se busca el campo etiqueta de dicha entrada y una vez encontrado se obtienen
      'los ndices que permiten devolver la posibilidad asociada a dicha etiqueta.
      For j = 0 To NConjDif(E, I) - 1
        If UCase$(Trim$(SistActual.Entrada(I).ConjDif(j).Etiqueta)) = UCase$(Trim$(Etiqueta)) Then GoTo ENCONTRADO
      Next j
    End If
  Next I
  PossTermino = 0#
  Exit Function
ENCONTRADO:
  If SistActual.Regla(IdRegla).Antecedente(IdAntecedente).OpNot = True Then
    PossTermino = 1 - SistActual.Entrada(I).ConjDif(j).Poss
  Else
    PossTermino = SistActual.Entrada(I).ConjDif(j).Poss
  End If
  
End Function

'**************************************************************************************************
'   FUNCIN: GACTIVACION_REGLAS()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub GACTIVACION_REGLAS(Optional AllReglas As Boolean = True, Optional IdRegla As Integer)
  Dim I As Integer, j As Integer, k As Integer
  Dim TempGA As Double, TempPoss As Double
  
  If Not AllReglas Then
    'SLO SI LA REGLA EST ACTIVA O SEMI-ACTIVA
    If SistActual.Regla(IdRegla).Estado = Activo Or SistActual.Regla(IdRegla).Estado = SemiActivo Then
      'SE BUSCA EL PRIMER TRMINO ACTIVO Y SE ALMACENA SU POSIBILIDAD (POSS)
      For I = 0 To NAntecedentes(IdRegla) - 1
        If SistActual.Regla(IdRegla).Antecedente(I).Activa = Si Then
          TempGA = PossTermino(IdRegla, I, SistActual.Regla(IdRegla).Antecedente(I).NombreEoS, _
                               SistActual.Regla(IdRegla).Antecedente(I).EtiquetaConjDif)
          j = I
          Exit For
        End If
      Next I
      If NAntecedentes(IdRegla) = 1 Then
        SistActual.Regla(IdRegla).GACTIVACION = TempGA
        Exit Sub
      End If
      'A PARTIR DEL PRIMER TRMINO SE RECORREN LOS RESTANTES
      For I = j To NAntecedentes(IdRegla) - 1
        If SistActual.Regla(IdRegla).Antecedente(I).Activa = Si Then
          TempPoss = PossTermino(IdRegla, I, SistActual.Regla(IdRegla).Antecedente(I).NombreEoS, _
                                 SistActual.Regla(IdRegla).Antecedente(I).EtiquetaConjDif)
          'EN FUNCIN DEL OPERADOR CONECTIVO SE VA FORMANDO DOS A DOS
          'GRADO DE ACTIVACION DE LA REGLA
          If SistActual.Regla(IdRegla).Operador = AAnd Then
            TempGA = TyS_NORMAS(TempGA, TempPoss, SistActual.Inferencia.OpAnd)
          Else
            TempGA = TyS_NORMAS(TempGA, TempPoss, SistActual.Inferencia.OpOr)
          End If
          
        End If
      Next I
    End If
    SistActual.Regla(IdRegla).GACTIVACION = TempGA
  
  Else
    For k = 0 To NReglas - 1
      'SLO SI LA REGLA EST ACTIVA O SEMI-ACTIVA
      If SistActual.Regla(k).Estado = Activo Or SistActual.Regla(k).Estado = SemiActivo Then
        'SE BUSCA EL PRIMER TRMINO ACTIVO Y SE ALMACENA SU POSIBILIDAD (POSS)
        For I = 0 To NAntecedentes(k) - 1
          If SistActual.Regla(k).Antecedente(I).Activa = Si Then
            TempGA = PossTermino(k, I, SistActual.Regla(k).Antecedente(I).NombreEoS, _
                                 SistActual.Regla(k).Antecedente(I).EtiquetaConjDif)
            j = I
            Exit For
          End If
        Next I
        If NAntecedentes(k) = 1 Then
          GoTo SGTE_REGLA
        End If
        'A PARTIR DEL PRIMER TRMINO SE RECORREN LOS RESTANTES
        For I = j To NAntecedentes(k) - 1
          If SistActual.Regla(k).Antecedente(I).Activa = Si Then
            TempPoss = PossTermino(k, I, SistActual.Regla(k).Antecedente(I).NombreEoS, _
                                   SistActual.Regla(k).Antecedente(I).EtiquetaConjDif)
            'EN FUNCIN DEL OPERADOR CONECTIVO SE VA FORMANDO DOS A DOS
            'GRADO DE ACTIVACION DE LA REGLA
            If SistActual.Regla(k).Operador = AAnd Then
              TempGA = TyS_NORMAS(TempGA, TempPoss, SistActual.Inferencia.OpAnd)
            Else
              TempGA = TyS_NORMAS(TempGA, TempPoss, SistActual.Inferencia.OpOr)
            End If
            
          End If
        Next I
      End If
SGTE_REGLA:
      SistActual.Regla(k).GACTIVACION = TempGA
    Next k
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: GACT_MAX_ETIQ_SAL()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub GACT_MAX_ETIQ_SAL(TNorma As TSNorma, Optional AllSalidas As Boolean = False, _
                                                Optional Salida As String, Optional Etiqueta As String)
                                              
  Dim I As Integer, j As Integer, k As Integer
  Dim t As Integer, w As Integer
  Dim TempGAMAX As Double
  
  Salida = UCase$(Trim$(Salida))
  Etiqueta = UCase$(Trim$(Etiqueta))
  
  If Not AllSalidas Then
    For I = 0 To NReglas - 1
      If SistActual.Regla(I).Estado = Activo Or SistActual.Regla(I).Estado = SemiActivo Then
          For j = 0 To NConsecuentes(I) - 1
            'SI ES LA PRIMERA ETIQUETA SE ALMACENA EL GACTIVACION ASOCIADO A DICHA REGLA
            If SistActual.Regla(I).Consecuente(j).Activa = Si And (Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) = Salida And _
               UCase$(Trim$(SistActual.Regla(I).Consecuente(j).EtiquetaConjDif)) = Etiqueta Then
               
               TempGAMAX = SistActual.Regla(I).GACTIVACION
               k = I
               GoTo SGTES_A
            Else
              'SI HA RECORRIDO TODA LA BASE DE CONOCIMIENTO Y NO HAY REGLA QUE DISPARE LA ETIQUETA
              'SE ALMACENA COMO VALOR INDICATIVO 0.0 INDICANDO QUE NO EXISTE G. ACTIVACIN PARA DICHA ETIQUETA
              If I = NReglas - 1 Then
                SistActual.Salida(QIdEoS(S, Salida)).ConjDif(QIdEtiqueta(S, QIdEoS(S, Salida), Etiqueta)).GACTIVACION_MAX = 0#
                Exit Sub
              End If
            End If
          Next j
      End If
    Next I
    
SGTES_A:
    For I = k To NReglas - 1
      If SistActual.Regla(I).Estado = Activo Or SistActual.Regla(I).Estado = SemiActivo Then
          For j = 0 To NConsecuentes(I) - 1
            'A PARTIR DEL PRIMER TRMINO SE RECORREN LOS RESTANTES
            If SistActual.Regla(I).Consecuente(j).Activa = Si And UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) = Salida And _
               UCase$(Trim$(SistActual.Regla(I).Consecuente(j).EtiquetaConjDif)) = Etiqueta Then
               
               TempGAMAX = TyS_NORMAS(TempGAMAX, SistActual.Regla(I).GACTIVACION, TNorma)
               Exit For 'SE SALE PQ AUNQUE HAYA ETIQUETAS IGUALES REPETIDAS EN EL CONSECUENTE EL GRADO ACTIVACIN ES EL MISMO
            End If
          Next j
      End If
    Next I
    SistActual.Salida(QIdEoS(S, Salida)).ConjDif(QIdEtiqueta(S, QIdEoS(S, Salida), Etiqueta)).GACTIVACION_MAX = TempGAMAX
  
  
  'PARA TODAS LAS SALIDAS
  Else
  
    For t = 0 To NSalidas - 1
      If SistActual.Salida(t).Activa = Si Then
        
        For w = 0 To NConjDif(S, t) - 1
          
          For I = 0 To NReglas - 1
            If SistActual.Regla(I).Estado = Activo Or SistActual.Regla(I).Estado = SemiActivo Then
                For j = 0 To NConsecuentes(I) - 1
                  'SI ES LA PRIMERA ETIQUETA SE ALMACENA EL GACTIVACION ASOCIADO A DICHA REGLA
                  If SistActual.Regla(I).Consecuente(j).Activa = Si And UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) = UCase$(Trim$(SistActual.Salida(t).Nombre)) And _
                     UCase$(Trim$(SistActual.Regla(I).Consecuente(j).EtiquetaConjDif)) = UCase$(Trim$(SistActual.Salida(t).ConjDif(w).Etiqueta)) Then
                     
                    TempGAMAX = SistActual.Regla(I).GACTIVACION
                    k = I
                    GoTo SGTES_B
                  Else
                    'SI HA RECORRIDO TODA LA BASE DE CONOCIMIENTO Y NO HAY REGLA QUE DISPARE LA ETIQUETA
                    'SE ALMACENA COMO VALOR INDICATIVO -1 INDICANDO QUE NO EXISTE G. ACTIVACIN PARA DICHA ETIQUETA
                    If I = NReglas - 1 Then
                      SistActual.Salida(t).ConjDif(w).GACTIVACION_MAX = 0#
                      GoTo SGTE_CONJUNTO
                    End If
                  End If
                Next j
            End If
          Next I
          
SGTES_B:
          For I = k To NReglas - 1
            If SistActual.Regla(I).Estado = Activo Or SistActual.Regla(I).Estado = SemiActivo Then
                For j = 0 To NConsecuentes(I) - 1
                  'A PARTIR DEL PRIMER TRMINO SE RECORREN LOS RESTANTES
                  If SistActual.Regla(I).Consecuente(j).Activa = Si And UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) = UCase$(Trim$(SistActual.Salida(t).Nombre)) And _
                     UCase$(Trim$(SistActual.Regla(I).Consecuente(j).EtiquetaConjDif)) = UCase$(Trim$(SistActual.Salida(t).ConjDif(w).Etiqueta)) Then
                     
                     TempGAMAX = TyS_NORMAS(TempGAMAX, SistActual.Regla(I).GACTIVACION, TNorma)
                     Exit For
                  End If
                Next j
            End If
          Next I
          SistActual.Salida(t).ConjDif(w).GACTIVACION_MAX = TempGAMAX
          
SGTE_CONJUNTO:
        Next w
      End If
    Next t
  End If
  
End Sub

'--------------------------------------------------------------
' FASE: IMPLICACIN
'--------------------------------------------------------------

'**************************************************************************************************
'   FUNCIN: Metodo_Sencillo_Implicacion()
'
'       OBJETIVO:
'
'**************************************************************************************************
Private Function Metodo_Sencillo_Implicacion(CoordYCDif() As Double, GActiv As Double) As Boolean
  Dim I As Integer
  
  For I = 0 To UBound(CoordYCDif(), 1)
    If CoordYCDif(I) > GActiv Then
      Metodo_Sencillo_Implicacion = False
      Exit Function
    End If
  Next I
  Metodo_Sencillo_Implicacion = True
  
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: OBTENER_CDIF_GACT_ETIQ()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub OBTENER_CDIF_GACT_ETIQ(TipoImplicacion As ET_IMPLICACIONES, Vble As TSVARIABLE, IdConjDif As Integer, _
                                  Optional ConInterfazGrafica As Boolean = False, Optional LblEtiq As Label, Optional LblGActivacion As Label, _
                                  Optional LblInform As Label, Optional Grafico As PictureBox)
      
  Dim CoordXGActiv() As Double, CoordYGActiv() As Double
  Dim CoordXCDif() As Double, CoordYCDif() As Double
  Dim RectaXGActiv(0 To 1) As Double, RectaYGActiv(0 To 1) As Double
  
  SPLINES_OUTPUT Vble, IdConjDif, CoordXGActiv(), CoordYGActiv(), CoordXCDif(), CoordYCDif()
  
  Erase Vble.ConjDif(IdConjDif).CoordXGActiv()
  Erase Vble.ConjDif(IdConjDif).CoordYGActiv()

  Select Case TipoImplicacion
  
    Case Implicacion_T_Minimo:
        
        If Metodo_Sencillo_Implicacion(CoordYCDif(), CoordYGActiv(1)) Then
          Vble.ConjDif(IdConjDif).CoordXGActiv() = CoordXCDif()
          Vble.ConjDif(IdConjDif).CoordYGActiv() = CoordYCDif()
        Else
          INTERSECCION_TNORMA_MINIMO CoordXGActiv(), CoordYGActiv(), CoordXCDif(), CoordYCDif(), _
          Vble.ConjDif(IdConjDif).CoordXGActiv(), Vble.ConjDif(IdConjDif).CoordYGActiv()
        End If
    
    'Case
    
  End Select
  
  If ConInterfazGrafica Then

    Grafico.Cls
    Dibujar_Spline CoordXCDif(), CoordYCDif(), &HFF&, 4, Grafico 'ROJO
    RectaXGActiv(0) = CoordXGActiv(1)
    RectaXGActiv(1) = CoordXGActiv(2)
    RectaYGActiv(0) = CoordYGActiv(1)
    RectaYGActiv(1) = CoordYGActiv(2)
    Dibujar_Spline RectaXGActiv(), RectaYGActiv(), &HFF8080, 4, Grafico   'AZUL
    Dibujar_Spline Vble.ConjDif(IdConjDif).CoordXGActiv(), Vble.ConjDif(IdConjDif).CoordYGActiv(), _
                                                   &HFFFF&, 2, Grafico, False 'AMARILLO
    LblEtiq.Caption = "<< ETIQ. " & Trim$(Vble.ConjDif(IdConjDif).Etiqueta) & " >>"
    LblGActivacion.Caption = "G.ACTIVACIN = " & Round(Vble.ConjDif(IdConjDif).GACTIVACION_MAX, NUMDEC)
    LblInform.Caption = "OP. IMPLICACIN: " & QImplicacion(SistActual.Inferencia.OpImplicacion)

  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: OBTENER_CDIF_GACT_SAL()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub OBTENER_CDIF_GACT_SAL(TipoImplicacion As ET_IMPLICACIONES, _
                                 Optional AllSalidas As Boolean = False, Optional IdSalida As Integer)
  Dim I As Integer, j As Integer
  
  If Not AllSalidas Then
    For I = 0 To NConjDif(S, IdSalida) - 1
      OBTENER_CDIF_GACT_ETIQ TipoImplicacion, SistActual.Salida(IdSalida), I
    Next I
  
  Else
    For I = 0 To NSalidas - 1
      If SistActual.Salida(I).Activa = Si Then
        For j = 0 To NConjDif(S, I) - 1
          OBTENER_CDIF_GACT_ETIQ TipoImplicacion, SistActual.Salida(I), j
        Next j
      End If
    Next I
    
  End If
  
End Sub

'--------------------------------------------------------------
' FASE: AGREGACIN
'--------------------------------------------------------------

'**************************************************************************************************
'   PROCEDIMIENTO: HayQObtener_CDif_Agreg()
'
'       OBJETIVO:
'
'**************************************************************************************************
Private Function HayQObtener_CDif_Agreg(IdSalida As Integer) As Boolean
  
  'MTODOS DE DEFUZZIFICACIN QUE SI NECESITAN REALIZAR EL PROCESO DE AGREGACIN.
  If SistActual.Salida(IdSalida).Defuzzificador < 100 Then
    HayQObtener_CDif_Agreg = True
  Else 'MTODOS DE DEFUZZIFICACIN QUE NO NECESITAN REALIZAR EL PROCESO DE AGREGACIN.
    HayQObtener_CDif_Agreg = False
  End If
  
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: OpAgregacion_CDif()
'
'       OBJETIVO:
'
'**************************************************************************************************
Private Sub OpAgregacion_CDif(TipoAgregacion As ET_TNORMAS_SNORMAS, CoordXA() As Double, CoordYA() As Double, _
                              CoordXB() As Double, CoordYB() As Double, _
                              CoordXUnion() As Double, CoordYUnion() As Double)

  Select Case TipoAgregacion
  
    Case SNorma_Maximo:
      UNION_SNORMA_MAXIMO CoordXA(), CoordYA(), CoordXB(), CoordYB(), CoordXUnion(), CoordYUnion()
      
    'Case
    
  End Select
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: OBTENER_CDIF_AGREG_SAL()
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub OBTENER_CDIF_AGREG_SAL(TipoAgregacion As ET_TNORMAS_SNORMAS, _
                                  Optional AllSalidas As Boolean = False, Optional IdSalida As Integer)
  Dim I As Integer, j As Integer, w As Integer
  Dim TempX() As Double, TempY() As Double
  
  If Not AllSalidas Then
    If NConjDif(S, IdSalida) > 0 Then   'SI HAY AL MENOS UNA ETIQUETA SE ALMACENAN LAS COORDENADAS
      For I = 0 To NConjDif(S, IdSalida) - 1
        If SistActual.Salida(IdSalida).ConjDif(I).GACTIVACION_MAX > 0# Then
          IntroducirValoresMatrizAenB SistActual.Salida(IdSalida).ConjDif(I).CoordXGActiv(), SistActual.Salida(IdSalida).CoordXAgreg()
          IntroducirValoresMatrizAenB SistActual.Salida(IdSalida).ConjDif(I).CoordYGActiv(), SistActual.Salida(IdSalida).CoordYAgreg()
          w = I
          GoTo SGTE_PASO_A
        End If
      Next I
      IntroducirValoresMatrizAenB SistActual.Salida(IdSalida).ConjDif(0).CoordXGActiv(), SistActual.Salida(IdSalida).CoordXAgreg()
      IntroducirValoresMatrizAenB SistActual.Salida(IdSalida).ConjDif(0).CoordYGActiv(), SistActual.Salida(IdSalida).CoordYAgreg()
      Exit Sub
      
SGTE_PASO_A:
      If NConjDif(S, IdSalida) > w Then 'SI HAY AL MENOS DOS ETIQUETAS YA SE EMPIEZAN A AGREGAR
        For I = w + 1 To NConjDif(S, IdSalida) - 1
                
          If SistActual.Salida(IdSalida).ConjDif(I).GACTIVACION_MAX > 0# Then
              IntroducirValoresMatrizAenB SistActual.Salida(IdSalida).CoordXAgreg(), TempX()
              IntroducirValoresMatrizAenB SistActual.Salida(IdSalida).CoordYAgreg(), TempY()
              OpAgregacion_CDif TipoAgregacion, TempX(), TempY(), _
                                SistActual.Salida(IdSalida).ConjDif(I).CoordXGActiv(), SistActual.Salida(IdSalida).ConjDif(I).CoordYGActiv(), _
                                SistActual.Salida(IdSalida).CoordXAgreg(), SistActual.Salida(IdSalida).CoordYAgreg()
          End If
        Next I
      End If
    End If
    
  Else
    For j = 0 To NSalidas - 1
      If SistActual.Salida(j).Activa = Si Then
        If NConjDif(S, j) > 0 Then
          For I = 0 To NConjDif(S, j) - 1
            If SistActual.Salida(j).ConjDif(I).GACTIVACION_MAX > 0# Then
              IntroducirValoresMatrizAenB SistActual.Salida(j).ConjDif(I).CoordXGActiv(), SistActual.Salida(j).CoordXAgreg()
              IntroducirValoresMatrizAenB SistActual.Salida(j).ConjDif(I).CoordYGActiv(), SistActual.Salida(j).CoordYAgreg()
              w = I
              GoTo SGTE_PASO_B
            End If
          Next I
          IntroducirValoresMatrizAenB SistActual.Salida(j).ConjDif(0).CoordXGActiv(), SistActual.Salida(j).CoordXAgreg()
          IntroducirValoresMatrizAenB SistActual.Salida(j).ConjDif(0).CoordYGActiv(), SistActual.Salida(j).CoordYAgreg()
          GoTo SGTE_SALIDA
          
SGTE_PASO_B:

          If NConjDif(S, j) > w Then
            For I = w + 1 To NConjDif(S, j) - 1
                
            If SistActual.Salida(j).ConjDif(I).GACTIVACION_MAX > 0# Then
              IntroducirValoresMatrizAenB SistActual.Salida(j).CoordXAgreg(), TempX()
              IntroducirValoresMatrizAenB SistActual.Salida(j).CoordYAgreg(), TempY()
              OpAgregacion_CDif TipoAgregacion, TempX(), TempY(), _
                                SistActual.Salida(j).ConjDif(I).CoordXGActiv(), SistActual.Salida(j).ConjDif(I).CoordYGActiv(), _
                                SistActual.Salida(j).CoordXAgreg(), SistActual.Salida(j).CoordYAgreg()
             End If
            
            Next I
          End If
        End If
      End If
SGTE_SALIDA:
    Next j
  
  End If
  
End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: INFERIR_REGLAS
'
'       OBJETIVO:
'
'**************************************************************************************************
Public Sub INFERIR_REGLAS()
  Dim SNorma As TSNorma
  Dim I As Integer
  
  
  GACTIVACION_REGLAS True
  
  SNorma.TipoNorma = SNorma_Maximo
  GACT_MAX_ETIQ_SAL SNorma, True
  
  OBTENER_CDIF_GACT_SAL SistActual.Inferencia.OpImplicacion, True
  
  For I = 0 To NSalidas - 1
    If SistActual.Salida(I).Activa = Si Then
      If HayQObtener_CDif_Agreg(I) Then
        OBTENER_CDIF_AGREG_SAL SistActual.Inferencia.OpAgregacion.TipoNorma, False, I
      End If
    End If
  Next I
  
End Sub


