Attribute VB_Name = "ManagementWindows"
'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: MANAGEMENTWINDOWS.BAS ##
'
' OBJETIVO: Gestionar las ventanas de variables de entrada y salida
'
'-------------------------------------------------------------------------------------------------------


'**************************************************************************************************
'   PROCEDIMIENTO: CargarVentanaEntrada()
'**************************************************************************************************
Public Sub CargarVentanaEntrada(IdE As Integer)
  On Error Resume Next
  Dim I As Integer, j As Integer
  'Se comprueba la dimensin del vector de ventanas de entrada. Si no existiesen ninguna ventana
  'de entrada abierta, oculta, o sea cargada en memoria habr que dimensionar a ndice 0.
  I = UBound(VentanaEntrada(), 1)
  If Err.Number = 9 Then
    ReDim VentanaEntrada(0)
    I = 0
    Err.Clear
  Else
    'Comprobamos que el nmero de ventanas abiertas actualmente sea inferior en al menos
    'una al nmero mximo de ventanas que queremos permitir que se abran simultaneamente
    'de variables de entrada. En caso de que as lo fuera se descargar la primera ventana
    'para liberar espacio de memoria y as proceder a cargar la ventana en cuestin.
    If I = OpcionesInicio.NMaxVEntrada - 1 Then
      MsgBox "El nmero de ventanas de entrada abiertas simultaneamente excede la cantidad lmite" & vbCrLf & _
               "establecida en la ventana de Opciones -> Gestin de la Memoria." & vbCrLf & vbCrLf & _
               "Se procede a descargar de memoria la inicialmente cargada", vbExclamation + vbOKOnly, "Atencin!"
      Unload VentanaEntrada(0)
      I = UBound(VentanaEntrada(), 1)
    End If
    'Se recorren todas las posibles ventanas de entradas cargadas para
    'comprobar si la entrada a cargar en ventana ya estuviese cargada.
    For j = 0 To I
      If VentanaEntrada(j).IdEntrada = IdE Then 'Entrada ya cargada en ventana
        I = j
        GoTo FINALIZAR_CARGA_VENTANA_ENTRADA
      End If
    Next j
    'En el caso de que la variable de entrada no est cargada en
    'una ventana, se proceder a la carga de la misma en memoria, dnde?: en la ltima
    'posicin del vector VentanaEntrada + una posicin (nuevo ltimo ndice).
    I = I + 1
    ReDim Preserve VentanaEntrada(I)
  End If
  'IMPORTANTE!: El ndice de ventana de entrada (i) no tiene pq coincidir con el ndice
  '              de entrada que dicha ventana va a contener, es decir, una ventana de entrada
  '              con ndice x puede contener la informacin de la variable de entrada y dado
  '              que todo va a depender del orden en el cual el usuario vaya visualizando y cerrando
  '              las distintas ventanas de entrada.
  Set VentanaEntrada(I) = New FrmEntrada
  VentanaEntrada(I).IdEntrada = IdE
  'Es necesario disponer del ndice de la ventana como propiedad del objeto FrmEntrada
  'pues en el procedimiento de evento de descarga de dicho formulario (VentanaEntrada recuerde
  'que es un objeto FrmEntrada) ser necesario indicarle cual es la ventana activa que va
  'a ser descargada.
  VentanaEntrada(I).IdVentanaEntrada = I
  
FINALIZAR_CARGA_VENTANA_ENTRADA:
  'VisualizarFrm VentanaEntrada(i)
  'VentanaEntrada(i).ActualizarMe
  With VentanaEntrada(I)
    .Show
    .ActualizarMe
    .WindowState = vbNormal
    .SetFocus
  End With
  
End Sub
                                    
'**************************************************************************************************
'   PROCEDIMIENTO: DesCargarVentanaEntrada()
'**************************************************************************************************
Public Sub DesCargarVentanaEntrada(IdE As Integer)
  
  On Error Resume Next
  Dim I As Integer, j As Integer
  'Se comprueba la dimensin del vector de ventanas de entrada. Si no existiesen ninguna ventana
  'de entrada abierta, oculta, o sea cargada en memoria no habr que descargar nada.
  j = UBound(VentanaEntrada(), 1)
  
  If Err.Number = 9 Then
    Err.Clear
    GoTo ENTRADA_YA_DESCARGADA
  End If
  
  For I = 0 To j
    If VentanaEntrada(I).IdEntrada = IdE Then
      Exit For
    End If
  Next I
  Unload VentanaEntrada(I)

ENTRADA_YA_DESCARGADA:
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: ReestructurarVectorVentanaEntrada()
'**************************************************************************************************
Public Sub ReestructurarVectorVentanaEntrada(PosicionEliminar As Integer)
  Dim I As Integer
  
  If PosicionEliminar = 0 And UBound(VentanaEntrada(), 1) = 0 Then
    Erase VentanaEntrada()
  ElseIf PosicionEliminar < UBound(VentanaEntrada(), 1) Then
    For I = PosicionEliminar To (UBound(VentanaEntrada(), 1) - 1)
      Set VentanaEntrada(I) = VentanaEntrada(I + 1)
      VentanaEntrada(I).IdVentanaEntrada = I
    Next I
    ReDim Preserve VentanaEntrada(UBound(VentanaEntrada(), 1) - 1)
  Else
    ReDim Preserve VentanaEntrada(UBound(VentanaEntrada(), 1) - 1)
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: QVentanaEntradaTieneLaEntrada()
'**************************************************************************************************
Public Function QVentanaEntradaTieneLaEntrada(IdE As Integer) As Integer
  Dim I As Integer
  
  For I = 0 To UBound(VentanaEntrada(), 1)
    If VentanaEntrada(I).IdEntrada = IdE Then
      QVentanaEntradaTieneLaEntrada = I
      Exit Function
    End If
  Next I
  
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: CargarVentanaSalida()
'**************************************************************************************************
Public Sub CargarVentanaSalida(IdS As Integer)
  On Error Resume Next
  Dim I As Integer, j As Integer
  'Se comprueba la dimensin del vector de ventanas de Salida. Si no existiesen ninguna ventana
  'de Salida abierta, oculta, o sea cargada en memoria habr que dimensionar a ndice 0.
  I = UBound(VentanaSalida(), 1)
  If Err.Number = 9 Then
    ReDim VentanaSalida(0)
    I = 0
    Err.Clear
  Else
    'Comprobamos que el nmero de ventanas abiertas actualmente sea inferior en al menos
    'una al nmero mximo de ventanas que queremos permitir que se abran simultaneamente
    'de variables de salida. En caso de que as lo fuera se descargar la primera ventana
    'para liberar espacio de memoria y as proceder a cargar la ventana en cuestin.
    If I = OpcionesInicio.NMaxVSalida - 1 Then
      MsgBox "El nmero de ventanas de salida abiertas simultaneamente excede la cantidad lmite" & vbCrLf & _
               "establecida en la ventana de Opciones -> Gestin de la Memoria." & vbCrLf & vbCrLf & _
               "Se procede a descargar de memoria la inicialmente cargada", vbExclamation + vbOKOnly, "Atencin!"
      Unload VentanaSalida(0)
      I = UBound(VentanaSalida(), 1)
    End If
    'Se recorren todas las posibles ventanas de Salidas cargadas para
    'comprobar si la Salida a cargar en ventana ya estuviese cargada.
    For j = 0 To I
      If VentanaSalida(j).IdSalida = IdS Then 'Salida ya cargada en ventana
        I = j
        GoTo FINALIZAR_CARGA_VENTANA_SALIDA
      End If
    Next j
    'En el caso de que la variable de Salida no est cargada en
    'una ventana, se proceder a la carga de la misma en memoria. Dnde?: en la ltima
    'posicin del vector VentanaSalida + una posicin (nuevo ltimo ndice).
    I = I + 1
    ReDim Preserve VentanaSalida(I)
  End If
  'IMPORTANTE!: El ndice de ventana de Salida (i) no tiene pq coincidir con el ndice
  '              de Salida que dicha ventana va a contener, es decir, una ventana de Salida
  '              con ndice x puede contener la informacin de la variable de Salida y dado
  '              que todo va a depender del orden en el cual el usuario vaya visualizando y cerrando
  '              las distintas ventanas de Salida.
  Set VentanaSalida(I) = New FrmSalida
  VentanaSalida(I).IdSalida = IdS
  VentanaSalida(I).IdVentanaSalida = I
  
FINALIZAR_CARGA_VENTANA_SALIDA:
  With VentanaSalida(I)
    .Show
    .ActualizarMe
    .WindowState = vbNormal
    .SetFocus
  End With
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DesCargarVentanaSalida()
'**************************************************************************************************
Public Sub DesCargarVentanaSalida(IdS As Integer)
  
  On Error Resume Next
  Dim I As Integer, j As Integer
  'Se comprueba la dimensin del vector de ventanas de salida. Si no existiesen ninguna ventana
  'de entrada abierta, oculta, o sea cargada en memoria no habr que descargar nada.
  j = UBound(VentanaSalida(), 1)
  
  If Err.Number = 9 Then
    Err.Clear
    GoTo SALIDA_YA_DESCARGADA
  End If
  
  For I = 0 To j
    If VentanaSalida(I).IdSalida = IdS Then
      Exit For
    End If
  Next I
  Unload VentanaSalida(I)

SALIDA_YA_DESCARGADA:

End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: ReestructurarVectorVentanaSalida()
'**************************************************************************************************
Public Sub ReestructurarVectorVentanaSalida(PosicionEliminar As Integer)
  Dim I As Integer
  
  If PosicionEliminar = 0 And UBound(VentanaSalida(), 1) = 0 Then
    Erase VentanaSalida()
  ElseIf PosicionEliminar < UBound(VentanaSalida(), 1) Then
    For I = PosicionEliminar To (UBound(VentanaSalida(), 1) - 1)
      Set VentanaSalida(I) = VentanaSalida(I + 1)
      VentanaSalida(I).IdVentanaSalida = I
    Next I
    ReDim Preserve VentanaSalida(UBound(VentanaSalida(), 1) - 1)
  Else
    ReDim Preserve VentanaSalida(UBound(VentanaSalida(), 1) - 1)
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: QVentanaSalidaTieneLaSalida()
'**************************************************************************************************
Public Function QVentanaSalidaTieneLaSalida(IdS As Integer) As Integer
  Dim I As Integer
  
  For I = 0 To UBound(VentanaSalida(), 1)
    If VentanaSalida(I).IdSalida = IdS Then
      QVentanaSalidaTieneLaSalida = I
      Exit Function
    End If
  Next I
  
End Function
