Attribute VB_Name = "Outputs"
'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO : OUTPUTS.BAS ##
'
' OBJETIVO: Contener el cdigo (propiedades, funciones, procedimientos, vbles, ctes,...)
'           necesarios para procesar la informacin acerca de las salidas del sistema.
'-------------------------------------------------------------------------------------------------------

Option Explicit

Public SuprimidaSalida As TSVARIABLE
Public HaySalidaEliminada As Boolean

'**************************************************************************************************
'   PROPIEDAD: NSalidas() -> Indica el n de salidas que hay en el sistema.
'**************************************************************************************************
Public Property Get NSalidas() As Integer
  On Error GoTo Fallo
  Dim i As Integer
  
  i = UBound(SistActual.Salida(), 1)
  NSalidas = i + 1
  Exit Property
Fallo:
  If Err.Number = 9 Then
    NSalidas = 0
    Err.Clear
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD: NSalidasActivas() -> Indica el n de salidas activas que hay en el sistema.
'**************************************************************************************************
Public Property Get NSalidasActivas() As Integer
  Dim i As Integer, j As Integer
  j = 0
  For i = 0 To NSalidas - 1
    If SistActual.Salida(i).Activa = Si Then
      j = j + 1
    End If
  Next i
  NSalidasActivas = j
  
End Property

'**************************************************************************************************
'   PROPIEDAD: NSalidasPasivas() -> Indica el n de salidas desactivadas en el sistema.
'**************************************************************************************************
Public Property Get NSalidasPasivas() As Integer
  
  NSalidasPasivas = NSalidas - NSalidasActivas
End Property

'**************************************************************************************************
'   PROCEDIMIENTO: EliminarSalidaMatriz()
'**************************************************************************************************
Public Sub EliminarSalidaMatriz(Matriz() As TSVARIABLE, Posicion As Integer, SalidaSuprimida As TSVARIABLE)
  On Error Resume Next
  Dim i As Integer, NElementosMatriz As Integer
  'Se calcula el nmero de elementos de la matriz
  i = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = i + 1
  End If
  'Antes de proceder a su eliminacin este se almacena como informacin a reconstruir en caso de querer
  'el usuario deshacer la eliminacin de entrada
  SalidaSuprimida = Matriz(Posicion)
  'Dado que la matriz comienza en el elemento Matriz(0) el ndice ira de 0 -> (NElementosMatriz -1)
  'SI LA MATRIZ CONTIENE UN SOLO ELEMENTO
  If NElementosMatriz = 1 Then
    'Se liberar memoria ocupada por la matriz dinmica Matriz()
    Erase Matriz()
  'SI LA MATRIZ CONTIENE MS DE UN ELEMENTO Y EL SELECCIONADO PARA ELIMINAR ES EL LTIMO
  ElseIf (Posicion = (NElementosMatriz - 1)) And NElementosMatriz > 1 Then
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  Else
    'Trasladamos los datos de la posicin i a i-1
    For i = Posicion + 1 To (NElementosMatriz - 1)
      Matriz(i - 1) = Matriz(i)
    Next i
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  End If
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: AadirSalidaMatriz()
'**************************************************************************************************
Public Sub AadirSalidaMatriz(Matriz() As TSVARIABLE, SalidaInsertar As TSVARIABLE, _
                               Optional PosicionFinal As Boolean = True, Optional Posicion As Integer)
  On Error Resume Next
  Dim i As Integer, MIndInicial As Integer, NSaltos As Integer, NElementosMatriz As Integer
  
  'Se calcula el nmero de elementos de la matriz
  i = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = i + 1
  End If
  
  MIndInicial = NElementosMatriz - 1
  If PosicionFinal Then
    ReDim Preserve Matriz(MIndInicial + 1)
    Matriz(MIndInicial + 1) = SalidaInsertar
    Exit Sub
  End If
  
  If NElementosMatriz = 0 Then
    ReDim Matriz(0)
    Matriz(0) = SalidaInsertar
  ElseIf Posicion > MIndInicial Then
    ReDim Preserve Matriz(MIndInicial + 1)
    Matriz(Posicion) = SalidaInsertar
    Exit Sub
  ElseIf Posicion < 0 Then
    Posicion = 0
    GoTo Seguir
  Else
Seguir:
    ReDim Preserve Matriz(MIndInicial + 1)
    NSaltos = (MIndInicial + 1) - Posicion
    For i = 1 To NSaltos
      Matriz((MIndInicial + 2) - i) = Matriz((MIndInicial + 1) - i)
    Next i
    Matriz(Posicion) = SalidaInsertar
  End If
  
End Sub

