Attribute VB_Name = "Points"
 '-------------------------------------------------------------------------------------------------------
'                                  ## MDULO DE CDIGO : POINTS.BAS ##
'
' OBJETIVO: Gestiona la manipulacin de los puntos que definen los conjuntos difusos.
'-------------------------------------------------------------------------------------------------------
Option Explicit
'---------------------------------
'     - DECLARACIN DE TIPOS -
'---------------------------------
'Estructura para contener la informacin del punto que se est modificando
Public Type TPTOSELECCIONADO
    DifoConjDif As ETDIFOCONDIF
    IdConjDif As Integer
    IdPto As ETPBASICOS
    IdPtoExtendido As Integer
    x As Double
    xMin As Double
    yMin As Double
    xMax As Double
    yMax As Double
    y As Double
End Type
          
'Almacena el si el nuevo pto extendido es en el difusor o C.Difuso y si es sobre una entrada o salida.
Public NuevoPExtendidoEnDifusor As Boolean
Public NuevoPExtendidoEnEntrada As Boolean


'**************************************************************************************************
'   PROPIEDAD: NPExtendidos() -> Devuelve el n de p.extendidos contenidos en el campo
'                                PExtendidos() de las distintas estructuras del sistema.
'**************************************************************************************************
Public Property Get NPExtendidos(EoS As ETEOS, IdEoS As Integer, DifoConjDif As ETDIFOCONDIF, IdConjDif As Integer) As Integer
  On Error Resume Next
  Dim I As Integer
  
  Select Case EoS
    Case Entrada:
      Select Case DifoConjDif
        Case Difusor:
          I = UBound(SistActual.Entrada(IdEoS).Difusor.PExtendidos(), 1)
        Case ConjDif:
          I = UBound(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(), 1)
      End Select
    
    Case Salida:
      I = UBound(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(), 1)
  End Select
  If Err.Number = 9 Then
    NPExtendidos = 0
  Else
    NPExtendidos = (I + 1) \ 2
  End If
  
End Property


'**************************************************************************************************
'   PROPIEDAD: QPBasicos() -> Devuelve en forma de cadena el vector formado por los P.Bsicos
'                             del campo PBasicos() de las distintas estructuras del sistema.
'**************************************************************************************************
Public Property Get QPBasicos(EoS As ETEOS, IdEoS As Integer, DifoConjDif As ETDIFOCONDIF, IdConjDif As Integer) As String
  Dim Cadena As String
  Select Case EoS
    Case Entrada:
      Select Case DifoConjDif
        Case Difusor:
          Cadena = "[(" & Round(SistActual.Entrada(IdEoS).Difusor.PBasicos(A), NUMDEC) & ":0); (" & Round(SistActual.Entrada(IdEoS).Difusor.PBasicos(B), NUMDEC) & ":1); (" _
                        & Round(SistActual.Entrada(IdEoS).Difusor.PBasicos(C), NUMDEC) & ":1); (" & Round(SistActual.Entrada(IdEoS).Difusor.PBasicos(D), NUMDEC) & ":0)]"
        Case ConjDif:
          Cadena = "[(" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PBasicos(A), NUMDEC) & ":0); (" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PBasicos(B), NUMDEC) & ":1); (" _
                        & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PBasicos(C), NUMDEC) & ":1); (" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PBasicos(D), NUMDEC) & ":0)]"
      End Select
    Case Salida:
      Cadena = "[(" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PBasicos(A), NUMDEC) & ":0); (" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PBasicos(B), NUMDEC) & ":1); (" _
                    & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PBasicos(C), NUMDEC) & ":1); (" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PBasicos(D), NUMDEC) & ":0)]"
  End Select
  QPBasicos = Cadena
  
End Property

'**************************************************************************************************
'   PROPIEDAD: QPExtendidos() -> Devuelve en forma de cadena el vector formado por los P.Extendidos
'                                del campo PBasicos() de las distintas estructuras del sistema.
'**************************************************************************************************
Public Property Get QPExtendidos(EoS As ETEOS, IdEoS As Integer, DifoConjDif As ETDIFOCONDIF, IdConjDif As Integer) As String
  Dim NExtendidos As Integer
  Dim Cadena As String
  
  NExtendidos = NPExtendidos(EoS, IdEoS, DifoConjDif, IdConjDif)
  Select Case EoS
    Case Entrada:
        Select Case DifoConjDif
          Case ConjDif:
            If NExtendidos = 1 Then
              Cadena = "[(" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(1), NUMDEC) & ")]"
              
            ElseIf NExtendidos = 2 Then
              Cadena = "[(" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(1), NUMDEC) & "); (" _
                            & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(2), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(3), NUMDEC) & ")]"
            ElseIf NExtendidos > 2 Then
              Cadena = "[(" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos(1), NUMDEC) & "); ... ;(" _
                            & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos((NExtendidos * 2) - 2), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).PExtendidos((NExtendidos * 2) - 1), NUMDEC) & ")]"
            End If
          
          Case Difusor:
            If NExtendidos = 1 Then
              Cadena = "[(" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(1), NUMDEC) & ")]"
              
            ElseIf NExtendidos = 2 Then
              Cadena = "[(" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(1), NUMDEC) & "); (" _
                            & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(2), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(3), NUMDEC) & ")]"
            ElseIf NExtendidos > 2 Then
              Cadena = "[(" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos(1), NUMDEC) & "); ... ;(" _
                            & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos((NExtendidos * 2) - 2), NUMDEC) & ":" & Round(SistActual.Entrada(IdEoS).Difusor.PExtendidos((NExtendidos * 2) - 1), NUMDEC) & ")]"
            End If
        End Select
    Case Salida:
        If NExtendidos = 1 Then
          Cadena = "[(" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(1), NUMDEC) & ")]"
          
        ElseIf NExtendidos = 2 Then
          Cadena = "[(" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(1), NUMDEC) & "); (" _
                        & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(2), NUMDEC) & ":" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(3), NUMDEC) & ")]"
        ElseIf NExtendidos > 2 Then
          Cadena = "[(" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(0), NUMDEC) & ":" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos(1), NUMDEC) & "); ... ;(" _
                        & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos((NExtendidos * 2) - 2), NUMDEC) & ":" & Round(SistActual.Salida(IdEoS).ConjDif(IdConjDif).PExtendidos((NExtendidos * 2) - 1), NUMDEC) & ")]"
        End If
  End Select
  QPExtendidos = Cadena
  
End Property

'**************************************************************************************************
'   PROCEDIMIENTO: ReordenarPByE()
'**************************************************************************************************
Public Sub ReordenarPByE(MPB() As Double, MPE() As Double, MPBYE() As Double)
  On Error Resume Next
  Dim I As Integer
  Dim PEDCHA() As Double
  Dim MSUB_MPE As Integer, MSUB_PEDCHA As Integer, MSUB_MPBYE As Integer

  'Almacena (A,0)
  ReDim MPBYE(1)
  MPBYE(0) = MPB(A)
  MPBYE(1) = 0#

  MSUB_MPE = UBound(MPE(), 1)
  If Err.Number = 9 Then
    MSUB_MPE = 0
    Err.Clear
  End If
  'Si hay algn P.Extendido
  If MSUB_MPE >= 1 Then
    For I = 0 To MSUB_MPE - 1 Step 2
      'PARAMETRO EXTENDIDO DE LA IZDA
      If MPE(I) <= MPB(B) And MPE(I) >= MPB(A) Then
        ReDim Preserve MPBYE(UBound(MPBYE(), 1) + 2)
        MPBYE(I + 2) = MPE(I)
        MPBYE((I + 2) + 1) = MPE(I + 1)
      'PARAMETRO EXTENDIDO DE LA DCHA
      ElseIf MPE(I) >= MPB(C) And MPE(I) <= MPB(D) Then
        MSUB_PEDCHA = UBound(PEDCHA(), 1)
        If Err.Number = 9 Then
          MSUB_PEDCHA = 0
          Err.Clear
          ReDim PEDCHA(1)
        Else
          ReDim Preserve PEDCHA(MSUB_PEDCHA + 2)
        End If
        MSUB_PEDCHA = UBound(PEDCHA(), 1)
        PEDCHA(MSUB_PEDCHA - 1) = MPE(I)
        PEDCHA(MSUB_PEDCHA) = MPE(I + 1)
      End If
    Next I
  End If
  'Almacena (B,1); (C,1)
  ReDim Preserve MPBYE(UBound(MPBYE(), 1) + 4)
  MPBYE(UBound(MPBYE(), 1) - 3) = MPB(B)
  MPBYE(UBound(MPBYE(), 1) - 2) = 1#
  MPBYE(UBound(MPBYE(), 1) - 1) = MPB(C)
  MPBYE(UBound(MPBYE(), 1)) = 1#
  MSUB_MPBYE = UBound(MPBYE(), 1)
  For I = 0 To MSUB_PEDCHA - 1 Step 2
    ReDim Preserve MPBYE(UBound(MPBYE(), 1) + 2)
    MPBYE(UBound(MPBYE(), 1) - 1) = PEDCHA(I)
    MPBYE(UBound(MPBYE(), 1)) = PEDCHA(I + 1)
  Next I
  'Almacena (D,0)
  ReDim Preserve MPBYE(UBound(MPBYE(), 1) + 2)
  MPBYE(UBound(MPBYE(), 1) - 1) = MPB(D)
  MPBYE(UBound(MPBYE(), 1)) = 1#
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: BuscarLimitadoresPto()
'**************************************************************************************************
Public Sub BuscarLimitadoresPto(PtoSeleccionado As TPTOSELECCIONADO, PBasicos() As Double, PExtendidos() As Double, UDiscurso() As Double)
  On Error Resume Next
  Dim I As Integer, NPExt As Integer
  Dim PosLMin As Integer, PosLMax As Integer
  Dim Dstancia As Double, MenorDstancia As Double
  
  NPExt = UBound(PExtendidos(), 1)
  If Err.Number = 9 Then
    NPExt = 0
    Err.Clear
  End If
  
  Select Case PtoSeleccionado.IdPto
    'SI ES EL PTO BSICO A
    '-----
    '  A
    '-----
    Case A:
      'LIMITADOR MNIMO (CONJ.DIFUSOS) -> VALOR MNIMO DEL U.DISCURSO
      PtoSeleccionado.xMin = UDiscurso(0) + EPSILON
      PtoSeleccionado.yMin = 0#
      'LIMITADOR MXIMO -> SI EXISTE ALGN P.EXTENDIDO ENTRE A-B SU LIMITADOR SER EL PRIMERO SINO SER B SU LIMITADOR MXIMO
      If NPExt > 0 And PExtendidos(0) <= PBasicos(1) Then
        PtoSeleccionado.xMax = PExtendidos(0) - EPSILON
        PtoSeleccionado.yMax = PExtendidos(1)
      Else
        PtoSeleccionado.xMax = PBasicos(1) - EPSILON
        PtoSeleccionado.yMax = 1#
      End If
    'SI ES EL PTO BSICO B
    '-----
    '  B
    '-----
    Case B:
      'LIMITADOR MNIMO -> SI EXISTE ALGN P.EXTENDIDO ENTRE A-B SER EL LIMITADOR MNIMO DE B AQUEL MS PRXIMO POR SU IZDA.
      If NPExt > 0 And PExtendidos(0) <= PBasicos(1) Then
        MenorDstancia = PBasicos(1) - PExtendidos(0) 'De partida el ms prximo es el primero.
        For I = 0 To NPExt - 1 Step 2                'Se recorre la matriz de p.extendidos buscando posibles p.extendidos
          If PExtendidos(I) <= PBasicos(1) Then      'ms prximos a B por la izda que el ltimo localizado.
            Dstancia = PBasicos(1) - PExtendidos(I)  'Ante p.extendidos de igual x prevalece el de mayor ndice en la matriz pues esta
            If Dstancia <= MenorDstancia Then        'se recorre de izda -> dchas.
              MenorDstancia = Dstancia
              PosLMin = I
            End If
          End If
        Next I
        PtoSeleccionado.xMin = PExtendidos(PosLMin) + EPSILON
        PtoSeleccionado.yMin = PExtendidos(PosLMin + 1)
      Else
        PtoSeleccionado.xMin = PBasicos(0) + EPSILON         'SI NO EXISTES P.EXTENDIDOS ENTRE A-B SU LIMITADOR MIN SER EL PTO A.
        PtoSeleccionado.yMin = 0#
      End If
      'LIMITADOR MXIMO -> C
      PtoSeleccionado.xMax = PBasicos(2) - EPSILON
      PtoSeleccionado.yMax = 1#
    'SI ES EL PTO BSICO C
    '-----
    '  C
    '-----
    Case C:
      For I = 0 To NPExt - 1 Step 2                 'SI EXISTE ALGN P.EXTENDIDO ENTRE C-D SER SU LIMITADOR MX AQUEL MS PRXIMO POR SU DCHA.
        If PExtendidos(I) >= PBasicos(2) Then
          PtoSeleccionado.xMax = PExtendidos(I) - EPSILON
          PtoSeleccionado.yMax = PExtendidos(I + 1)
          GoTo Salir_C
        End If
      Next I
      PtoSeleccionado.xMax = PBasicos(3) - EPSILON            'SI NO EXISTES P.EXTENDIDOS ENTRE C-D SU LIMITADOR MAX SER EL PTO D.
      PtoSeleccionado.yMax = 1#
Salir_C:
      'LIMITADOR MNIMO -> B
      PtoSeleccionado.xMin = PBasicos(1) + EPSILON
      PtoSeleccionado.yMin = 1#
    'SI ES EL PTO BSICO D
    '-----
    '  D
    '-----
    Case D:
      For I = NPExt - 1 To 0 Step -2                 'SI EXISTE ALGN P.EXTENDIDO ENTRE C-D SER SU LIMITADOR MIN AQUEL MS PRXIMO POR SU IZDA.
        If PExtendidos(I) <= PBasicos(3) Then
          PtoSeleccionado.xMin = PExtendidos(I) + EPSILON
          PtoSeleccionado.yMin = PExtendidos(I + 1)
          GoTo Salir_D
        End If
      Next I
      PtoSeleccionado.xMin = PBasicos(2) + EPSILON            'SI NO EXISTES P.EXTENDIDOS ENTRE C-D SU LIMITADOR MIN SER EL PTO C.
      PtoSeleccionado.yMin = 1#
Salir_D:
      'LIMITADOR MXIMO (CONJ.DIFUSOS) -> VALOR MXIMO DEL U.DISCURSO
      PtoSeleccionado.xMax = UDiscurso(1) - EPSILON
      PtoSeleccionado.yMax = 1#
    'SI ES UN PTO EXTENDIDO
    '-----
    '  EX
    '-----
    Case EX:
      Dim MPBYE() As Double
      ReordenarPByE PBasicos(), PExtendidos(), MPBYE()  'SE FORMA UNA MATRIZ TEMPORAL CON LOS P.BSICOS Y EXTENDIDOS ORDENADOS
      For I = 0 To UBound(MPBYE(), 1) - 1 Step 2        'SE RECORRE DICHA MATRIZ EN BUSCA DEL PTO EXTENDIDO Y AL ENCONTRARLO SUS
                                                        'SUS LIMITADORES SERN LOS PUNTOS ANTERIOR Y CONSECUTIVO PARA EL MNIMO Y
                                                        'MXIMO RESPECTIVAMENTE.
        If PtoSeleccionado.x = MPBYE(I) And PtoSeleccionado.y = MPBYE(I + 1) Then
          PtoSeleccionado.xMin = MPBYE(I - 2) + EPSILON
          PtoSeleccionado.yMin = MPBYE(I - 1)
          PtoSeleccionado.xMax = MPBYE(I + 2) - EPSILON
          PtoSeleccionado.yMax = MPBYE(I + 3)
          GoTo Salir_E
        End If
      Next I
Salir_E:

  End Select
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: IntroducirPEMatriz()
'**************************************************************************************************
Public Sub IntroducirPEMatriz(PE_X As Double, PE_Y As Double, MPE() As Double)
  On Error GoTo Fallo
  Dim I As Integer, j As Integer, Dimension_MPE As Integer
  Dim IntroducidoPE As Boolean
  Dim NMPE() As Double
  
  IntroducidoPE = False
  'Se calcula el n de p.extendidos que existes antes de introducir uno ms.
  I = UBound(MPE(), 1)
  Dimension_MPE = I + 1
  GoTo Seguir
Fallo:
  If Err.Number = 9 Then
    Dimension_MPE = 0
    Err.Clear
  End If
Seguir:
  If Dimension_MPE = 0 Then 'Si no haba el nico que va a existir es el nuevo.
    ReDim MPE(1)
    MPE(0) = PE_X
    MPE(1) = PE_Y
    Exit Sub
  End If
  For I = 0 To Dimension_MPE - 1 Step 2 'En el caso de que ya existieran
    If Not IntroducidoPE Then           'mientras que no se haya introducido el nuevo
      'Se ordenan de menor a mayor valor los p.extendidos en la matriz
      If MPE(I) < PE_X Then
        If I = 0 Then
          ReDim NMPE(1)
          j = 0
        Else
          ReDim Preserve NMPE(UBound(NMPE(), 1) + 2)
          j = UBound(NMPE(), 1) - 1
        End If
        
        NMPE(j) = MPE(I)
        NMPE(j + 1) = MPE(I + 1)
      ElseIf MPE(I) = PE_X Then 'A IGUALDAD DE X EL NUEVO PTO.
        If I = 0 Then
          ReDim NMPE(3)
          j = 0
        Else
          ReDim Preserve NMPE(UBound(NMPE(), 1) + 4)
          j = UBound(NMPE(), 1) - 3
        End If
        
        NMPE(j) = MPE(I)
        NMPE(j + 1) = MPE(I + 1)
        NMPE(j + 2) = PE_X
        NMPE(j + 3) = PE_Y
        IntroducidoPE = True
      ElseIf MPE(I) > PE_X Then
        If I = 0 Then
          ReDim NMPE(3)
          j = 0
        Else
          ReDim Preserve NMPE(UBound(NMPE(), 1) + 4)
          j = UBound(NMPE(), 1) - 3
        End If
        
        NMPE(j) = PE_X
        NMPE(j + 1) = PE_Y
        NMPE(j + 2) = MPE(I)
        NMPE(j + 3) = MPE(I + 1)
        IntroducidoPE = True
      End If
    ElseIf IntroducidoPE Then 'UNA VEZ QUE YA SE HAYA INTRODUCIDO EL NUEVO PTO EXTENDIDO SE TERMINAN DE INSERTAN LOS RESTANTES PTOS.EXTENDIDOS
      If I = 0 Then
        ReDim NMPE(1)
        j = 0
      Else
        ReDim Preserve NMPE(UBound(NMPE(), 1) + 2)
        j = UBound(NMPE(), 1) - 1
      End If
      NMPE(j) = MPE(I)
      NMPE(j + 1) = MPE(I + 1)
    End If
  Next I
  'Si no se ha insertado es pq es el mayor de todos luego va en la ltima posicin
  If Not IntroducidoPE Then
    ReDim Preserve NMPE(UBound(NMPE(), 1) + 2)
    j = UBound(NMPE(), 1) - 1
    NMPE(j) = PE_X
    NMPE(j + 1) = PE_Y
  End If
  'Por ltimo se asigna a la inicial matriz de ptos.extendidos la que ya contiene en su posicin correcta el nuevo pto.extendido.
  MPE() = NMPE()
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: EliminarPEMatriz()
'**************************************************************************************************
Public Sub EliminarPEMatriz(Matriz() As Double, Posicion As Integer, PE_XSuprimido As Double, PE_YSuprimido As Double)
  On Error Resume Next
  Dim I As Integer, NElementosMatriz As Integer
  'Se calcula el nmero de elementos de la matriz
  I = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = I + 1
  End If
  'Antes de proceder a su eliminacin este se almacena como informacin a reconstruir en caso de querer
  'el usuario deshacer la eliminacin.
  PE_XSuprimido = Matriz(Posicion)
  PE_YSuprimido = Matriz(Posicion + 1)
  'Dado que la matriz comienza en el elemento Matriz(0) el ndice ira de 0 -> (NElementosMatriz -1)
  'SI LA MATRIZ CONTIENE UN SOLO ELEMENTO
  If NElementosMatriz = 2 Then
    'Se liberar memoria ocupada por la matriz dinmica Matriz()
    Erase Matriz()
  'SI LA MATRIZ CONTIENE MS DE UN ELEMENTO Y EL SELECCIONADO PARA ELIMINAR ES EL LTIMO
  ElseIf (Posicion = (NElementosMatriz - 2)) And NElementosMatriz > 2 Then
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 3)
  Else
    'Trasladamos los datos de la posicin i a i-1
    For I = Posicion + 2 To (NElementosMatriz - 2)
      Matriz(I - 2) = Matriz(I)
      Matriz(I - 1) = Matriz(I + 1)
    Next I
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 3)
  End If
End Sub

'**************************************************************************************************
'   FUNCIN: EsValidoPto()
'**************************************************************************************************
Public Function EsValidoPto(PtoSeleccionado As TPTOSELECCIONADO, P_X As Double, P_Y As Double, PB() As Double, PE() As Double, UDiscurso() As Double, Mensaje As String) As Boolean
  
  'SI EL PTO A COMPROBAR EST FUERA DEL LMITE [0.0,1.0] EN EL EJE Y AUTOMTICA NO ES PUNTO VLIDO
  If P_Y < 0# Or P_Y > 1# Then
    Mensaje = "El punto (" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
              "su coordenada Y: " & Round(P_Y, NUMDEC) & " est fuera del intervalo vlido [0.0, 1.0]"
    EsValidoPto = False
    Exit Function
  End If
   
  'EN FUNCIN DEL TIPO DE PUNTO A COMPROBAR
  Select Case PtoSeleccionado.IdPto
    '-----
    '  A
    '-----
    Case A:
      If PtoSeleccionado.DifoConjDif = ConjDif Then 'SI SE TRATA DEL PTO A EN UN CONJ.DIFUSO EXISTE LIMITADOR MNIMO
        If P_X < UDiscurso(0) Then
          Mensaje = "El punto A(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "su coordenada X: " & Round(P_X, NUMDEC) & " est fuera del U.Discurso por su mnimo " & Round(UDiscurso(0), NUMDEC)
          EsValidoPto = False
          Exit Function
        End If
      End If
      'SI EL PUNTO A COMPROBAR SUPERA SU MXIMO
      If P_X > PtoSeleccionado.xMax Then
        Mensaje = "El punto A(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " supera su limitador mximo " & Round(PtoSeleccionado.xMax, NUMDEC)
        EsValidoPto = False
        Exit Function
      'SI EL PUNTO A COMPROBAR ES IGUAL A SU MXIMO
      ElseIf P_X = PtoSeleccionado.xMax Then
        'EN EL CASO DE QUE SU PTO LIMITADOR MXIMO TENGA LA MISMA ALTURA EL PUNTO NO SER VLIDO PUES SERAN COINCIDENTES
        If P_Y = PtoSeleccionado.yMax Then
          Mensaje = "El punto A(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "existe un punto extendido E con las mismas coordenadas."
          EsValidoPto = False
          Exit Function
        End If
        GoTo VALIDO_A 'SI SUS ALTURAS SON DISTINTAS PASA A SER PUNTO VLIDO
      Else
VALIDO_A:
        Mensaje = ""
        EsValidoPto = True
        Exit Function
      End If
    '-----
    '  B
    '-----
    Case B:
      'SI EL PUNTO A COMPROBAR ES INFERIOR A SU MNIMO
      If P_X < PtoSeleccionado.xMin Then
        Mensaje = "El punto B(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " es inferior a su limitador mnimo " & Round(PtoSeleccionado.xMin, NUMDEC)
        EsValidoPto = False
        Exit Function
      'SI EL PUNTO A COMPROBAR ES IGUAL A SU MNIMO
      ElseIf P_X = PtoSeleccionado.xMin Then
        'EN EL CASO DE QUE SU PTO LIMITADOR MNIMO TENGA LA MISMA ALTURA EL PUNTO NO SER VLIDO PUES SERAN COINCIDENTES
        If P_Y = PtoSeleccionado.yMin Then
          Mensaje = "El punto B(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "existe un punto extendido E con las mismas coordenadas."
          EsValidoPto = False
          Exit Function
        End If
        GoTo VALIDO_B 'SI SUS ALTURAS SON DISTINTAS PASA A SER PUNTO VLIDO
      'SI EL PUNTO A COMPROBAR SUPERA SU MXIMO (IMPORTANTE!!, NTESE QUE SE PERMITE QUE B=C)
      ElseIf P_X > PtoSeleccionado.xMax Then
        Mensaje = "El punto B(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " supera su limitador mximo " & Round(PtoSeleccionado.xMax, NUMDEC)
        EsValidoPto = False
        Exit Function
      Else
VALIDO_B:
        Mensaje = ""
        EsValidoPto = True
        Exit Function
      End If
  
    '-----
    '  C
    '-----
    Case C:
      'SI EL PUNTO A COMPROBAR ES INFERIOR A SU MNIMO
      If P_X < PtoSeleccionado.xMin Then '(IMPORTANTE!!, NTESE QUE SE PERMITE QUE B=C)
        Mensaje = "El punto C(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " es inferior a su limitador mnimo " & Round(PtoSeleccionado.xMin, NUMDEC)
        EsValidoPto = False
        Exit Function
      ElseIf P_X > PtoSeleccionado.xMax Then
        Mensaje = "El punto C(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " supera su limitador mximo " & Round(PtoSeleccionado.xMax, NUMDEC)
        EsValidoPto = False
        Exit Function
      'SI EL PUNTO A COMPROBAR ES IGUAL A SU MXIMO
      ElseIf P_X = PtoSeleccionado.xMax Then
        'EN EL CASO DE QUE SU PTO LIMITADOR MXIMO TENGA LA MISMA ALTURA EL PUNTO NO SER VLIDO PUES SERAN COINCIDENTES
        If P_Y = PtoSeleccionado.yMax Then
          Mensaje = "El punto C(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "existe un punto extendido E con las mismas coordenadas."
          EsValidoPto = False
          Exit Function
        End If
        GoTo VALIDO_C 'SI SUS ALTURAS SON DISTINTAS PASA A SER PUNTO VLIDO
      Else
VALIDO_C:
        Mensaje = ""
        EsValidoPto = True
        Exit Function
      End If
    
    '-----
    '  D
    '-----
    Case D:
      If PtoSeleccionado.DifoConjDif = ConjDif Then 'SI SE TRATA DEL PTO D EN UN CONJ.DIFUSO EXISTE LIMITADOR MXIMO
        If P_X > UDiscurso(1) Then
          Mensaje = "El punto D(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "su coordenada X: " & Round(P_X, NUMDEC) & " est fuera del U.Discurso por su mximo " & Round(UDiscurso(1), NUMDEC)
          EsValidoPto = False
          Exit Function
        End If
      End If
      'SI EL PUNTO A COMPROBAR SUPERA SU MNIMO
      If P_X < PtoSeleccionado.xMin Then
        Mensaje = "El punto D(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " es inferior a su limitador mnimo " & Round(PtoSeleccionado.xMin, NUMDEC)
        EsValidoPto = False
        Exit Function
      'SI EL PUNTO A COMPROBAR ES IGUAL A SU MNIMO
      ElseIf P_X = PtoSeleccionado.xMin Then
        'EN EL CASO DE QUE SU PTO LIMITADOR MNIMO TENGA LA MISMA ALTURA EL PUNTO NO SER VLIDO PUES SERAN COINCIDENTES
        If P_Y = PtoSeleccionado.yMin Then
          Mensaje = "El punto D(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "existe un punto extendido E con las mismas coordenadas."
          EsValidoPto = False
          Exit Function
        End If
        GoTo VALIDO_D 'SI SUS ALTURAS SON DISTINTAS PASA A SER PUNTO VLIDO
      Else
VALIDO_D:
        Mensaje = ""
        EsValidoPto = True
        Exit Function
      End If
    
    '-----
    '  EX
    '-----
    Case EX:
      Dim MPBYE() As Double
      Dim I As Integer
      Dim NRepeticiones As Integer
      'SI EL PUNTO A COMPROBAR ES INFERIOR A SU MNIMO
      If P_X < PtoSeleccionado.xMin Then
        Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " es inferior a su limitador mnimo " & Round(PtoSeleccionado.xMin, NUMDEC)
        EsValidoPto = False
        Exit Function
      'SI EL PUNTO A COMPROBAR ES SUPERIOR A SU MXIMO
      ElseIf P_X > PtoSeleccionado.xMax Then
        Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                  "su coordenada X: " & Round(P_X, NUMDEC) & " supera su limitador mximo " & Round(PtoSeleccionado.xMax, NUMDEC)
        EsValidoPto = False
        Exit Function
      'SI EL PUNTO A COMPROBAR ES IGUAL A SU MNIMO
      ElseIf P_X = PtoSeleccionado.xMin Then
        'EN EL CASO DE QUE SU PTO LIMITADOR MNIMO TENGA LA MISMA ALTURA EL PUNTO NO SER VLIDO PUES SERAN COINCIDENTES
        If P_Y = PtoSeleccionado.yMin Then
          Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "existe un punto con las mismas coordenadas."
          EsValidoPto = False
          Exit Function
        Else 'SI SUS ALTURAS SON DISTINTAS PASA A SER UN POSIBLE!! PTO VLIDO
          'SER VLIDO SI NO EXISTEN PARA DICHA X MS DE UN PUNTO, ES DECIR, AL VALIDAR EL PUNTO ESTE FORMARA UN SEGMENTO VERTICAL
          'CON EL PUNTO YA EXISTENTE PARA DICHO X PERO SI EXISTIAN DE ANTES DOS EXISTIRIAN AL VALIDARLO 3 PUNTOS DANDO REDUNDANCIA
          'A LA DEFINICIN DE UN SEGMENTO. NOTA!! HAY QUE PERMITIR EL CASO DE B=C Y OTRO P.EXTENDIDO QUE FORMAN UN SEGMENTO VERTICAL
          '(EXCEPCIN DEL CASO DE REDUNDANCIA EN LA DEFINICIN DE UN SEGMENTO MEDIANTE 3 PTOS).
          ReordenarPByE PB(), PE(), MPBYE()
          NRepeticiones = 0
          For I = 0 To UBound(MPBYE(), 1) - 1 Step 2
            If MPBYE(I) = PtoSeleccionado.xMin Then
              NRepeticiones = NRepeticiones + 1
            End If
          Next I
          If NRepeticiones > 1 And PB(1) <> PB(2) Then 'SI EXISTE MS DE UN PUNTO, ES DECIR HAY UN SEGMENTO VERTICAL Y ESTE
                                                       'NO ES PQ B=C
            Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                      "existe ya un segmento vertical definido por dos puntos extendidos."
            EsValidoPto = False
            Exit Function
          ElseIf NRepeticiones > 2 And PB(1) = PB(2) Then 'SI NOS ENCONTRAMOS ANTE LA SITUACIN DE QUE B=C Y QUE PARA DICHO
                                                          'X DONDE B=C EXISTE ADEMS ALGN PTO EXTENDIDO YA NO ES POSIBLE
                                                          'INTRODUCIR MS.
            Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                      "existe ya un segmento vertical definido."
            EsValidoPto = False
          Else
            GoTo VALIDO_EX
          End If
        End If
      'SI EL PUNTO A COMPROBAR ES IGUAL A SU MXIMO
      ElseIf P_X = PtoSeleccionado.xMax Then
        'EN EL CASO DE QUE SU PTO LIMITADOR MXIMO TENGA LA MISMA ALTURA EL PUNTO NO SER VLIDO PUES SERAN COINCIDENTES
        If P_Y = PtoSeleccionado.yMax Then
          Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                    "existe un punto con las mismas coordenadas."
          EsValidoPto = False
          Exit Function
        Else 'SI SUS ALTURAS SON DISTINTAS PASA A SER UN POSIBLE!! PTO VLIDO
          'SER VLIDO SI NO EXISTEN PARA DICHA X MS DE UN PUNTO, ES DECIR, AL VALIDAR EL PUNTO ESTE FORMARA UN SEGMENTO VERTICAL
          'CON EL PUNTO YA EXISTENTE PARA DICHO X PERO SI EXISTIAN DE ANTES DOS EXISTIRIAN AL VALIDARLO 3 PUNTOS DANDO REDUNDANCIA
          'A LA DEFINICIN DE UN SEGMENTO. NOTA!! HAY QUE PERMITIR EL CASO DE B=C Y OTRO P.EXTENDIDO QUE FORMAN UN SEGMENTO VERTICAL
          '(EXCEPCIN DEL CASO DE REDUNDANCIA EN LA DEFINICIN DE UN SEGMENTO MEDIANTE 3 PTOS).
          ReordenarPByE PB(), PE(), MPBYE()
          NRepeticiones = 0
          For I = 0 To UBound(MPBYE(), 1) - 1 Step 2
            If MPBYE(I) = PtoSeleccionado.xMax Then
              NRepeticiones = NRepeticiones + 1
            End If
          Next I
          If NRepeticiones > 1 And PB(1) <> PB(2) Then 'SI EXISTE MS DE UN PUNTO, ES DECIR HAY UN SEGMENTO VERTICAL Y ESTE
                                                       'NO ES PQ B=C
            Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                      "existe ya un segmento vertical definido por dos puntos extendidos."
            EsValidoPto = False
            Exit Function
          ElseIf NRepeticiones > 2 And PB(1) = PB(2) Then 'SI NOS ENCONTRAMOS ANTE LA SITUACIN DE QUE B=C Y QUE PARA DICHO
                                                          'X DONDE B=C EXISTE ADEMS ALGN PTO EXTENDIDO YA NO ES POSIBLE
                                                          'INTRODUCIR MS.
            Mensaje = "El punto E(" & Round(P_X, NUMDEC) & ":" & Round(P_Y, NUMDEC) & ") NO ES VLIDO pues " & vbCr & vbLf & _
                      "existe ya un segmento vertical definido."
            EsValidoPto = False
          Else
            GoTo VALIDO_EX
          End If
        End If
      'SI NO SUPERA NI IGUALA A LOS LIMITADORES PASA A SER UN PUNTO VLIDO
      Else
VALIDO_EX:
        Mensaje = ""
        EsValidoPto = True
        Exit Function
      End If
    
  End Select
  
End Function
