Attribute VB_Name = "Rules"
'-------------------------------------------------------------------------------------------------------
'                                    MDULO DE CDIGO : RULES.BAS
'
' OBJETIVO: Contener el cdigo (propiedades, funciones, procedimientos, vbles, ctes,...)
'           necesarias para procesar la informacin acerca de las reglas del sistema.
'-------------------------------------------------------------------------------------------------------

Option Explicit


'---------------------------------
' --- DECLARACIN DE VARIABLES ---
'---------------------------------

Public HayQEliminarRegla As Boolean
Public HayReglaEliminada As Boolean
Public SuprimidaRegla As TBCONOCIMIENTO
Public NuevaRegla As TBCONOCIMIENTO
Public SuprimidaPosicion As Integer
                                                                                                        
Public AoSemi As ETESTADO
Public NEPasivas As Integer
Public NSPasivas As Integer
Public EPasivas() As Integer
Public SPasivas() As Integer

'---------------------------------
' - DECLARACIN DE ENUMERACIONES -
'---------------------------------
Public Enum ETAOC
  Antecedente = 0
  Consecuente = 1
End Enum


                                '------------------------------------------------------------------
                                ' BLOQUE [PROPIEDADES DE LECTURA DERIVADAS DE UNA REGLA]
                                '
                                ' OBJETIVO:
                                '------------------------------------------------------------------

'**************************************************************************************************
'   PROPIEDAD: NReglas() -> Indica el n de reglas que hay en el sistema.
'**************************************************************************************************
Public Property Get NReglas() As Integer
  On Error GoTo Fallo
  Dim I As Integer
  
  I = UBound(SistActual.Regla(), 1)
  NReglas = I + 1
  Exit Property
Fallo:
  If Err.Number = 9 Then
    NReglas = 0
    Err.Clear
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD: NReglasActivas() -> Indica el n de reglas activas que hay en el sistema.
'**************************************************************************************************
Public Property Get NReglasActivas() As Integer
  Dim I As Integer, j As Integer
    
  For I = 1 To NReglas
    If SistActual.Regla(I - 1).Estado = Activo Then
      j = j + 1
    End If
  Next I
  NReglasActivas = j
  
End Property

'**************************************************************************************************
'   PROPIEDAD: NReglasSemiActivas() -> Indica el n de reglas semi-activas que hay en el sistema.
'**************************************************************************************************
Public Property Get NReglasSemiActivas() As Integer
  Dim I As Integer, j As Integer
    
  For I = 1 To NReglas
    If SistActual.Regla(I - 1).Estado = SemiActivo Then
      j = j + 1
    End If
  Next I
  NReglasSemiActivas = j
  
End Property

'**************************************************************************************************
'   PROPIEDAD: NReglasPasivas() -> Indica el n de reglas desactivas en el sistema.
'**************************************************************************************************
Public Property Get NReglasPasivas() As Integer
  
  NReglasPasivas = NReglas - (NReglasActivas + NReglasSemiActivas)
  
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get NReglasParaBCCompleto() As Integer
  Dim I As Integer
  NReglasParaBCCompleto = 1
  For I = 1 To NEntradas
     NReglasParaBCCompleto = NReglasParaBCCompleto * NConjDif(E, I)
  Next I
 
End Property

'**************************************************************************************************
'   PROPIEDAD: NAntecedentes() ->
'**************************************************************************************************
Public Property Get NAntecedentes(IdRegla As Integer) As Integer
  On Error GoTo Fallo
  Dim I As Integer
  
  I = UBound(SistActual.Regla(IdRegla).Antecedente(), 1)
  NAntecedentes = I + 1
  Exit Property
Fallo:
  If Err.Number = 9 Then
    NAntecedentes = 0
    Err.Clear
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD: NConsecuentes() -> Indica el n de trminos en el consecuente de la regla indicada
'                                 por IdRegla.
'**************************************************************************************************
Public Property Get NConsecuentes(IdRegla As Integer) As Integer
  On Error GoTo Fallo
  Dim I As Integer
  
  I = UBound(SistActual.Regla(IdRegla).Consecuente(), 1)
  NConsecuentes = I + 1
  Exit Property
Fallo:
  If Err.Number = 9 Then
    NConsecuentes = 0
    Err.Clear
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD: NTerminosRegla() -> Agrupa la funcionalidad de las dos propiedades anteriores
'                                  NAntecedentes() y NConsecuentes().
'**************************************************************************************************
Public Property Get NTerminosRegla(AoC As ETAOC, IdRegla As Integer) As Integer
  On Error GoTo Fallo
  Dim I As Integer
  
  Select Case AoC
    Case Antecedente:
      I = UBound(SistActual.Regla(IdRegla).Antecedente(), 1)
    Case Consecuente:
      I = UBound(SistActual.Regla(IdRegla).Consecuente(), 1)
  End Select
  NTerminosRegla = I + 1
  Exit Property
Fallo:
  If Err.Number = 9 Then
    NTerminosRegla = 0
    Err.Clear
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD: QAntecedente() -> Devuelve la cadena tras tomar los campos contenidos en la
'                                regla (IdRegla) y trmino (IdAntecedente).
'
'                                Campos: Nombre,Op.Not y Etiqueta.
'**************************************************************************************************
Public Property Get QAntecedente(IdRegla As Integer, IdAntecedente As Integer) As String
  Dim Temp As String
  
  Temp = UCase$(Trim$(SistActual.Regla(IdRegla).Antecedente(IdAntecedente).NombreEoS))
  If SistActual.Regla(IdRegla).Antecedente(IdAntecedente).OpNot = True Then
    Temp = Temp & " NO ES "
  Else
    Temp = Temp & " ES "
  End If
  Temp = Temp & UCase$(Trim$(SistActual.Regla(IdRegla).Antecedente(IdAntecedente).EtiquetaConjDif))
  QAntecedente = Temp
End Property

'**************************************************************************************************
'   PROPIEDAD: QConsecuente() -> Devuelve la cadena tras tomar los campos contenidos en la
'                                regla (IdRegla) y trmino (IdConsecuente).
'
'                                Campos: Nombre,Op.Not y Etiqueta.
'**************************************************************************************************
Public Property Get QConsecuente(IdRegla As Integer, IdConsecuente As Integer) As String
  Dim Temp As String
  
  Temp = UCase$(Trim$(SistActual.Regla(IdRegla).Consecuente(IdConsecuente).NombreEoS))
  If SistActual.Regla(IdRegla).Consecuente(IdConsecuente).OpNot = True Then
    Temp = Temp & " NO ES "
  Else
    Temp = Temp & " ES "
  End If
  Temp = Temp & UCase$(Trim$(SistActual.Regla(IdRegla).Consecuente(IdConsecuente).EtiquetaConjDif))
  QConsecuente = Temp
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get QEstadoAntecedente(IdRegla As Integer, IdAntecedente As Integer) As String
  Dim IdE As Integer
  IdE = QIdEoS(E, SistActual.Regla(IdRegla).Antecedente(IdAntecedente).NombreEoS)
  If SistActual.Entrada(IdE).Activa = Si Then
    QEstadoAntecedente = "TA"
  ElseIf SistActual.Entrada(IdE).Activa = No Then
    QEstadoAntecedente = "TP"
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get AntecedenteActivo(IdRegla As Integer, IdAntecedente As Integer) As String
  Dim IdE As Integer
  IdE = QIdEoS(E, SistActual.Regla(IdRegla).Antecedente(IdAntecedente).NombreEoS)
  If SistActual.Entrada(IdE).Activa = Si Then
    AntecedenteActivo = "Si"
  ElseIf SistActual.Entrada(IdE).Activa = No Then
    AntecedenteActivo = "No"
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get QEstadoConsecuente(IdRegla As Integer, IdConsecuente As Integer) As String
  Dim IdS As Integer
  IdS = QIdEoS(S, SistActual.Regla(IdRegla).Consecuente(IdConsecuente).NombreEoS)
  If SistActual.Salida(IdS).Activa = Si Then
    QEstadoConsecuente = "TA"
  ElseIf SistActual.Salida(IdS).Activa = No Then
    QEstadoConsecuente = "TP"
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get ConsecuenteActivo(IdRegla As Integer, IdConsecuente As Integer) As String
  Dim IdS As Integer
  IdS = QIdEoS(S, SistActual.Regla(IdRegla).Consecuente(IdConsecuente).NombreEoS)
  If SistActual.Salida(IdS).Activa = Si Then
    ConsecuenteActivo = "Si"
  ElseIf SistActual.Salida(IdS).Activa = No Then
    ConsecuenteActivo = "No"
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get QOperador(Tipo As ETOPERADOR) As String
  Dim Cadena As String
  Select Case Tipo
    Case AAnd:
      Cadena = "Y"
    Case OOr:
      Cadena = "O"
  End Select
  QOperador = Cadena
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get OpNotActivado(Tipo As Boolean) As String
  If Tipo = True Then
    OpNotActivado = "Si"
  Else
    OpNotActivado = "No"
  End If
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get ExpresionRegla(IdRegla As Integer) As String
  Dim Expresion As String
  Dim NTerminos  As Integer, IdEoS As Integer, IdEtiqueta As Integer, I As Integer
  
  '------------- FORMACIN DE LA EXPRESIN DEL ANTECEDENTE -------------
  Expresion = "SI "
  
  NTerminos = NAntecedentes(IdRegla)
  For I = 0 To NTerminos - 1
    IdEoS = QIdEoS(E, SistActual.Regla(IdRegla).Antecedente(I).NombreEoS)
    IdEtiqueta = QIdEtiqueta(E, IdEoS, SistActual.Regla(IdRegla).Antecedente(I).EtiquetaConjDif)
    If I = NTerminos - 1 Then
      Expresion = Expresion & QAntecedente(IdRegla, I) & " "
    Else
      Expresion = Expresion & QAntecedente(IdRegla, I) & " " & QOperador(SistActual.Regla(IdRegla).Operador) & " "
    End If
  Next I
  
  '------------- FORMACIN DE LA EXPRESIN DEL CONSECUENTE -------------
  Expresion = Expresion & "ENTONCES "
  
  NTerminos = NConsecuentes(IdRegla)
  For I = 0 To NTerminos - 1
    IdEoS = QIdEoS(S, SistActual.Regla(IdRegla).Consecuente(I).NombreEoS)
    IdEtiqueta = QIdEtiqueta(S, IdEoS, SistActual.Regla(IdRegla).Consecuente(I).EtiquetaConjDif)
    If I = NTerminos - 1 Then
      Expresion = Expresion & QConsecuente(IdRegla, I) & " "
    Else
      Expresion = Expresion & QConsecuente(IdRegla, I) & " " & QOperador(AAnd) & " "
    End If
  Next I
  
  ExpresionRegla = Expresion
End Property

'**************************************************************************************************
'   PROCEDIMIENTO:
'**************************************************************************************************
Public Sub ExpresionDescompuestaRegla(IdRegla As Integer, Optional ExpresionAnt As String, Optional ExpresionCons As String)
  Dim NTerminos  As Integer, IdEoS As Integer, IdEtiqueta As Integer, I As Integer
  
  '------------- FORMACIN DE LA EXPRESIN DEL ANTECEDENTE -------------
  ExpresionAnt = "<SI>"
  
  NTerminos = NAntecedentes(IdRegla)
  For I = 0 To NTerminos - 1
    IdEoS = QIdEoS(E, SistActual.Regla(IdRegla).Antecedente(I).NombreEoS)
    IdEtiqueta = QIdEtiqueta(E, IdEoS, SistActual.Regla(IdRegla).Antecedente(I).EtiquetaConjDif)
    If I = NTerminos - 1 Then
      ExpresionAnt = ExpresionAnt & vbCrLf & QAntecedente(IdRegla, I) & " "
    Else
      ExpresionAnt = ExpresionAnt & vbCrLf & QAntecedente(IdRegla, I) & " " & QOperador(SistActual.Regla(IdRegla).Operador) & " "
    End If
  Next I
  
  '------------- FORMACIN DE LA EXPRESIN DEL CONSECUENTE -------------
  ExpresionCons = "<ENTONCES>"
  
  NTerminos = NConsecuentes(IdRegla)
  For I = 0 To NTerminos - 1
    IdEoS = QIdEoS(S, SistActual.Regla(IdRegla).Consecuente(I).NombreEoS)
    IdEtiqueta = QIdEtiqueta(S, IdEoS, SistActual.Regla(IdRegla).Consecuente(I).EtiquetaConjDif)
    If I = NTerminos - 1 Then
      ExpresionCons = ExpresionCons & vbCrLf & QConsecuente(IdRegla, I) & " "
    Else
      ExpresionCons = ExpresionCons & vbCrLf & QConsecuente(IdRegla, I) & " " & QOperador(AAnd) & " "
    End If
  Next I
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO:
'**************************************************************************************************
Public Sub ExpresionPossRegla(IdRegla As Integer, ExpresionAnt As String)
  Dim NTerminos  As Integer, IdE As Integer, IdEtiqueta As Integer, I As Integer
  Dim Ant As String
  
  '------------- FORMACIN DE LA EXPRESIN DEL ANTECEDENTE -------------
  ExpresionAnt = "<SI>"
  
  NTerminos = NAntecedentes(IdRegla)
  For I = 0 To NTerminos - 1
    IdE = QIdEoS(E, SistActual.Regla(IdRegla).Antecedente(I).NombreEoS)
    IdEtiqueta = QIdEtiqueta(E, IdE, SistActual.Regla(IdRegla).Antecedente(I).EtiquetaConjDif)
    Ant = QAntecedente(IdRegla, I)
    If I = NTerminos - 1 Then
      If SistActual.Regla(IdRegla).Antecedente(I).Activa = Si Then
        If SistActual.Regla(IdRegla).Antecedente(I).OpNot = True Then
          ExpresionAnt = ExpresionAnt & vbCrLf & Ant & "  -->  POSS = " & _
                                    Round(1# - SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC)
        Else
          ExpresionAnt = ExpresionAnt & vbCrLf & Ant & "  -->  POSS = " & _
                                    Round(SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC)
        
        End If
          
      Else
        If SistActual.Regla(IdRegla).Antecedente(I).OpNot = True Then
          ExpresionAnt = ExpresionAnt & vbCrLf & Ant & "  -->  POSS = " & _
                                   Round(1# - SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC) & " [variable PASIVA]"
        Else
          ExpresionAnt = ExpresionAnt & vbCrLf & Ant & "  -->  POSS = " & _
                                   Round(SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC) & " [variable PASIVA]"
        End If
      
      End If
    Else
      If SistActual.Regla(IdRegla).Antecedente(I).Activa = Si Then
        If SistActual.Regla(IdRegla).Antecedente(I).OpNot = True Then
          ExpresionAnt = ExpresionAnt & vbCrLf & Ant & " " & QOperador(SistActual.Regla(IdRegla).Operador) & "  --> POSS = " & _
                                  Round(1# - SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC)
        Else
          ExpresionAnt = ExpresionAnt & vbCrLf & Ant & " " & QOperador(SistActual.Regla(IdRegla).Operador) & "  --> POSS = " & _
                                  Round(SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC)
        End If
        
      Else
        If SistActual.Regla(IdRegla).Antecedente(I).OpNot = True Then
            ExpresionAnt = ExpresionAnt & vbCrLf & Ant & " " & QOperador(SistActual.Regla(IdRegla).Operador) & "  --> POSS = " & _
                                  Round(1# - SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC) & " [variable PASIVA]"
        Else
            ExpresionAnt = ExpresionAnt & vbCrLf & Ant & " " & QOperador(SistActual.Regla(IdRegla).Operador) & "  --> POSS = " & _
                                  Round(SistActual.Entrada(IdE).ConjDif(IdEtiqueta).Poss, NUMDEC) & " [variable PASIVA]"
        End If

      End If
    End If
  Next I
  
End Sub


'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get EstaEntradaEnAntecedente(IdRegla As Integer, Nombre As String) As Boolean
  Dim I As Integer
    For I = 1 To NAntecedentes(IdRegla)
      If UCase$(Trim$(SistActual.Regla(IdRegla).Antecedente(I - 1).NombreEoS)) = UCase$(Trim$(Nombre)) Then
        EstaEntradaEnAntecedente = True
        Exit Property
      End If
    Next I
    EstaEntradaEnAntecedente = False
    
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get EstaSalidaEnConsecuente(IdRegla As Integer, Nombre As String) As Boolean
  Dim I As Integer
    For I = 1 To NConsecuentes(IdRegla)
      If UCase$(Trim$(SistActual.Regla(IdRegla).Consecuente(I - 1).NombreEoS)) = UCase$(Trim$(Nombre)) Then
        EstaSalidaEnConsecuente = True
        Exit Property
      End If
    Next I
    EstaSalidaEnConsecuente = False
    
End Property

'**************************************************************************************************
'   PROPIEDAD:
'**************************************************************************************************
Public Property Get EstaEtiquetaEnConsecuente(IdRegla As Integer, NombreSalida As String, NombreEtiqueta As String) As Boolean
  Dim I As Integer
    For I = 1 To NConsecuentes(IdRegla)
      If UCase$(Trim$(SistActual.Regla(IdRegla).Consecuente(I - 1).NombreEoS)) = UCase$(Trim$(NombreSalida)) And _
         UCase$(Trim$(SistActual.Regla(IdRegla).Consecuente(I - 1).EtiquetaConjDif)) = UCase$(Trim$(NombreEtiqueta)) Then
        EstaEtiquetaEnConsecuente = True
        Exit Property
      End If
    Next I
    EstaEtiquetaEnConsecuente = False
    
End Property

'------------------------------------------------------------------
' BLOQUE [OPERACIONES SOBRE REGLAS]
'
' OBJETIVO:
'------------------------------------------------------------------


'**************************************************************************************************
'   PROCEDIMIENTO: AadirReglaMatriz()
'**************************************************************************************************
Public Sub AadirReglaMatriz(Matriz() As TBCONOCIMIENTO, Posicion As Integer, ReglaInsertar As TBCONOCIMIENTO)
  On Error Resume Next
  Dim I As Integer, MIndInicial As Integer, NSaltos As Integer, NElementosMatriz As Integer
  
  'Se calcula el nmero de elementos de la matriz
  I = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = I + 1
  End If
  
  MIndInicial = NElementosMatriz - 1
  If NElementosMatriz = 0 Then
    ReDim Matriz(0)
    Matriz(0) = ReglaInsertar
  ElseIf Posicion > MIndInicial Then
    ReDim Preserve Matriz(MIndInicial + 1)
    Matriz(Posicion) = ReglaInsertar
    Exit Sub
  ElseIf Posicion < 0 Then
    Posicion = 0
    GoTo Seguir
  Else
Seguir:
    ReDim Preserve Matriz(MIndInicial + 1)
    NSaltos = (MIndInicial + 1) - Posicion
    For I = 1 To NSaltos
      Matriz((MIndInicial + 2) - I) = Matriz((MIndInicial + 1) - I)
    Next I
    Matriz(Posicion) = ReglaInsertar
  End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: EliminarReglaMatriz()
'**************************************************************************************************
Public Sub EliminarReglaMatriz(Matriz() As TBCONOCIMIENTO, Posicion As Integer, ReglaSuprimida As TBCONOCIMIENTO)
  On Error Resume Next
  Dim I As Integer, NElementosMatriz As Integer
  'Se calcula el nmero de elementos de la matriz
  I = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = I + 1
  End If
  'Antes de proceder a su eliminacin este se almacena como informacin a reconstruir en caso de querer
  'el usuario deshacer la eliminacin de regla
  ReglaSuprimida = Matriz(Posicion)
  'Dado que la matriz comienza en el elemento Matriz(0) el ndice ira de 0 -> (NElementosMatriz -1)
  'SI LA MATRIZ CONTIENE UN SOLO ELEMENTO
  If NElementosMatriz = 1 Then
    'Se liberar memoria ocupada por la matriz dinmica Matriz()
    Erase Matriz()
  'SI LA MATRIZ CONTIENE MS DE UN ELEMENTO Y EL SELECCIONADO PARA ELIMINAR ES EL LTIMO
  ElseIf (Posicion = (NElementosMatriz - 1)) And NElementosMatriz > 1 Then
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  Else
    'Trasladamos los datos de la posicin i a i-1
    For I = Posicion + 1 To (NElementosMatriz - 1)
      Matriz(I - 1) = Matriz(I)
    Next I
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  End If
  HayReglaEliminada = True
  
End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: EliminarTerminosQTenganConjDif()
'**************************************************************************************************
Public Sub EliminarTerminosQTenganConjDif(EoS As ETEOS, IdEoS As Integer, IdConjDif As Integer)
    Dim I As Integer, j As Integer, PosiblesPosiciones() As Integer
    Dim Termino As String
    Dim NombreVble As String
    Dim ReglaSuprimida As TBCONOCIMIENTO
    
    If EoS = E Then
      For I = 0 To NReglas - 1
        NombreVble = QNombreEoS(E, IdEoS)
        If EstaEntradaEnAntecedente(I, NombreVble) Then
VOLVER_EMPEZAR_E:
          For j = 0 To NAntecedentes(I) - 1
            If UCase$(Trim$(SistActual.Entrada(IdEoS).ConjDif(IdConjDif).Etiqueta)) = UCase$(Trim$(SistActual.Regla(I).Antecedente(j).EtiquetaConjDif)) And _
               UCase$(Trim$(NombreVble)) = UCase$(Trim$(SistActual.Regla(I).Antecedente(j).NombreEoS)) Then
               
              Termino = QAntecedente(I, j)
              EliminarTerminoRegla SistActual.Regla(I).Antecedente(), j
              If AceptarTodoNA = False Then
                QAceptarTodo = NA
                MsgAdvertencia "Trmino eliminado en regla n " & I & ": " & vbCr & vbLf & vbCr & vbLf & "        " & Termino, "Aviso!: Eliminacin de trmino"
              End If
              If NAntecedentes(I) = 0 Then 'SI HEMOS ELIMINADO EL LTIMO TRMINO QUE CONTENIA LA REGLA EN SU
                                           'ANTECEDENTE LA REGLA YA NO TIENE SENTIDO Y SE PROCEDE A SU ELIMINACIN.
                MsgBox "A consecuencia de la eliminacin del nico trmino que contena la" & vbCr & vbLf & _
                       "regla n " & I & ", se procede a la eliminacin de la misma pues" & vbCr & vbLf & _
                       "sin al menos un trmino en su antecedente y/o consecuente la regla" & vbCr & vbLf & _
                       "no tiene sentido en la B.Conocimiento.", vbExclamation + vbOKOnly, "Eliminacin de Regla"
                EliminarReglaMatriz SistActual.Regla(), I, ReglaSuprimida
                Exit Sub
              Else
                'AL ELIMINAR UN TRMINO (NO TODOS) DE LA REGLA LO MS PROBABLE ES QUE AFECTE EN SU ESTADO POR TANTO
                'SE COMPRUEBA SI SU ESTADO ES EL CORRECTO.
                VerificarEstadoReglaAnteVariaciones I
                GoTo VOLVER_EMPEZAR_E
              End If
            End If
          Next j
        End If
      Next I
    ElseIf EoS = S Then
      For I = 0 To NReglas - 1
        NombreVble = QNombreEoS(S, IdEoS)
        If EstaSalidaEnConsecuente(I, NombreVble) Then
VOLVER_EMPEZAR_S:
          For j = 0 To NConsecuentes(I) - 1
            If UCase$(Trim$(SistActual.Salida(IdEoS).ConjDif(IdConjDif).Etiqueta)) = UCase$(Trim$(SistActual.Regla(I).Consecuente(j).EtiquetaConjDif)) And _
               UCase$(Trim$(NombreVble)) = UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) Then
               
              Termino = QConsecuente(I, j)
              EliminarTerminoRegla SistActual.Regla(I).Consecuente(), j
              If AceptarTodoNA = False Then
                QAceptarTodo = NA
                MsgAdvertencia "Trmino eliminado en regla n " & I & ": " & vbCr & vbLf & vbCr & vbLf & "        " & Termino, "Aviso!: Eliminacin de trmino"
              End If
              If NConsecuentes(I) = 0 Then 'SI HEMOS ELIMINADO EL LTIMO TRMINO QUE CONTENIA LA REGLA EN SU
                                           'CONSECUENTE LA REGLA YA NO TIENE SENTIDO Y SE PROCEDE A SU ELIMINACIN.
                MsgBox "A consecuencia de la eliminacin del nico trmino que contena la" & vbCr & vbLf & _
                       "regla n " & I & ", se procede a la eliminacin de la misma pues" & vbCr & vbLf & _
                       "sin al menos un trmino en su antecedente y/o consecuente la regla" & vbCr & vbLf & _
                       "no tiene sentido en la B.Conocimiento.", vbExclamation + vbOKOnly, "Eliminacin de Regla"
                EliminarReglaMatriz SistActual.Regla(), I, ReglaSuprimida
                Exit Sub
              Else
                'AL ELIMINAR UN TRMINO (NO TODOS) DE LA REGLA LO MS PROBABLE ES QUE AFECTE EN SU ESTADO POR TANTO
                'SE COMPRUEBA SI SU ESTADO ES EL CORRECTO.
                VerificarEstadoReglaAnteVariaciones I
                GoTo VOLVER_EMPEZAR_S
              End If
            End If
          Next j
        End If
      Next I
    End If
  
  AceptarTodoNA = False
  AceptarTodoNC = False
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: EliminarTerminosQTenganVariable()
'**************************************************************************************************
Public Sub EliminarTerminosQTenganVariable(EoS As ETEOS, IdEoS As Integer)
    Dim I As Integer, j As Integer, PosiblesPosiciones() As Integer
    Dim Termino As String
    Dim ReglaSuprimida As TBCONOCIMIENTO
    
    
    If EoS = E Then
VOLVER_EMPEZAR_BCONOCIMIENTO_E:
      For I = 0 To NReglas - 1
VOLVER_EMPEZAR_ANTECEDENTE:
        If EstaEntradaEnAntecedente(I, QNombreEoS(E, IdEoS)) Then
          For j = 0 To NAntecedentes(I) - 1
            If UCase$(Trim$(SistActual.Entrada(IdEoS).Nombre)) = UCase$(Trim$(SistActual.Regla(I).Antecedente(j).NombreEoS)) Then
              Termino = QAntecedente(I, j)
              EliminarTerminoRegla SistActual.Regla(I).Antecedente(), j
              If AceptarTodoNA = False Then
                QAceptarTodo = NA
                MsgAdvertencia "Trmino eliminado en regla n " & I & ": " & vbCr & vbLf & vbCr & vbLf & "        " & Termino, "Aviso!: Eliminacin de trmino"
              End If
              'AL ELIMINAR UN TRMINO (NO TODOS) DE LA REGLA LO MS PROBABLE ES QUE AFECTE EN SU ESTADO POR TANTO
              'SE COMPRUEBA SI SU ESTADO ES EL CORRECTO.
              VerificarEstadoReglaAnteVariaciones I
              If NAntecedentes(I) = 0 Then 'SI HEMOS ELIMINADO EL LTIMO TRMINO QUE CONTENIA LA REGLA EN SU
                                           'ANTECEDENTE LA REGLA YA NO TIENE SENTIDO Y SE PROCEDE A SU ELIMINACIN.
                MsgBox "A consecuencia de la eliminacin del nico trmino que contena la" & vbCr & vbLf & _
                       "regla n " & I & ", se procede a la eliminacin de la misma pues" & vbCr & vbLf & _
                       "sin al menos un trmino en su antecedente y/o consecuente la regla" & vbCr & vbLf & _
                       "no tiene sentido en la B.Conocimiento.", vbExclamation + vbOKOnly, "Eliminacin de Regla"
                EliminarReglaMatriz SistActual.Regla(), I, ReglaSuprimida
                GoTo VOLVER_EMPEZAR_BCONOCIMIENTO_E
              End If
              GoTo VOLVER_EMPEZAR_ANTECEDENTE
            End If
          Next j
        End If
      Next I
    
    ElseIf EoS = S Then

VOLVER_EMPEZAR_BCONOCIMIENTO_S:
      For I = 0 To NReglas - 1
VOLVER_EMPEZAR_CONSECUENTE:
        If EstaSalidaEnConsecuente(I, QNombreEoS(S, IdEoS)) Then
          For j = 0 To NConsecuentes(I) - 1
            If UCase$(Trim$(SistActual.Salida(IdEoS).Nombre)) = UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) Then
              Termino = QConsecuente(I, j)
              EliminarTerminoRegla SistActual.Regla(I).Consecuente(), j
              If AceptarTodoNA = False Then
                QAceptarTodo = NA
                MsgAdvertencia "Trmino eliminado en regla n " & I & ": " & vbCr & vbLf & vbCr & vbLf & "        " & Termino, "Aviso!: Eliminacin de trmino"
              End If
              'AL ELIMINAR UN TRMINO (NO TODOS) DE LA REGLA LO MS PROBABLE ES QUE AFECTE EN SU ESTADO POR TANTO
              'SE COMPRUEBA SI SU ESTADO ES EL CORRECTO.
              VerificarEstadoReglaAnteVariaciones I
              If NConsecuentes(I) = 0 Then  'SI HEMOS ELIMINADO EL LTIMO TRMINO QUE CONTENIA LA REGLA EN SU
                                           'ANTECEDENTE LA REGLA YA NO TIENE SENTIDO Y SE PROCEDE A SU ELIMINACIN.
                MsgBox "A consecuencia de la eliminacin del nico trmino que contena la" & vbCr & vbLf & _
                       "regla n " & I & ", se procede a la eliminacin de la misma pues" & vbCr & vbLf & _
                       "sin al menos un trmino en su antecedente y/o consecuente la regla" & vbCr & vbLf & _
                       "no tiene sentido en la B.Conocimiento.", vbExclamation + vbOKOnly, "Eliminacin de Regla"
                EliminarReglaMatriz SistActual.Regla(), I, ReglaSuprimida
                GoTo VOLVER_EMPEZAR_BCONOCIMIENTO_S
              End If
              GoTo VOLVER_EMPEZAR_CONSECUENTE
            End If
          Next j
        End If
      Next I
  
    End If
  AceptarTodoNA = False
  AceptarTodoNC = False
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: EliminarTerminoRegla()
'**************************************************************************************************
Public Sub EliminarTerminoRegla(Matriz() As TTERMINO, Posicion As Integer)
  On Error Resume Next
  Dim I As Integer, NElementosMatriz As Integer
  'Se calcula el nmero de elementos de la matriz
  I = UBound(Matriz(), 1)
  If Err.Number = 9 Then
    NElementosMatriz = 0
    Err.Clear
  Else
    NElementosMatriz = I + 1
  End If
  'Dado que la matriz comienza en el elemento Matriz(0) el ndice ira de 0 -> (NElementosMatriz -1)
  'SI LA MATRIZ CONTIENE UN SOLO ELEMENTO
  If NElementosMatriz = 1 Then
    'Se liberar memoria ocupada por la matriz dinmica Matriz()
    Erase Matriz()
  'SI LA MATRIZ CONTIENE MS DE UN ELEMENTO Y EL SELECCIONADO PARA ELIMINAR ES EL LTIMO
  ElseIf (Posicion = (NElementosMatriz - 1)) And NElementosMatriz > 1 Then
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  Else
    'Trasladamos los datos de la posicin i a i-1
    For I = Posicion + 1 To (NElementosMatriz - 1)
      Matriz(I - 1) = Matriz(I)
    Next I
    'Redimensionamos la matriz a un elemento menos conservando los existentes menos el ltimo
    ReDim Preserve Matriz(NElementosMatriz - 2)
  End If
End Sub
 
'**************************************************************************************************
'   PROCEDIMIENTO: ModificarEstadoTerminosPorVariacionEstadoEoS()
'**************************************************************************************************
Public Sub ModificarEstadoTerminosPorVariacionEstadoEoS(EoS As ETEOS, IdEoS As Integer, Activar As ETACTIVADO)
  Dim I As Integer, j As Integer
  Dim NombreEoS As String
  Dim Estado As String
  Dim Estadopuesto As ETACTIVADO
  
  If Activar = Si Then
    Estado = "ACTIVO"
    Estadopuesto = No
  ElseIf Activar = No Then
    Estado = "PASIVO"
    Estadopuesto = Si
  End If
  
  If EoS = E Then
    NombreEoS = UCase$(Trim$(QNombreEoS(E, IdEoS)))
    For I = 0 To NReglas - 1
      For j = 0 To NAntecedentes(I) - 1
        If UCase$(Trim$(SistActual.Regla(I).Antecedente(j).NombreEoS)) = NombreEoS And SistActual.Regla(I).Antecedente(j).Activa = Estadopuesto Then
          If AceptarTodoNB = False Then
            QAceptarTodo = NB
            MsgAdvertencia "TRMINO: " & "[" & QAntecedente(I, j) & "]" & " PASA A ESTADO >> " & "[" & Estado & "]" & " EN REGLA N " & I & ".", "Aviso!: Cambio de estado en trmino"
          End If
          SistActual.Regla(I).Antecedente(j).Activa = Activar
        End If
      Next j
      VerificarEstadoReglaAnteVariaciones (I)
    Next I
  ElseIf EoS = S Then
    NombreEoS = UCase$(Trim$(QNombreEoS(S, IdEoS)))
    For I = 0 To NReglas - 1
      For j = 0 To NConsecuentes(I) - 1
        If UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) = NombreEoS And SistActual.Regla(I).Consecuente(j).Activa = Estadopuesto Then
          If AceptarTodoNB = False Then
            QAceptarTodo = NB
            MsgAdvertencia "TRMINO: " & "[" & QConsecuente(I, j) & "]" & " PASA A ESTADO >> " & "[" & Estado & "]" & " EN REGLA N " & I & ".", "Aviso!: Cambio de estado en trmino"
          End If
          SistActual.Regla(I).Consecuente(j).Activa = Activar
        End If
      Next j
      VerificarEstadoReglaAnteVariaciones (I)
    Next I
  End If
  
  AceptarTodoNB = False
  AceptarTodoNC = False
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: VerificarEstadoReglaAnteVariaciones()
'**************************************************************************************************
Public Sub VerificarEstadoReglaAnteVariaciones(IdRegla As Integer)
  Dim I As Integer, NTAPasivos As Integer, NTCPasivos As Integer
  
  NTAPasivos = 0
  NTCPasivos = 0
  
  For I = 0 To NAntecedentes(IdRegla) - 1
    If SistActual.Regla(IdRegla).Antecedente(I).Activa = No Then NTAPasivos = NTAPasivos + 1
  Next I
  
  For I = 0 To NConsecuentes(IdRegla) - 1
    If SistActual.Regla(IdRegla).Consecuente(I).Activa = No Then NTCPasivos = NTCPasivos + 1
  Next I

  If NTAPasivos = NAntecedentes(IdRegla) Or NTCPasivos = NConsecuentes(IdRegla) Then
    If SistActual.Regla(IdRegla).Estado <> Pasivo Then
      If AceptarTodoNC = False Then
        QAceptarTodo = NC
        MsgAdvertencia "REGLA N " & IdRegla & " >> [DESACTIVADA]", "Aviso!: Cambio de estado en regla"
      End If
      SistActual.Regla(IdRegla).Estado = Pasivo
    End If
    
  ElseIf (NTAPasivos > 0 And NTAPasivos < NAntecedentes(IdRegla)) Or (NTCPasivos > 0 And NTCPasivos < NConsecuentes(IdRegla)) Then
    If SistActual.Regla(IdRegla).Estado <> SemiActivo Then
      If AceptarTodoNC = False Then
        QAceptarTodo = NC
        MsgAdvertencia "REGLA N " & IdRegla & " >> [SEMI-ACTIVADA]", "Aviso!: Cambio de estado en regla"
      End If
      SistActual.Regla(IdRegla).Estado = SemiActivo
    End If
  
  ElseIf NTAPasivos = 0 And NTCPasivos = 0 Then
    If SistActual.Regla(IdRegla).Estado <> Activo Then
      If AceptarTodoNC = False Then
        QAceptarTodo = NC
        MsgAdvertencia "REGLA N " & IdRegla & " >> [ACTIVADA]", "Aviso!: Cambio de estado en regla"
      End If
      SistActual.Regla(IdRegla).Estado = Activo
    End If
    
  End If
  
  'AceptarTodoNC = False AQU NO SE COLOCA PQ EN UNA LLAMADA DIRECTA DE ESTA FUNCIN NO LANZARIA EL
  '                       MENSAJE DE AVISO SI ANTERIORMENTE ESTA FUNCIN HA SIDO LLAMADA POR UNA COLA
  '                       DE FUNCIONES COMO MODICACIN DE TRMINOS,..., NO TIENE SENTIDO AQU PUES NO HAY BUCLE
  '                       EN CAMBIO SI HAY BUCLE EN LA FUNCIN ANTERIOR ModificarEstadoTerminosPorVariacionEstadoEoS y
  '                       EliminarTerminosQTenganConjDif,...
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: ModReglasVariacionNombre()
'**************************************************************************************************
Public Sub ModReglasVariacionNombre(EoS As ETEOS, NombreBuscar As String, NombreSustituir)
  Dim I As Integer, j As Integer
  
  If NReglas = 0 Then
    Exit Sub
  Else
    If EoS = E Then
      For I = 0 To NReglas - 1
        For j = 0 To NAntecedentes(I) - 1
          If UCase$(Trim$(SistActual.Regla(I).Antecedente(j).NombreEoS)) = UCase$(Trim$(NombreBuscar)) Then
            SistActual.Regla(I).Antecedente(j).NombreEoS = UCase$(Trim$(NombreSustituir))
          End If
        Next j
      Next I
    Else
      For I = 0 To NReglas - 1
        For j = 0 To NConsecuentes(I) - 1
          If UCase$(Trim$(SistActual.Regla(I).Consecuente(j).NombreEoS)) = UCase$(Trim$(NombreBuscar)) Then
            SistActual.Regla(I).Consecuente(j).NombreEoS = UCase$(Trim$(NombreSustituir))
          End If
        Next j
      Next I
    End If
  End If
      
End Sub

  
'**************************************************************************************************
'   PROCEDIMIENTO: ModReglasVariacionEtiquetaConjDif()
'**************************************************************************************************
Public Sub ModReglasVariacionEtiquetaConjDif(EoS As ETEOS, EtiquetaBuscar As String, EtiquetaSustituir)
  Dim I As Integer, j As Integer
  
  If NReglas = 0 Then
    Exit Sub
  Else
    If EoS = E Then
      For I = 0 To NReglas - 1
        For j = 0 To NAntecedentes(I) - 1
          If UCase$(Trim$(SistActual.Regla(I).Antecedente(j).EtiquetaConjDif)) = UCase$(Trim$(EtiquetaBuscar)) Then
            SistActual.Regla(I).Antecedente(j).EtiquetaConjDif = UCase$(Trim$(EtiquetaSustituir))
          End If
        Next j
      Next I
    ElseIf EoS = S Then
      For I = 0 To NReglas - 1
        For j = 0 To NConsecuentes(I) - 1
          If UCase$(Trim$(SistActual.Regla(I).Consecuente(j).EtiquetaConjDif)) = UCase$(Trim$(EtiquetaBuscar)) Then
            SistActual.Regla(I).Consecuente(j).EtiquetaConjDif = UCase$(Trim$(EtiquetaSustituir))
          End If
        Next j
      Next I
    End If
  End If
      
End Sub

'*********************************************************************************************************
'   FUNCIN: EsPosibleConstruirRegla()
'*********************************************************************************************************
Public Function EsPosibleConstruirRegla() As Boolean
  Dim I As Integer
  Dim HayAnt As Boolean, HayCon As Boolean
  
  If NEntradas > 0 And NSalidas > 0 Then
    For I = 0 To NEntradas - 1
      If NConjDif(E, I) > 0 Then
        HayAnt = True
        Exit For
      End If
    Next I
    For I = 0 To NSalidas - 1
      If NConjDif(S, I) - 1 Then
        HayCon = True
        Exit For
      End If
    Next I
    If HayAnt And HayCon Then
      EsPosibleConstruirRegla = True
    Else
      EsPosibleConstruirRegla = False
    End If
  Else
    EsPosibleConstruirRegla = False
  End If
              
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: EstaSalidaEnBConocimiento()
'**************************************************************************************************
Public Function EstaSalidaEnBConocimiento(IdS As Integer) As Boolean
  Dim I As Integer
  
  For I = 0 To NReglas - 1
    If EstaSalidaEnConsecuente(I, UCase$(Trim$(SistActual.Salida(IdS).Nombre))) Then
      EstaSalidaEnBConocimiento = True
      Exit Function
    End If
  Next I
  EstaSalidaEnBConocimiento = False
  
End Function
