Attribute VB_Name = "Segments"

Option Explicit

'-------------------------------------------------------------------------------------------------------
'                                  ## MDULO DE CDIGO: SEGMENTS.BAS ##
'
' OBJETIVO: CONTENER EL CONJUNTO DE PROCEDIMIENTOS Y FUNCIONES NECESARIAS PARA EL TRATAMIENTO
'           DE SEGMENTOS EN DOS DIMENSIONES.
'-------------------------------------------------------------------------------------------------------

Public Enum ETSEGACTUAL
  SegA = 0
  SegB = 1
End Enum

'**************************************************************************************************
'   FUNCIN: CoordYPtoCEnSegAB()
'**************************************************************************************************
Public Function CoordYPtoCEnSegAB(PTOAX As Double, PTOAY As Double, PTOBX As Double, PTOBY As Double, _
                                  PTOCX As Double, PTOCY As Double, IntersectaSegVertPtoCEnSegAB As Boolean)
  
  Dim NumPdte As Double, DenPdte As Double
  
  'SE COMPRUEBA SI EL PTO C INTERSECTA AL SEGMENTO A - B
  If PTOAX <= PTOBX Then
    If PTOCX < PTOAX Or PTOCX > PTOBX Then
      IntersectaSegVertPtoCEnSegAB = False
      Exit Function
    Else
      IntersectaSegVertPtoCEnSegAB = True
    End If
  Else
    If PTOCX < PTOBX Or PTOCX > PTOAX Then
      IntersectaSegVertPtoCEnSegAB = False
      Exit Function
    Else
      IntersectaSegVertPtoCEnSegAB = True
    End If
  End If
  
  
  NumPdte = PTOBY - PTOAY
  DenPdte = PTOBX - PTOAX
  
  If NumPdte = 0 Then         ' SEGMENTO HORIZONTAL
    CoordYPtoCEnSegAB = PTOAY
    Exit Function
  
  ElseIf DenPdte = 0 Then     ' SEGMENTO VERTICAL
    CoordYPtoCEnSegAB = PTOCY
    Exit Function
    
  Else                        ' SEGMENTO CON PDTE FINITA
    CoordYPtoCEnSegAB = PTOAY + (NumPdte / DenPdte) * (PTOCX - PTOAX)
  End If
  
  
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: PtoIntSegQSeCruzanUnPto()
'**************************************************************************************************
Public Sub PtoIntSegQSeCruzanUnPto(PTOAX As Double, PTOAY As Double, PTOBX As Double, PTOBY As Double, _
                                   PTOCX As Double, PTOCY As Double, PTODX As Double, PTODY As Double, _
                                   PTOINTX As Double, PTOINTY As Double)
  
    Dim EsVerticalSeg1 As Boolean, EsVerticalSeg2 As Boolean
    Dim EsHorizontSeg1 As Boolean, EsHorizontSeg2 As Boolean
    Dim EsPtoSeg1 As Boolean, EsPtoSeg2 As Boolean
    Dim Seg1EsPto As Boolean, Seg2EsPto As Boolean
    Dim Pdte1 As Double, Pdte2 As Double, PdteTemp As Double
    
    'Inicializacin
    EsHorizontSeg1 = False
    EsVerticalSeg1 = False
    EsPtoSeg1 = False
    
    EsHorizontSeg2 = False
    EsVerticalSeg2 = False
    EsPtoSeg2 = False
    
    'EN QUE SITUACIN SE ENCUENTRAN LOS SEGMENTOS?
    If PTOAX = PTOBX And Not PTOAY = PTOBY Then
      'x1=x2 => Pdte Seg 1 Infinita, es decir, Seg 1 VERTICAL
      EsVerticalSeg1 = True
    ElseIf PTOAY = PTOBY And Not PTOAX = PTOBX Then
      'y1=y2 => Pdte Seg 1 nula, es decir, Seg 1 HORIZONTAL
      EsHorizontSeg1 = True
    ElseIf PTOAX = PTOBX And PTOAY = PTOBY Then
      'Pto inicial y final =, es decir el segmento es un punto
      EsPtoSeg1 = True
    Else
      Pdte1 = (PTOBY - PTOAY) / (PTOBX - PTOAX)
    End If
    
    If PTOCX = PTODX And Not PTOCY = PTODY Then
      'x3=x4 => Pdte Seg 2 Infinita, es decir, Seg 2 VERTICAL
      EsVerticalSeg2 = True
    ElseIf PTOCY = PTODY And Not PTOCX = PTODX Then
      'y3=y4 => Pdte Seg 2 nula, es decir, Seg 2 HORIZONTAL
      EsHorizontSeg2 = True
    ElseIf PTOCX = PTODX And PTOCY = PTODY Then
      'Pto inicial y final =, es decir el segmento es un punto
      EsPtoSeg2 = True
    Else
      Pdte2 = (PTODY - PTOCY) / (PTODX - PTOCX)
    End If
    
    'PRIMER CASO
    If EsVerticalSeg1 And EsHorizontSeg2 Then
      PTOINTX = PTOAX
      PTOINTY = PTOCY
      
    ElseIf EsVerticalSeg2 And EsHorizontSeg1 Then
      PTOINTX = PTOCX
      PTOINTY = PTOAY
    
    'SEGUNDO CASO
    ElseIf EsVerticalSeg1 And Not EsHorizontSeg2 And Not EsVerticalSeg2 And Not EsPtoSeg2 Then
      PTOINTX = PTOAX
      PTOINTY = Pdte2 * (PTOINTX - PTOCX) + PTOCY
      
    ElseIf EsVerticalSeg2 And Not EsHorizontSeg1 And Not EsVerticalSeg1 And Not EsPtoSeg1 Then
      PTOINTX = PTOCX
      PTOINTY = Pdte1 * (PTOINTX - PTOAX) + PTOAY
    
    'TERCER CASO
    ElseIf EsHorizontSeg1 And Not EsHorizontSeg2 And Not EsVerticalSeg2 And Not EsPtoSeg2 Then
      PTOINTY = PTOAY
      PTOINTX = (PTOINTY - PTOCY) / Pdte2 + PTOCX
    
    ElseIf EsHorizontSeg2 And Not EsHorizontSeg1 And Not EsVerticalSeg1 And Not EsPtoSeg1 Then
      PTOINTY = PTOCY
      PTOINTX = (PTOINTY - PTOAY) / Pdte1 + PTOAX
    
    'CUARTO CASO
    ElseIf EsPtoSeg1 Then
      PTOINTY = PTOAY
      PTOINTX = PTOAX
    
    'QUINTO CASO
    ElseIf EsPtoSeg2 Then
      PTOINTY = PTOCY
      PTOINTX = PTOCX
      
    Else
      If Pdte1 = Pdte2 Then
        Pdte1 = Pdte1 + EPSILON
      End If
      PTOINTX = ((PTOCY - PTOAY) - Pdte2 * PTOCX + Pdte1 * PTOAX) / (Pdte1 - Pdte2)
      PTOINTY = PTOAY + Pdte1 * (PTOINTX - PTOAX)
      
    End If
  
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: PtosCorteSegmentos()
'**************************************************************************************************
Public Sub PtoCorteSegmentos(PTOAX As Double, PTOAY As Double, PTOBX As Double, PTOBY As Double, _
                             PTOCX As Double, PTOCY As Double, PTODX As Double, PTODY As Double, _
                             PTOINTX As Double, PTOINTY As Double, _
                             PTOASEGINTX As Double, PTOASEGINTY As Double, PTOBSEGINTX As Double, PTOBSEGINTY As Double, _
                             HayPtoInt As Boolean, HaySegInt As Boolean)

    'Coordenadas Segmento 1
    '                       (Ax,Ay) #--------------# (Bx,By)
   
    
    'Coordenadas Segmento 2
    '                       (Cx,Cy) #--------------# (Dx,Dy)
   
   
    Dim EsVerticalSeg1 As Boolean, EsVerticalSeg2 As Boolean
    Dim EsHorizontSeg1 As Boolean, EsHorizontSeg2 As Boolean
    Dim Seg1EsPto As Boolean, Seg2EsPto As Boolean
    Dim Pdte1 As Double, Pdte2 As Double, PdteTemp As Double
    Dim TemptoX As Double, TemptoY As Double
    
    'Inicializacin
    EsHorizontSeg1 = False
    EsVerticalSeg1 = False
    
    EsHorizontSeg2 = False
    EsVerticalSeg2 = False
    
    Seg1EsPto = False
    Seg2EsPto = False
    
    HayPtoInt = False
    HaySegInt = False
    
    'EN QUE SITUACIN SE ENCUENTRAN LOS SEGMENTOS?
    If PTOAX = PTOBX And Not PTOAY = PTOBY Then
      'x1=x2 => Pdte Seg 1 Infinita, es decir, Seg 1 VERTICAL
      EsVerticalSeg1 = True
    ElseIf PTOAY = PTOBY And Not PTOAX = PTOBX Then
      'y1=y2 => Pdte Seg 1 nula, es decir, Seg 1 HORIZONTAL
      EsHorizontSeg1 = True
    ElseIf PTOAX = PTOBX And PTOAY = PTOBY Then
      'El Seg 1 es un PTO
      Seg1EsPto = True
    Else
      Pdte1 = (PTOBY - PTOAY) / (PTOBX - PTOAX)
    End If
    
    If PTOCX = PTODX Then
      'x3=x4 => Pdte Seg 2 Infinita, es decir, Seg 2 VERTICAL
      EsVerticalSeg2 = True
    ElseIf PTOCY = PTODY Then
      'y3=y4 => Pdte Seg 2 nula, es decir, Seg 2 HORIZONTAL
      EsHorizontSeg2 = True
    ElseIf PTOCX = PTODX And PTOCY = PTODY Then
     'El Seg 2 es un PTO
     Seg2EsPto = True
    Else
      Pdte2 = (PTODY - PTOCY) / (PTODX - PTOCX)
    End If
    
    '***************************
    'SITUACIONES ENTRE SEGMENTOS
    '***************************
    
    '+++++++++++++++++++++++++++++++++++++
    'SEGMENTO 1 ES UN PTO
    '+++++++++++++++++++++++++++++++++++++
    If Seg1EsPto And Not Seg2EsPto Then
      PdteTemp = (PTODY - PTOAY) / (PTODX - PTOAX)
      
      If Pdte2 <> PdteTemp Then
        'PTO (SEG 1) FUERA SEGMENTO 2
        Exit Sub
      
      ElseIf Pdte2 = PdteTemp Then
        'PTO (SEG 1) PERTENECE RECTA SEGMENTO 2
        If EsVerticalSeg2 Then
          'SEGMENTO 2 VERTICAL
          OrdenarMenorAMayorPtosSegmento PTOCX, PTOCY, PTODX, PTODY, False
          If PTOCY <= PTOAY <= PTODY Then
            'PTO (SEG 1) PERTENECIENTE SEGMENTO 2 VERTICAL
            HayPtoInt = True
            PTOINTX = PTOAX
            PTOINTY = PTOAY
          Else
            Exit Sub
          End If
        
        Else
          'SEGMENTO 2 HORIZONTAL O CUALQUIER OTRA PDTE
          OrdenarMenorAMayorPtosSegmento PTOCX, PTOCY, PTODX, PTODY, True
          If PTOCX <= PTOAX <= PTODX Then
            'PTO (SEG 1) PERTENECIENTE SEGMENTO 2
            HayPtoInt = True
            PTOINTX = PTOAX
            PTOINTY = PTOAY
          Else
            Exit Sub
          End If
        End If
      End If
      
    '+++++++++++++++++++++++++++++++++++++
    'SEGMENTO 2 ES UN PTO
    '+++++++++++++++++++++++++++++++++++++
    ElseIf Seg2EsPto And Not Seg1EsPto Then
      PdteTemp = (PTOBY - PTOCY) / (PTOBX - PTOCX)
      
      If Pdte1 <> PdteTemp Then
        'PTO (SEG 2) FUERA SEGMENTO 1
        Exit Sub
      
      ElseIf Pdte1 = PdteTemp Then
        'PTO (SEG 2) PERTENECE RECTA SEGMENTO 1
        If EsVerticalSeg1 Then
          'SEGMENTO 1 VERTICAL
          OrdenarMenorAMayorPtosSegmento PTOAX, PTOAY, PTOBX, PTOBY, False
          If PTOAY <= PTOCY <= PTOBY Then
            'PTO (SEG 2) PERTENECIENTE SEGMENTO 1 VERTICAL
            HayPtoInt = True
            PTOINTX = PTOCX
            PTOINTY = PTOCY
          Else
            Exit Sub
          End If
        
        Else
          'SEGMENTO 1 HORIZONTAL O CUALQUIER OTRA PDTE
          OrdenarMenorAMayorPtosSegmento PTOAX, PTOAY, PTOBX, PTOBY, True
          If PTOAX <= PTOCX <= PTOBX Then
            'PTO (SEG 2) PERTENECIENTE SEGMENTO 1
            HayPtoInt = True
            PTOINTX = PTOCX
            PTOINTY = PTOCY
          Else
            Exit Sub
          End If
        End If
      End If
      
    '+++++++++++++++++++++++++++++++++++++
    'SEGMENTOS Q SON PTOS
    '+++++++++++++++++++++++++++++++++++++
    ElseIf Seg1EsPto And Seg2EsPto Then
      If PTOAX = PTOCX Then
        HayPtoInt = True
        PTOINTX = PTOAX
        PTOINTY = PTOAY
      Else
        Exit Sub
      End If
      
    '+++++++++++++++++++++++++++++++++++++
    'SEGMENTOS VERTICALES
    '+++++++++++++++++++++++++++++++++++++
    ElseIf EsVerticalSeg1 And EsVerticalSeg2 Then
      
      OrdenarMenorAMayorPtosSegmento PTOAX, PTOAY, PTOBX, PTOBY, False
      OrdenarMenorAMayorPtosSegmento PTOCX, PTOCY, PTODX, PTODY, False
      
      
      If PTOAY < PTOCY And PTOBY < PTODY Then
      'SEGMENTO 1 INFERIOR SEGMENTO 2
        If PTOCY > PTOBY Then
          'NO HAY PTO NI SEGMENTO DE INTERSECCIN
          Exit Sub
        ElseIf PTOCY = PTOBY Then
          'HAY PTO INTERSECCION
          HayPtoInt = True
          PTOINTX = PTOCX
          PTOINTY = PTOCY
        ElseIf PTOCY < PTOBY Then
          'HAY SEGMENTO DE INTERSECCIN
          HaySegInt = True
          PTOASEGINTX = PTOCX
          PTOASEGINTY = PTOCY
          PTOBSEGINTX = PTOBX
          PTOBSEGINTY = PTOBY
          OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
        End If
      
      ElseIf PTOAY > PTOCY And PTOBY > PTODY Then
      'SEGMENTO 2 INFERIOR SEGMENTO 1
        If PTOAY > PTODY Then
          'NO HAY PTO NI SEGMENTO DE INTERSECCIN
          Exit Sub
        ElseIf PTOAY = PTODY Then
          'HAY PTO INTERSECCION
          HayPtoInt = True
          PTOINTX = PTOAX
          PTOINTY = PTOAY
        ElseIf PTOAY < PTODY Then
          'HAY SEGMENTO DE INTERSECCIN
          HaySegInt = True
          PTOASEGINTX = PTOAX
          PTOASEGINTY = PTOAY
          PTOBSEGINTX = PTODX
          PTOBSEGINTY = PTODY
          OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
        End If
      
      ElseIf PTOAY = PTOCY And PTOBY = PTODY Then
      'SEGMENTO 1 Y 2 COINCIDENTES
        HaySegInt = True
        PTOASEGINTX = PTOAX
        PTOASEGINTY = PTOAY
        PTOBSEGINTX = PTOBX
        PTOBSEGINTY = PTOBY
        OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
      
      ElseIf PTOAY > PTOCY And PTOBY < PTODY Then
      'SEGMENTO 1 INCLUIDO EN SEGMENTO 2
        HaySegInt = True
        PTOASEGINTX = PTOAX
        PTOASEGINTY = PTOAY
        PTOBSEGINTX = PTOBX
        PTOBSEGINTY = PTOBY
        OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
      
      ElseIf PTOAY < PTOCY And PTOBY > PTODY Then
      'SEGMENTO 2 INCLUIDO EN SEGMENTO 1
        HaySegInt = True
        PTOASEGINTX = PTOCX
        PTOASEGINTY = PTOCY
        PTOBSEGINTX = PTODX
        PTOBSEGINTY = PTODY
        OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
      
      End If
    
    '+++++++++++++++++++++++++++++++++++++
    'SEGMENTOS HORIZONTALES (O)
    '          PDTES IGUALES
    '+++++++++++++++++++++++++++++++++++++
    ElseIf (EsHorizontSeg1 And EsHorizontSeg2) Or Pdte1 = Pdte2 Then
      
      OrdenarMenorAMayorPtosSegmento PTOAX, PTOAY, PTOBX, PTOBY, True
      OrdenarMenorAMayorPtosSegmento PTOCX, PTOCY, PTODX, PTODY, True
      
      
      If PTOAX < PTOCX And PTOBX < PTODX Then
      'SEGMENTO 1 INFERIOR SEGMENTO 2
        If PTOCX > PTOBX Then
          'NO HAY PTO NI SEGMENTO DE INTERSECCIN
          Exit Sub
        ElseIf PTOCX = PTOBX Then
          'HAY PTO INTERSECCION
          HayPtoInt = True
          PTOINTX = PTOCX
          PTOINTY = PTOCY
        ElseIf PTOCX < PTOBX Then
          'HAY SEGMENTO DE INTERSECCIN
          HaySegInt = True
          PTOASEGINTX = PTOCX
          PTOASEGINTY = PTOCY
          PTOBSEGINTX = PTOBX
          PTOBSEGINTY = PTOBY
          OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
        End If
      
      ElseIf PTOAX > PTOCX And PTOBX > PTODX Then
      'SEGMENTO 2 INFERIOR SEGMENTO 1
        If PTOAX > PTODX Then
          'NO HAY PTO NI SEGMENTO DE INTERSECCIN
          Exit Sub
        ElseIf PTOAX = PTODX Then
          'HAY PTO INTERSECCION
          HayPtoInt = True
          PTOINTX = PTOAX
          PTOINTY = PTOAY
        ElseIf PTOAX < PTODX Then
          'HAY SEGMENTO DE INTERSECCIN
          HaySegInt = True
          PTOASEGINTX = PTOAX
          PTOASEGINTY = PTOAY
          PTOBSEGINTX = PTODX
          PTOBSEGINTY = PTODY
          OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
        End If
      
      ElseIf PTOAX = PTOCX And PTOBX = PTODX Then
      'SEGMENTO 1 Y 2 COINCIDENTES
        HaySegInt = True
        PTOASEGINTX = PTOAX
        PTOASEGINTY = PTOAY
        PTOBSEGINTX = PTOBX
        PTOBSEGINTY = PTOBY
        OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
      
      ElseIf PTOAX > PTOCX And PTOBX < PTODX Then
      'SEGMENTO 1 INCLUIDO EN SEGMENTO 2
        HaySegInt = True
        PTOASEGINTX = PTOAX
        PTOASEGINTY = PTOAY
        PTOBSEGINTX = PTOBX
        PTOBSEGINTY = PTOBY
        OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
      
      ElseIf PTOAX < PTOCX And PTOBX > PTODX Then
      'SEGMENTO 2 INCLUIDO EN SEGMENTO 1
        HaySegInt = True
        PTOASEGINTX = PTOCX
        PTOASEGINTY = PTOCY
        PTOBSEGINTX = PTODX
        PTOBSEGINTY = PTODY
        OrdenarMenorAMayorPtosSegmento PTOASEGINTX, PTOASEGINTY, PTOBSEGINTX, PTOBSEGINTY, True
      
      End If
      
    '+++++++++++++++++++++++++++++++++++++
    'SEGMENTOS PDTES DISTINTAS
    '+++++++++++++++++++++++++++++++++++++
    ElseIf Pdte1 <> Pdte2 Then
      
      'PUNTO X INTERSECCIN ENTRE RECTAS
      TemptoX = ((Pdte1 * PTOAX) - PTOAY - (Pdte2 * PTOCX) + PTOCY) / (Pdte1 - Pdte2)
      'PUNTO Y INTERSECCIN ENTRE RECTAS
      TemptoY = Pdte1 * (TemptoX - PTOAX) + PTOAY
      
      OrdenarMenorAMayorPtosSegmento PTOAX, PTOAY, PTOBX, PTOBY, True
      OrdenarMenorAMayorPtosSegmento PTOCX, PTOCY, PTODX, PTODY, True
      If PTOAX <= TemptoX <= PTOBX And PTOCX <= TemptoX <= PTODX Then
        'SI EL PUNTO X INTERSECCION ENTRE RECTAS EST COMPRENDIDO ENTRE EL SEG 1 y SEG 2
        OrdenarMenorAMayorPtosSegmento PTOAX, PTOAY, PTOBX, PTOBY, False
        OrdenarMenorAMayorPtosSegmento PTOCX, PTOCY, PTODX, PTODY, False
        'SI EL PUNTO Y INTERSECCION ENTRE RECTAS EST COMPRENDIDO ENTRE EL SEG 1 y SEG 2
        If PTOAY <= TemptoY <= PTOBY And PTOCY <= TemptoY <= PTODY Then
          
          'EN ESTA SITUACIN EL PTO INTERSECCIN (X,Y) EST COMPRENDIDO ENTRE LOS SEG 1 y SEG 2
          'POR TANTO EXISTE UN PUNTO DE INTERSECCIN ENTRE LOS SEG 1 y SEG 2 Y ES EL PTO INTERSECCIN
          'CALCULADO ANTERIORMENTE
          HayPtoInt = True
          PTOINTX = TemptoX
          PTOINTY = TemptoY
        
        Else
          Exit Sub
        
        End If
      
      Else
        Exit Sub
        
      End If
        
     
    End If
    

End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: OrdenarMenorAMayorPtosSegmento()
'**************************************************************************************************
Private Sub OrdenarMenorAMayorPtosSegmento(AX As Double, AY As Double, BX As Double, BY As Double, Optional OrdenarSegunX As Boolean = True)
  
  If OrdenarSegunX Then
    If AX > BX Then
      AX = BX
      AY = BY
      BX = AX
      BY = AY
    End If
    
  Else
    If AY > BY Then
      AX = BX
      AY = BY
      BX = AX
      BY = AY
    End If
  End If
  
End Sub
