Attribute VB_Name = "SubMain"

Option Explicit

'-------------------------------------------------------------------------------------------------------
'                                  ## MDULO DE CDIGO: SUBMAIN.BAS ##
'
' OBJETIVO: Contener el cdigo necesario para la carga e inicializacin de la aplicacin.
'-------------------------------------------------------------------------------------------------------

'---------------------------------
' - DECLARACIN DE ENUMERACIONES -
'---------------------------------

'---------------------------------
' -- DECLARACIN DE CONSTANTES --
'---------------------------------
Public Const SECCION_RECIENTES = "RECIENTES"

Public Const CLAVE_USUARIOINI = "USUARIO#"
Public Const CLAVE_TITULOINI = "TITULO#"
Public Const CLAVE_ARCHIVOINI = "ARCHIVO#"

Public Const SECCION_GESTION_MEMORIA = "GESTIN DE MEMORIA"

Public Const CLAVE_NUMERO_MAX_VENTANAS_ABIERTAS_ENTRADA = "N MAX. VENTANAS ENTRADA"
Public Const NUMERO_MAX_VENTANAS_ABIERTAS_SALIDA_PORDEFECTO = 2

Public Const CLAVE_NUMERO_MAX_VENTANAS_ABIERTAS_SALIDA = "N MAX. VENTANAS SALIDA"
Public Const NUMERO_MAX_VENTANAS_ABIERTAS_ENTRADA_PORDEFECTO = 2


Public Const SECCION_GRAFICOS = "GRFICOS"
Public Const CLAVE_NUM_DIV_HORIZ = "N DIVISIONES HORIZONTALES"
Public Const NUMERO_DIVISIONES_HORIZONTALES_PORDEFECTO = 4

Public Const CLAVE_NUM_DIV_VERTI = "N DIVISIONES VERTICALES"
Public Const NUMERO_DIVISIONES_VERTICALES_PORDEFECTO = 10

'---------------------------------
' ----- DECLARACIN DE TIPOS -----
'---------------------------------
Public Type TOPCIONESINICIO
  Archivo(1 To 5) As String
  Usuario(1 To 5) As String * 50
  Titulo(1 To 5) As String * 80
  'N Mx. de posibles ventanas abiertas
  'simultaneamente de entrada y salida.
  NMaxVEntrada As Integer
  NMaxVSalida As Integer
  'N divisiones verticales y horizontales
  'para formar cuadrcula de grficos.
  NDivisionesHorizontales As Integer
  NDivisionesVerticales As Integer
  
End Type

'---------------------------------
' --- DECLARACIN DE VARIABLES ---
'---------------------------------
'MBITO GLOBAL
Public OpcionesInicio As TOPCIONESINICIO


'**************************************************************************************************
'   PROCEDIMIENTO: CargarOpcionesInicializacion()
'
'       OBJETIVO: Almacenar en memoria las opciones de inicio del fichero de
'                 inicializacin (si existe).
'
'**************************************************************************************************
Private Sub CargarOpcionesInicializacion()
  Dim I As Integer, Cadena As String, Ruta As String
  
  'Comprobacin de existencia del fichero de inicializacin
  Ruta = App.Path & "\" & SIGLA & "." & EXTENSION_ARCHIVO_INICIALIZACION
  If Not ExisteFichero(Ruta, False) Then
   RegistrarIncidencias "Inicializacin", "No se encuentra el archivo de inicializacin " & SIGLA & "." & _
                                           EXTENSION_ARCHIVO_INICIALIZACION & ", por tanto se procede a su generacin por defecto"
   GenerarDefectoINI
  End If
  
  'LECTURA DE DATOS
  
  '-- SECCION RECIENTES --
  For I = 1 To 5
    OpcionesInicio.Archivo(I) = LeerINI(Ruta, SECCION_RECIENTES, CLAVE_ARCHIVOINI & I, "?")
    OpcionesInicio.Usuario(I) = LeerINI(Ruta, SECCION_RECIENTES, CLAVE_USUARIOINI & I, "?")
    OpcionesInicio.Titulo(I) = LeerINI(Ruta, SECCION_RECIENTES, CLAVE_TITULOINI & I, "?")
  Next I
  
  '-- SECCION GESTION MEMORIA --
  LeerNumeroINI Ruta, SECCION_GESTION_MEMORIA, CLAVE_NUMERO_MAX_VENTANAS_ABIERTAS_ENTRADA, _
                NUMERO_MAX_VENTANAS_ABIERTAS_ENTRADA_PORDEFECTO, OpcionesInicio.NMaxVEntrada
  
  LeerNumeroINI Ruta, SECCION_GESTION_MEMORIA, CLAVE_NUMERO_MAX_VENTANAS_ABIERTAS_SALIDA, _
                NUMERO_MAX_VENTANAS_ABIERTAS_SALIDA_PORDEFECTO, OpcionesInicio.NMaxVSalida
  
  
  '-- SECCION GRAFICOS --
  LeerNumeroINI Ruta, SECCION_GRAFICOS, CLAVE_NUM_DIV_VERTI, _
                NUMERO_DIVISIONES_VERTICALES_PORDEFECTO, OpcionesInicio.NDivisionesVerticales
  
  LeerNumeroINI Ruta, SECCION_GRAFICOS, CLAVE_NUM_DIV_HORIZ, _
                NUMERO_DIVISIONES_HORIZONTALES_PORDEFECTO, OpcionesInicio.NDivisionesHorizontales
  
End Sub



'**************************************************************************************************
'   PROCEDIMIENTO: GenerarDefectoINI()
'
'       OBJETIVO: Generar e inicializar a cadenas vacias las distintas opciones de inicio
'                 almacenadas en el fichero de inicializacin.
'
'**************************************************************************************************
Private Sub GenerarDefectoINI()
  Dim I As Integer, Ruta As String
  
  Ruta = App.Path & "\" & SIGLA & "." & EXTENSION_ARCHIVO_INICIALIZACION
  '-- BLOQUE FRM GENERAL RECIENTES --
  'Inicializan ficheros recientes a cadenas vacas
  For I = 1 To 5
    EscribirINI Ruta, SECCION_RECIENTES, CLAVE_ARCHIVOINI & I, ""
  Next I
  'Inicializan usuarios recientes a cadenas vacas
  For I = 1 To 5
    EscribirINI Ruta, SECCION_RECIENTES, CLAVE_USUARIOINI & I, ""
  Next I
  'Inicializan ttulos recientes a cadenas vacas
  For I = 1 To 5
    EscribirINI Ruta, SECCION_RECIENTES, CLAVE_TITULOINI & I, ""
  Next I
  
  '-- SECION MEMORIA --
  EscribirINI Ruta, SECCION_GESTION_MEMORIA, CLAVE_NUMERO_MAX_VENTANAS_ABIERTAS_ENTRADA, _
              NUMERO_MAX_VENTANAS_ABIERTAS_ENTRADA_PORDEFECTO
                                                
  EscribirINI Ruta, SECCION_GESTION_MEMORIA, CLAVE_NUMERO_MAX_VENTANAS_ABIERTAS_SALIDA, _
                    NUMERO_MAX_VENTANAS_ABIERTAS_SALIDA_PORDEFECTO
                                                
  '-- SECCION GRFICOS --
  EscribirINI Ruta, SECCION_GRAFICOS, CLAVE_NUM_DIV_VERTI, NUMERO_DIVISIONES_VERTICALES_PORDEFECTO
                                                            
  EscribirINI Ruta, SECCION_GRAFICOS, CLAVE_NUM_DIV_HORIZ, NUMERO_DIVISIONES_HORIZONTALES_PORDEFECTO

 End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: RegistrarIncidencias()
'
'       OBJETIVO: Escribir sobre fichero (REGISTRO DE LA APLICACION) los mensajes que reciba
'                 de forma que cada nuevo mensaje generar una nueva incidencia. Cada seccin []
'                 corresponder al dia de uso de la aplicacin.
'
'**************************************************************************************************
Public Sub RegistrarIncidencias(Clave As String, Mensaje As String)
  Dim Ruta As String
  
  'Ubicacin del archivo de registro
  Ruta = App.Path & "\" & NOMBRE_ARCHIVO_REGISTRO & "." & EXTENSION_ARCHIVO_REGISTRO
  
  'Cabecera del fichero de registro
  EscribirINI Ruta, "REGISTRO DEL " & UCase$(NOMBRE_APLICACION_ESPAOL), "", _
  "============================================"
  
  'INCIDENCIA REGISTRADA SEGUN BLOQUE
  '[FECHA DE USO DE LA APLICACION]
  'CLASIFICADOR DE INCIDENCIA = << MENSAJE >>
  EscribirINI Ruta, Format$(Now, "ddd, dd-mmm-yy"), UCase$(Clave) & " ", " << " & UCase$(Mensaje) & " >>"
  
   
End Sub


'**************************************************************************************************
'   PROCEDIMIENTO: Main
'
'       OBJETIVO: DONDE TODO COMIENZA!
'
'                 - Establecer una pantalla de inicializacin de la aplicacin. En esta fase se
'                   cargarn en memoria los principales formularios de la misma de forma que
'                   la aplicacin cuando recurra a ellos su carga sea ms rpida y por tanto se
'                   aumente la velocidad de ejecucin de la aplicacin cuando se est trabajando
'                   junto a las opciones de inicio almacenadas en SCD.INI.
'
'**************************************************************************************************
Private Sub Main()
  
  'HABILITA VERSION DEMO, ES DECIR, LIMITA EL N DE ENTRADAS, SALIDAS Y REGLAS DE LA APLICACIN
  If StrComp(UCase$(Trim$(LineaComando)), LoadResString(101), vbTextCompare) = 0 Then
    VersionDemo = False
  Else
    'OJO EST PUESTO EN FALSO PARA QUE NO SE GENEREN VERSIONES DEMO
    VersionDemo = False
  End If
  
  'PROCESO DE CARGA DE FORMULARIOS DE LA APLICACIN, INICIALIZACIN DE OPCIONES...
  On Error GoTo ERROR_INICIALIZACION
  'Declaracin de varibles
  Dim iLlenado_barraprogresso As Integer
  
  'Ancho del rectngulo indicando el 100% de carga
  iLlenado_barraprogresso = 4500
  'Cursor se visualiza en modo espera mediante un reloj de arena
  Screen.MousePointer = vbHourglass
  
  
  'Visualiza la pantalla de presentacin y carga del formulario Inicializacion (flash form)
  FrmInicializacion.Show
  DoEvents
    
  '-------------- AHORA SE CARGAN LOS FORMULARIOS Y CDIGO QUE SE NECESITE INICIALIZAR --------------
  
  ' OPCIONES DE INICIO
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Inicio -> " & SIGLA & ".INI...]"
  CargarOpcionesInicializacion
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.15
  
  'IMPIDE DE COPIAS EXTRAS DE LA APLICACIN
  ImpedirArranqueCopiasAplicacion FrmMdi.Caption
  
  ' FORMULARIO ANLISIS
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Anlisis...]"
  Load FrmAnalisis
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.35
  ' FORMULARIO GENERAL
  FrmInicializacion.LblCargando = "Cargando: Mdulo [General...]"
  'ATENCIN! Dado que el primer formulario secundario en cargarse es FrmGeneral y previamente
  '           no ha sido cargado el correspondiente formulario principal FrmMDI este ser
  '           cargado primero, es decir, antes de cargarse un formulario secundario ser
  '           cargado el correspondiente primario.
  Load FrmGeneral
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.45
  ' FORMULARIO ENTRADA Y MODIFICADOR DE ENTRADA
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Entradas...]"
  Load FrmEntrada
  Load FrmEntradaModificar
  Load FrmSecuenciaEntrada
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.65
  ' FORMULARIO SALIDA Y MODIFICADOR DE SALIDA
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Salidas...]"
  Load FrmSalida
  Load FrmSalidaModificar
  Load FrmSecuenciaSalida
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.75
  ' FORMULARIO MOTOR DE INFERENCIA
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Inferencia...]"
  Load FrmOpcionesInferencia
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.85
  ' FORMULARIO REGLAS
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Reglas...]"
  Load FrmReglas
  DoEvents
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 0.95
  'FORMULARIO MDI
  FrmInicializacion.LblCargando = "Cargando: Mdulo [Principal...]"
  FrmMdi.Show
  FrmInicializacion.ShapeRectangulo.Width = iLlenado_barraprogresso * 1
  
  '-------------------------------------------------------------------------------------------------------
  
  'Descarga de pantalla de inicializacin
  Unload FrmInicializacion
  Screen.MousePointer = vbDefault
  
  
  'COMPROBACIN SI LA APLICACIN HA SIDO CARGADA A TRAVS DE UN FICHERO
  If ExisteFichero(LineaComando, False) Then
    ArchivoActual.Nombre = LineaComando
    'MsgBox "Fichero: " & ArchivoActual.Nombre, vbExclamation + vbOKOnly, "Atencin!"
    'Si el anlisis del fichero es correcto los datos cargados sern vlidos
    If FrmAnalisis.Analisis(True, ArchivoActual.Nombre) Then
      ArchivoActual.Cargado = True
      ArchivoActual.Modificado = False
      ArchivoActual.Nuevo = False
      FrmMdi.CargarSistema
    End If
  End If
  
  RegistrarIncidencias "Inicio", "Satisfactorio"
  GoTo FIN_PROCEDIMIENTO
  

ERROR_INICIALIZACION:
  Unload FrmInicializacion
  'Restaura la forma del cursor
  Screen.MousePointer = vbDefault
  'Se informa al usuario del error a la vez que este queda registrado
  MsgBox Error$ & " - " & Str$(Err), vbCritical, "Error al cargar la aplicacin"
  RegistrarIncidencias "Error General", Error$ & " - " & Str$(Err)
  'HA FINALIZADO LA APLICACIN DE FORMA FORZADA
  End
  
FIN_PROCEDIMIENTO:
  
End Sub


