Attribute VB_Name = "Utilities"
'-------------------------------------------------------------------------------------------------------
'                                  ## MDULO DE CDIGO: UTILITIES.BAS ##
'
' OBJETIVO: Contener aquellas funciones y procedimientos de utilidad general para el tratamiento
'           de informacin en el desarrollo de aplicaciones.
'-------------------------------------------------------------------------------------------------------

Option Explicit
'---------------------------------
' - DECLARACIN DE ENUMERACIONES -
'---------------------------------
'AMBITO GLOBAL
Public Enum ETIPOPOSICION
  Centrado = 0
  Aleatorio = 1
  Determinada = 2
End Enum

'***************************************************************************************************
'   FUNCiN: ObtenerLineaComando()
'
'       OBJETiVO:
'
'***************************************************************************************************
Function LineaComando()
   Dim STmp As String
   Dim I As Long
   
   'Comprobar si hay algn archivo en la lnea de comandos
   STmp = Trim$(Command$())
   If Len(STmp) Then
    'Si tiene los caracteres de comillas, quitarselos
    I = InStr(STmp, Chr$(34))
    If I Then
      STmp = Left$(STmp, I - 1) & MId$(STmp, I + 1)
      I = InStr(STmp, Chr$(34))
      If I Then
        STmp = Left$(STmp, I - 1) & MId$(STmp, I + 1)
      End If
    End If
  End If
  LineaComando = STmp
  
End Function


'***************************************************************************************************
'   FUNCIN: ExisteFichero()
'
'       OBJETIVO: Comprobar si existe dicho archivo en la ruta especificada.
'
'       P.ENTRADA: Cadena que contiene la ruta y nombre del fichero a
'                  comprobar si existe.
'
'       P.SALIDA: Valor booleano indicado la veracidad o falsedad de dicha ubicacin.
'
'***************************************************************************************************
Public Function ExisteFichero(ByVal RutaNombre As String, ByVal ReintentarSiNoExisteRuta As Boolean) As Boolean
    'En caso de error en tiempo de ejecucin permite ejecutar la siguiente instruccin
    On Error Resume Next
    'Declaracin de variables
    Dim Temp_dcha As Integer, Resultado As Integer
    Dim Nombre As String, Unidad As String
    Dim ENCONTRADO As Boolean
    ENCONTRADO = False
    'Si la cadena est vacia
    If RutaNombre = "" Then
        ExisteFichero = False
        Exit Function
    End If
    'Se separa la ruta del nombre del fichero
    Temp_dcha = Len(RutaNombre)
    Do While Not (ENCONTRADO)
        Temp_dcha = Temp_dcha - 1
        If MId$(RutaNombre, Temp_dcha, 1) = "\" Then ENCONTRADO = True
    Loop
    Nombre = Right$(RutaNombre, Len(RutaNombre) - Temp_dcha)
    Nombre = UCase$(Nombre)
    
Volver:
    'La funcin Dir es la que comprueba si existe realmente
    RutaNombre = Dir(RutaNombre)
    'En caso de que se especifique una unidad que no contenga soporte se producir el error nmero 52
    If Err.Number = 52 And ReintentarSiNoExisteRuta Then
      'Informa al usuario de la situacin para que proceda a correccin
      Unidad = UCase$(Left$(RutaNombre, 1))
      Resultado = MsgBox("No se puede tener acceso a " & Unidad & "\:" & vbCr & vbLf & "Compruebe que el dispositivo est disponible", _
                          vbRetryCancel + vbCritical, "Explorando")
      Err.Clear
      If Resultado = vbRetry Then
        GoTo Volver
      Else
        ExisteFichero = False
        Exit Function
      End If
    End If
    'Si la funcin Dir devuelve el nombre del fichero es por que este existe
    If UCase$(RutaNombre) = Nombre Then
        ExisteFichero = True
    Else
        ExisteFichero = False
    End If
    
End Function

'***************************************************************************************************
'   FUNCIN: ResumirFichero()
'
'       OBJETIVO: Simplificar la cadena que forma la ruta de un archivo que se anida en varios
'                 directorios de forma que se indique el directorio inicial y el nombre del
'                 fichero.
'
'       P.ENTRADA: Cadena que contiene la ruta completa.
'
'       P.SALIDA: Cadena simplificada de la ruta.
'
'***************************************************************************************************
Public Function ResumirFichero(ByVal NombreCompleto As String) As String
    'Declaracin de variables
    Dim Temp_primerizda As Long, Temp_izda As Long, Temp_dcha As Long
    Dim Cadena_izda As String, Cadena_dcha As String
    Dim I As Integer
    Dim ENCONTRADO   As Boolean
    
    'Si la cadena est vacia
    If NombreCompleto = "" Then Exit Function
    'Se busca la primera y ltima incidencia de "\" para eliminar as las que existan entre la primera y ltima "\"
    Temp_primerizda = InStr(NombreCompleto, "\")
    Temp_izda = InStr(Temp_primerizda + 1, NombreCompleto, "\")
    If Temp_izda = 0 Then Temp_izda = Temp_primerizda
    ENCONTRADO = False
    Temp_dcha = Len(NombreCompleto)
    Do While Not (ENCONTRADO)
        Temp_dcha = Temp_dcha - 1
        If MId$(NombreCompleto, Temp_dcha, 1) = "\" Then ENCONTRADO = True
    Loop
    If Temp_dcha <= Temp_izda Then
        ResumirFichero = NombreCompleto
        Exit Function
    End If
    'Por ltimo el fichero resumido se componen de la parte izquierda y derecha a la primera y ltima "\"
    Cadena_izda = Left$(NombreCompleto, Temp_izda)
    Cadena_dcha = Right$(NombreCompleto, Len(NombreCompleto) - Temp_dcha + 1)
    ResumirFichero = Cadena_izda & " ... " & Cadena_dcha

End Function

'***************************************************************************************************
'   PROCEDIMIENTO: SepararRutaNombre()
'
'       OBJETIVO: Separar de la cadena que contiene la ruta y nombre de fichero, estas dos partes
'                 de forma que se devuelva cada parte por separado en una cadena.
'
'       P.ENTRADA: Cadena que contiene la ruta y nombre fichero.
'
'       P.SALIDA: Dos cadenas de caracteres, una con la ruta y otra con el nombre de fichero.
'
'***************************************************************************************************
Public Sub SepararRutaNombre(ByVal RutaNombre As String, Ruta As String, Nombre As String, Optional Extension As String)
    'Declaracin de variables
    Dim LongRutaNombre As Long, LongNombre As Long, Busqueda As Long
    Dim ENCONTRADO As Boolean
    
    If RutaNombre = "" Then Exit Sub
    LongRutaNombre = Len(RutaNombre)
    'Se busca el elemento separador de la ruta y nombre "\"
    ENCONTRADO = False
    Busqueda = LongRutaNombre
    Do While Not (ENCONTRADO)
        If MId$(RutaNombre, Busqueda, 1) = "\" Then ENCONTRADO = True
        Busqueda = Busqueda - 1
    Loop
    Ruta = Left$(RutaNombre, Busqueda + 1)
    LongNombre = LongRutaNombre - (Busqueda + 1)
    Nombre = Right$(RutaNombre, LongNombre)
    Extension = Right$(Nombre, 3)

End Sub

'***************************************************************************************************
'   PROCEDIMIENTO: NombreArchivo()
'
'       OBJETIVO: Devolver nicamente de la cadena que contiene la ruta y nombre de fichero este
'                 ltimo pero sin la extensin simplemente el nombre.
'
'       P.ENTRADA: Cadena que contiene la ruta completa del fichero.
'
'       P.SALIDA: Cadena que contiene el nombre de fichero sin la extensin.
'
'***************************************************************************************************
Public Function NombreArchivo(ByVal RutaNombre As String) As String
  Dim Ruta As String
  Dim NombreConExtension As String
  Dim Busqueda As Long
  Dim ENCONTRADO As Boolean
  
  SepararRutaNombre RutaNombre, Ruta, NombreConExtension
  Busqueda = Len(NombreConExtension)
  'Se busca el elemento indicador de extensin "."
  ENCONTRADO = False
  Do While Not (ENCONTRADO)
    If MId$(NombreConExtension, Busqueda, 1) = "." Then ENCONTRADO = True
    Busqueda = Busqueda - 1
  Loop
  NombreArchivo = Left$(NombreConExtension, Busqueda)
  
End Function

'***************************************************************************************************
'   PROCEDIMIENTO: RutaNombreArchivoCopSeg()
'***************************************************************************************************
Public Function RutaNombreArchivoCopSeg(RutaNombreArchivo As String) As String
  On Error Resume Next
  'Si el directorio ya existiese al controlar el error se permite la ejecucin de la siguiente instruccin
  'permitiendo obtener la ruta del archivo de copia de seguridad
  MkDir App.Path & "\" & NOMBRE_DIRECTORIO_COPSEG & "\"
  RutaNombreArchivoCopSeg = App.Path & "\" & NOMBRE_DIRECTORIO_COPSEG & "\" & NombreArchivo(RutaNombreArchivo) & "." & EXTENSION_ARCHIVOS_COPIAS_SEGURIDAD
  Err.Clear 'por si se produjese el error se elimina su identificador de causa
End Function

'***************************************************************************************************
'   PROCEDIMIENTO: FrmPosicionar()
'
'       OBJETIVO: Posicionar con respecto a la pantalla la ventana que reciba en la posicin
'                 seleccionada. Su utilidad radica en que posiciona la ventana independientemente
'                 de la resolucin del monitor pues se trabaja con el objeto Screen que ya
'                 contiene dicha informacin.
'
'       P.ENTRADA: Ventana a posicionar y opcin de posicin a establecer.
'
'***************************************************************************************************
Public Sub FrmPosicionar(Frm As Form, Opcion As ETIPOPOSICION)
  Select Case Opcion
    'PARA MOSTRAR EL FORMULARIO DE CARGA CENTRADO EN LA PANTALLA
    Case Centrado:
      Frm.Left = (Screen.Width - Frm.Width) / 2
      Frm.Top = (Screen.Height - Frm.Height) / 2
    'PARA MOSTRAR EL FORMULARIO EN UNA POSICIN ALEATORIA DENTRO DE LA PANTALLA
    Case Aleatorio:
      Randomize
      Frm.Left = ((Screen.Width - Frm.Width) * Rnd)
      Frm.Top = ((Screen.Height - Frm.Height) * Rnd)
    End Select
    
End Sub

'***************************************************************************************************
'   PROCEDIMIENTO: FrmPosicionarConRespectoMDI()
'***************************************************************************************************
Public Sub FrmPosicionarConRespectoMDI(Frm As Form, Opcion As ETIPOPOSICION, Optional CoordX As Long, Optional CoordY As Long)
  Select Case Opcion
    'PARA MOSTRAR EL FORMULARIO DE CARGA CENTRADO EN LA PANTALLA
    Case Centrado:
      Frm.Left = (FrmMdi.Width - Frm.Width) / 2
      Frm.Top = (FrmMdi.Height - Frm.Height) / 2
    'PARA MOSTRAR EL FORMULARIO EN UNA POSICIN ALEATORIA DENTRO DE LA PANTALLA
    Case Aleatorio:
      Randomize
      Frm.Left = ((FrmMdi.Width - Frm.Width) * Rnd)
      Frm.Top = ((FrmMdi.Height - Frm.Height) * Rnd)
    Case Determinada:
      Frm.Left = CoordX
      Frm.Top = CoordY
    End Select
    
End Sub

'*********************************************************************************************************
'   PROCEDIMIENTO: VisualizarFrm()
'
'       OBJETIVO: Hacer visible un formulario ya cargado o no, sobre todo su objetivo es el de que ante
'                 formulario (secundarios) que estn minimizados u ocultos estos aparezcan de forma
'                 centrada y maximizados.
'
'       P.ENTRADA: Ventana a visualizar.
'
'*********************************************************************************************************
Public Sub VisualizarFrm(Frm As Form, Optional SinCentrar As Boolean)
  With Frm
    .WindowState = vbNormal
    .Show
    .SetFocus
  End With
  If Not SinCentrar Then
    FrmPosicionar Frm, Centrado
  End If
End Sub

'*********************************************************************************************************
'   PROCEDIMIENTO: EstaCargadoFrm()
'*********************************************************************************************************
Public Function EstaCargadoFrm(Frm As Form) As Boolean
  Dim I As Integer
  
  For I = 0 To Forms.Count - 1
    If Forms(I) Is Frm Then
      EstaCargadoFrm = True
      Exit Function
    End If
  Next I
  EstaCargadoFrm = False
  
End Function

'*********************************************************************************************************
'   PROCEDIMIENTO: Esperar()
'*********************************************************************************************************
Public Sub Esperar(CuantoSegundos As Integer)
  Dim dtLastRun As Date, dtNextRun As Date
  Dim nMinRemain As Integer
  
  dtLastRun = Now
  dtNextRun = DateAdd("s", CuantoSegundos, Now)
  
  nMinRemain = DateDiff("s", Now, dtNextRun)
  While nMinRemain > 0
    nMinRemain = DateDiff("s", Now, dtNextRun)
    DoEvents
  Wend
    
End Sub

'**************************************************************************************************
'   FUNCIN: Fecha()
'**************************************************************************************************
Public Function Fecha() As String
  Fecha = Format$(Now, " h:mm, dd-mmm-yy ")
End Function


'*********************************************************************************************************
'   PROCEDIMIENTO: PintadoTransparente()
'*********************************************************************************************************
Public Sub PintadoTransparente(objDest As Object, picSource As StdPicture, lngX As Long, lngY As Long, _
                               ByVal lngMaskColor As Long)
  'Declaracin de variables
  Dim lngSrcDC As Long      'Mapa de bits de origen
  Dim lngSaveDC As Long     'Copia del mapa de bits de origen
  Dim lngMaskDC As Long     'Mapa de bits de mscara monocroma
  Dim lngInvDC As Long      'Reverso monocromo del mapa de bits de la mscara
  Dim lngNewPicDC As Long   'Combinacin de bmps de origen y fondo
  
  'Dim bmpSource As BITMAP   'Descripcin del mapa de bits de origen
  
  Dim hResultBmp As Long    'Combinacin de origen y fondo
  Dim hSaveBmp As Long      'Copia del mapa de bits de origen
  Dim hMaskBmp As Long      'Mapa de bits de mscara monocroma
  Dim hInvBmp As Long       'Inversin monocroma del mapa de bits de la mscara
  
  Dim hSrcPrevBmp As Long   'Contiene mapa de bits prev en DC de origen
  Dim hSavePrevBmp As Long  'Contiene mapa de bits prev en DC guardada
  Dim hDestPrevBmp As Long  'Contiene mapa de bits prev en DC de destino
  Dim hMaskPrevBmp As Long  'Contiene mapa de bits prev en el DC de la mscara
  Dim hInvPrevBmp As Long   'Contiene mapa de bits prev en el DC invertido de la mscara
  
  Dim lngOrigScaleMode&     'Contiene el ScaleMode original
  Dim lngOrigColor&         'Contiene el color de fondo original del DC de origen
  
  'Establecer ScaleMode a pxeles para las API GDI de Windows
  lngOrigScaleMode = objDest.ScaleMode
  objDest.ScaleMode = vbPixels
  'Cargar el mapa de bits de origen para obtener su ancho (bmpSoruce.bmWidth) y alto (bmpSource.bmHeight)
  'GetObject picSource, Len(bmpSource), bmpSource
  'Crear contextos de dispositivo compatible (DC) para contener los mapas de bits utilizados por este sub
  
  
  '      SIN TERMINAR !!
                          
End Sub

'***************************************************************************************************
'   PROCEDIMIENTO: ObtenerLineaComando()
'***************************************************************************************************
'Public Sub ObtenerLineaComando(MatrixArgs() As String, Optional NMaxArgs As Integer)
'                                                      'Optional. Indica que un argumento no es necesario
'   'Declaracin de variables
'   Dim i, j, LongLinComando, ArgIn, NumArgs As Integer
'   Dim LineaComando As String
'   Const N_MAX_ARGS = 10
'
'   If IsMissing(NMaxArgs) Then NMaxArgs = N_MAX_ARGS
'   'Se comprueba si se ha establecido un valor mximo
'   'de argumentos en la llamada al procedimiento, y en
'   'funcin de este valor de dimensiona la matriz contenedora
'  'de argumentos
'   ReDim MatrixArgs(NMaxArgs)
'   NumArgs = 0: ArgIn = False
'   'Obtiene los argumentos de la lnea de comandos
'   LineaComando = Command()
'   LongLinComando = Len(LineaComando)
'   'Recorre la lnea de comando carcter a carcter a la vez
'   For i = 1 To LongLinComando
'      j = Mid(LineaComando, i, 1)
'      'Comprueba espacio o tabulacin
'     If (j <> " " And j <> vbTab) Then
'         'Ningn espacio o tabulacin y comprueba
'         'si est en el argumento
'         If Not ArgIn Then
'          'Empieza el nuevo argumento y comprueba
'          'para ms argumentos
'           If NumArgs = NMaxArgs Then Exit For
'           NumArgs = NumArgs + 1
'           ArgIn = True
'         End If
'         'Concatenar el carcter al argumento actual.
'         MatrixArgs(NumArgs) = MatrixArgs(NumArgs) & j
'      Else
'         'Se encontr un espacio o tabulador, entonces ArgIn a False
'         ArgIn = False
'      End If
'   Next i
'  'Redimensiona la matriz lo suficiente para contener los argumentos
'   ReDim Preserve MatrixArgs(NumArgs)
'   'Devuelve la matriz en nombre de la funcin
'  ObtenerLineaComando = MatrixArgs()
'
'End Sub

