Attribute VB_Name = "Fuzzy"
Option Explicit
                            
'-------------------------------------------------------------------------------------------------------
'                                 ## MDULO DE CDIGO: FUZZY.BAS ##
'
' OBJETIVO: Contener el conjunto de funciones y procedimientos necesarios para el
'           proceso de fuzzificacin o difuminacin sobre las variables de entrada.
'-------------------------------------------------------------------------------------------------------


'**************************************************************************************************
'   PROCEDIMIENTO: COMPROBAR_METODO_DIFUMINACION()
'**************************************************************************************************
Private Sub COMPROBAR_METODO_DIFUMINACION(PBDifusor() As Double, PBCDif() As Double, VCrisp As Double, _
                                          DifuminadoRapido As Boolean, Poss As Double)
  
  Dim PtoRef As Double
  Dim PBRehubicados(0 To 3) As Double
  
  'SE ESCALA PTOS DIFUSOR AL VALOR CONCRETO DE LA VARIABLE
  PtoRef = (PBDifusor(1) + PBDifusor(2)) / 2
  PBRehubicados(0) = PBDifusor(0) - PtoRef + VCrisp
  PBRehubicados(1) = PBDifusor(1) - PtoRef + VCrisp
  PBRehubicados(2) = PBDifusor(2) - PtoRef + VCrisp
  PBRehubicados(3) = PBDifusor(3) - PtoRef + VCrisp
  
  'A' >= D OR D' <= A IMPLICA Q EL DIFUSOR Y EL C.DIFUSO NO SE TOCAN
  If PBRehubicados(0) >= PBCDif(3) Or PBRehubicados(3) <= PBCDif(0) Then
    DifuminadoRapido = True
    Poss = 0#
  'B' <= C AND C' >= B IMPLICA QUE LOS SOPORTES DEL DIFUSOR Y C.DIFUSO SE TOCAN
  ElseIf PBRehubicados(1) <= PBCDif(2) And PBRehubicados(2) >= PBCDif(1) Then
    DifuminadoRapido = True
    Poss = 1#
  Else
    DifuminadoRapido = False
  End If
    
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: POSIBILIDAD()
'**************************************************************************************************
Public Function POSIBILIDAD(CoordY() As Double, Optional DevolverPosicion As Boolean = False, Optional Posicion As Integer) As Double
  
  Dim I As Integer, NCoordY As Integer
  Dim TempPoss As Double
  
  TempPoss = 0#
  NCoordY = UBound(CoordY(), 1) + 1
  For I = 0 To NCoordY - 1
    If CoordY(I) >= TempPoss Then
      TempPoss = CoordY(I)
      Posicion = I
    End If
  Next I
  POSIBILIDAD = TempPoss
  
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: DIFUMINAR()
'**************************************************************************************************
Public Function DIFUMINAR(TipoDifuminacion As ET_TNORMAS_SNORMAS, Vble As TEVARIABLE, _
                          IdVCrisp As Integer, IdConjDif As Integer, _
                          Optional ConInterfazGrafica As Boolean = False, Optional Grafico As PictureBox) As Double
                     
  Dim CoordXInt() As Double, CoordYInt() As Double
  Dim CoordXDif() As Double, CoordYDif() As Double
  Dim CoordXCDif() As Double, CoordYCDif() As Double
  Dim TempValorX As Double, TempValorY As Double
  Dim I As Integer
  
  SPLINES_INPUT Vble, IdVCrisp, IdConjDif, CoordXDif(), CoordYDif(), CoordXCDif(), CoordYCDif()
  
  Select Case TipoDifuminacion
  
    'T-NORMA DEL MNIMO
    Case TNorma_Minino:
        INTERSECCION_TNORMA_MINIMO CoordXDif(), CoordYDif(), CoordXCDif(), CoordYCDif(), CoordXInt(), CoordYInt()
    
      
    
  End Select
                                
  DIFUMINAR = POSIBILIDAD(CoordYInt())
                                                            
  If ConInterfazGrafica Then
    Grafico.Cls
    Dibujar_Spline CoordXCDif(), CoordYCDif(), &HFF&, 4, Grafico  'ROJO
    Dibujar_Spline CoordXDif(), CoordYDif(), &HFF8080, 4, Grafico 'AZUL
    Dibujar_PtoVCrisp CoordXDif(0), CoordXDif(UBound(CoordXDif(), 1)), Vble.VCrisp(IdVCrisp), &HFF8080, 4, Grafico   'AZUL

    TempValorY = POSIBILIDAD(CoordYInt(), True, I)
    TempValorX = CoordXInt(I)
    Dibujar_Spline CoordXInt(), CoordYInt(), &HFFFF&, 2, Grafico, True, TempValorX, TempValorY 'AMARILLO

  End If
                                                            
End Function

'**************************************************************************************************
'   PROCEDIMIENTO: DIFUMINAR_SIN_REFRESCO()
'**************************************************************************************************
Public Sub DIFUMINAR_SIN_REFRESCO(TipoDifuminacion As ET_TNORMAS_SNORMAS, IdE As Integer, Grafico As PictureBox)
                     
  Dim CoordXInt() As Double, CoordYInt() As Double
  Dim CoordXDif() As Double, CoordYDif() As Double
  Dim CoordXCDif() As Double, CoordYCDif() As Double
  Dim TempValorY As Double, TempValorX As Double
  Dim Posicion As Integer
  Dim Espesor As Integer
  Dim I As Integer
  
  Grafico.Cls
  DibujarGraficosVariableProceso E, IdE, Grafico
  For I = 0 To NConjDif(E, IdE) - 1
    
    SPLINES_INPUT SistActual.Entrada(IdE), 0, I, CoordXDif(), CoordYDif(), CoordXCDif(), CoordYCDif()
    If I = 0 Then
      Dibujar_Spline CoordXDif(), CoordYDif(), &HFF8080, 2, Grafico 'AZUL
      Dibujar_PtoVCrisp CoordXDif(0), CoordXDif(UBound(CoordXDif(), 1)), SistActual.Entrada(IdE).VCrisp(0), &HFF8080, 2, Grafico    'AZUL
    End If
    
    Select Case TipoDifuminacion
    
      'T-NORMA DEL MNIMO
      Case TNorma_Minino:
          INTERSECCION_TNORMA_MINIMO CoordXDif(), CoordYDif(), CoordXCDif(), CoordYCDif(), CoordXInt(), CoordYInt()
        
      
    End Select
    
    TempValorY = POSIBILIDAD(CoordYInt(), True, Posicion)
    TempValorX = CoordXInt(Posicion)
    
    Espesor = Grafico.DrawWidth
    Grafico.DrawWidth = 6
    Grafico.PSet (CoordXInt(Posicion), TempValorY), vbYellow
    Grafico.DrawWidth = Espesor
  Next I
                                                            
End Sub

'**************************************************************************************************
'   PROCEDIMIENTO: DIFUMINAR_ENTRADAS()
'**************************************************************************************************
Public Sub DIFUMINAR_ENTRADAS(ByVal IdVCrisp As Integer, _
                              Optional AllEntradas As Boolean = True, Optional IdE As Integer)
                              
  Dim I As Integer, j As Integer
  Dim NumCDif As Integer
  Dim DifuminadoRapido As Boolean
  Dim TempPoss As Double
    
  If AllEntradas Then 'SI SE REALIZA EL PROCESO DE DIFUSION SOBRE TODAS LAS ENTRADAS *
    For I = 0 To NEntradas - 1
      NumCDif = NConjDif(E, I)
      '* SLO SI LA ENTRADA ES ACTIVA Y EXISTE AL MENOS UN C.DIFUSO EN LA MISMA
      '  TENDR SENTIDO CALCULAR LA COMPARACIN DE SU DIFUSOR Y LOS C.DIFUSOS QUE LA FORMAN.
      If SistActual.Entrada(I).Activa = Si And NumCDif > 0 Then
        For j = 0 To NumCDif - 1
          '1) SE COMPRUEBA SI LA DISPOSICIN DEL DIFUSOR Y C.DIFUSO A COMPARAR ES TAL
          '    QUE O BIEN NO SE TOCAN (POSS = 0.0) O DIRECTAMENTE LOS SOPORTES COINCIDEN
          '    EN ALGN PUNTO (POSS = 1.0). SI SE PRODUCE ALGUNA DE ESTAS DOS SITUACIONES
          '    LA FUNCIN DEVOLVER SOBRE DifuminadoRapido EL VALOR VERDADERO DE FORMA QUE
          '    SE PODR ALMACENAR DIRECTAMENTE SOBRE EL CAMPO Poss.
          COMPROBAR_METODO_DIFUMINACION SistActual.Entrada(I).Difusor.PBasicos(), SistActual.Entrada(I).ConjDif(j).PBasicos(), _
                                        SistActual.Entrada(I).VCrisp(IdVCrisp), DifuminadoRapido, TempPoss
          If Not DifuminadoRapido Then
            'SEGN EL MTODO SELECCIONADO DE DIFUSIN (COMPOSICIN: MAX-TNORMA IMPLICA CONCEPTO POSIBILIDAD)
            'SE REALIZAR LA COMPARACIN ENTRE DIFUSOR ESCALADO Y C.DIFUSO O ETIQUETA ACTUAL (j) PARA OBTENER
            'AS LA MEDIDA DE POSIBILIDAD EN QUE EL DIFUSOR SEA EL C.DIFUSO E IVICEVERSA.
                SistActual.Entrada(I).ConjDif(j).Poss = DIFUMINAR(SistActual.Inferencia.OpDifuminacion.TipoNorma, _
                                                                    SistActual.Entrada(I), IdVCrisp, j)
          Else
            SistActual.Entrada(I).ConjDif(j).Poss = TempPoss
          End If
        Next j
      End If
    Next I
  
  Else
    'SI SLO SE QUIERE REALIZAR EL PROCESO DE DIFUMINACIN SOBRE UNA VARIABLE EN CONCRETO
    'ENTONCES NOS BASAMOS EN EL IdE.
    NumCDif = NConjDif(E, IdE)
    If SistActual.Entrada(IdE).Activa = Si And NumCDif > 0 Then
      For j = 0 To NumCDif - 1
        COMPROBAR_METODO_DIFUMINACION SistActual.Entrada(IdE).Difusor.PBasicos(), SistActual.Entrada(IdE).ConjDif(j).PBasicos(), _
                                      SistActual.Entrada(IdE).VCrisp(IdVCrisp), DifuminadoRapido, TempPoss
        If Not DifuminadoRapido Then
              SistActual.Entrada(IdE).ConjDif(j).Poss = DIFUMINAR(SistActual.Inferencia.OpDifuminacion.TipoNorma, _
                                                                  SistActual.Entrada(IdE), IdVCrisp, j)
          
        Else
          SistActual.Entrada(IdE).ConjDif(j).Poss = TempPoss
        End If
      Next j
    End If
  End If
  
End Sub



'**************************************************************************************************
'   PROCEDIMIENTO: DIFUMINAR_ETIQUETA()
'**************************************************************************************************
Public Sub DIFUMINAR_ETIQUETA(ByVal IdE As Integer, ByVal IdConjDif As Integer, ByVal IdVCrisp As Integer, _
                              Optional InfDif As Label, Optional InfCDif As Label, Optional InfPoss As Label, Optional Grafico As PictureBox, _
                              Optional ConInterfazGrafica As Boolean = True)
                                
  If Not ConInterfazGrafica Then
     If SistActual.Entrada(IdE).Activa = Si And NConjDif(E, IdE) > IdConjDif Then
      
      SistActual.Entrada(IdE).ConjDif(IdConjDif).Poss = DIFUMINAR(SistActual.Inferencia.OpDifuminacion.TipoNorma, _
                                                                  SistActual.Entrada(IdE), IdVCrisp, IdConjDif, False)
    End If
  Else
    If SistActual.Entrada(IdE).Activa = Si And NConjDif(E, IdE) > IdConjDif Then
      
      SistActual.Entrada(IdE).ConjDif(IdConjDif).Poss = DIFUMINAR(SistActual.Inferencia.OpDifuminacion.TipoNorma, _
                                                                  SistActual.Entrada(IdE), IdVCrisp, IdConjDif, True, Grafico)
      
      InfDif.Caption = "DIFUMINADOR EN: " & Round(SistActual.Entrada(IdE).VCrisp(IdVCrisp), NUMDEC) & " [" & Trim$(SistActual.Entrada(IdE).UndSigla) & "]"
      InfCDif.Caption = "<< ETIQ. " & Trim$(SistActual.Entrada(IdE).ConjDif(IdConjDif).Etiqueta) & " >>"
      InfPoss.Caption = "OP. COMPARACIN: " & QNorma(SistActual.Inferencia.OpDifuminacion.TipoNorma) & _
                        " (POSS=" & Round(SistActual.Entrada(IdE).ConjDif(IdConjDif).Poss, NUMDEC) & ")"
      
    End If
  End If
  
End Sub


